package com.doext.module.activity.wxapi;

import android.app.Activity;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.pm.ResolveInfo;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.util.Log;
import android.widget.Toast;

import com.tencent.mm.opensdk.constants.ConstantsAPI;
import com.tencent.mm.opensdk.modelbase.BaseReq;
import com.tencent.mm.opensdk.modelbase.BaseResp;
import com.tencent.mm.opensdk.modelmsg.SendAuth;
import com.tencent.mm.opensdk.modelmsg.SendMessageToWX;
import com.tencent.mm.opensdk.modelmsg.WXFileObject;
import com.tencent.mm.opensdk.modelmsg.WXImageObject;
import com.tencent.mm.opensdk.modelmsg.WXMediaMessage;
import com.tencent.mm.opensdk.modelmsg.WXMiniProgramObject;
import com.tencent.mm.opensdk.modelmsg.WXMusicObject;
import com.tencent.mm.opensdk.modelmsg.WXWebpageObject;
import com.tencent.mm.opensdk.openapi.IWXAPI;
import com.tencent.mm.opensdk.openapi.IWXAPIEventHandler;
import com.tencent.mm.opensdk.openapi.WXAPIFactory;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.net.URL;
import java.util.List;

import core.DoServiceContainer;
import doext.module.do_TencentWX.app.do_TencentWX_App;
import doext.module.do_TencentWX.implement.do_TencentWX_Model;


public class WXEntryActivity extends Activity implements IWXAPIEventHandler {
    private IWXAPI api;
    // 图文分享标识
    public static final int SHARE_IMG_TEXT = 0;
    // 纯图分享标识
    public static final int SHARE_IMG = 1;
    // 音乐分享标识
    public static final int SHARE_MUSIC = 2;
    // 文件分享标识
    public static final int SHARE_FILE = 3;
    // 小程序分享标识
    public static final int SHARE_MINI_PROGRAM = 4;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        String appId = getIntent().getStringExtra("appId");
        api = WXAPIFactory.createWXAPI(this, appId);
        api.registerApp(appId);
        api.handleIntent(getIntent(), this);

        if (do_TencentWX_Model.OPERAT_FLAG.equals(do_TencentWX_Model.LOGIN_FLAG)) {
            login();
        } else if (appId != null && do_TencentWX_Model.OPERAT_FLAG.equals(do_TencentWX_Model.SHARE_FLAG)) {
            share();
        }

    }

    private void login() {
        boolean isFlag = getIntent().getBooleanExtra("isFlag", true);
        if (!isFlag) {
            SendAuth.Req req = new SendAuth.Req();
            req.scope = "snsapi_userinfo";
            req.state = "deviceone";
            api.sendReq(req);
            this.finish();
        }
    }

    private void share() {
        // 分享场景 0 分享到微信好友，1 分享都微信朋友圈
        int _scene = getIntent().getIntExtra("scene", 0);
        // 分享类型 0 图文分享，1 纯图分享 2 音乐分享
        int _type = getIntent().getIntExtra("type", 0);
        // 分享标题
        String _title = getIntent().getStringExtra("title");
        // 分享文本内容
        String _content = getIntent().getStringExtra("content");
        // 分享后点击文本后打开的地址
        String _url = getIntent().getStringExtra("url");
        // 分享图片路径
        String _image = getIntent().getStringExtra("image");
        // 分享音乐地址
        String _audio = getIntent().getStringExtra("audio");
        // 分享缩略图地址
        String _trumb = getIntent().getStringExtra("trumb");
        // 分享文件地址
        String _filePath = getIntent().getStringExtra("filePath");
        switch (_type) {
            case SHARE_IMG_TEXT:
                if (!checkIsNull("图文分享类型，文本内容", _content)) {
                    sendImgText(_url, _title, _content, _image, _scene);
                }
                break;
            case SHARE_IMG:
                if (!checkIsNull("纯图分享类型，图片路径", _content)) {
                    if (_trumb.equals("")) {//为空时则自动取缩放后的image参数图片
                        sendImg(_image, _scene);

                    } else {
                        sendImg(_trumb, _scene);
                    }
                }
                break;
            case SHARE_MUSIC:
                sendMusic(_audio, _scene, _title, _content, _image);
                break;
            case SHARE_FILE:
                if (_trumb.equals("")) {//为空时则自动取缩放后的image参数图片
                    sendFile(_filePath, _scene, _image, _content);
                } else {
                    sendFile(_filePath, _scene, _trumb, _content);
                }
                break;
            case SHARE_MINI_PROGRAM:

                break;
            default:
                Toast.makeText(this, "微信分享类型不合法，错误类型号：" + _type, Toast.LENGTH_SHORT).show();
                finish();
                return;
        }

    }

    @Override
    public void onReq(BaseReq req) {
    }

    @Override
    public void onResp(BaseResp resp) {
        try {
            do_TencentWX_Model _model = (do_TencentWX_Model) DoServiceContainer.getSingletonModuleFactory().getSingletonModuleByID(null, do_TencentWX_App.getInstance().getModuleTypeID());
            _model.callBack(resp);
            this.finish();
            //判断如果是从小程序退出的话 则重新调起本应用 否则会停留在微信界面上
            if (resp.getType() == ConstantsAPI.COMMAND_LAUNCH_WX_MINIPROGRAM) {
                wakeUpApp(this);
            }
        } catch (Exception e) {
            DoServiceContainer.getLogEngine().writeError("do_TencentWX_Model onResp \n\t", e);
        }

    }

    private void sendMiniProgram(String _webpageUrl, String _userName, String _path,
                                 boolean _withShareTicket, int _miniprogramType,
                                 Bitmap thumb, String title, String desc) {
        WXMiniProgramObject miniProgramObj = new WXMiniProgramObject();
        miniProgramObj.webpageUrl = _webpageUrl; // 兼容低版本的网页链接
        miniProgramObj.miniprogramType = _miniprogramType;// 正式版:0，测试版:1，体验版:2
        miniProgramObj.userName = _userName;     // 小程序原始id
        miniProgramObj.path = _path;            //小程序页面路径；对于小游戏，可以只传入 query 部分，来实现传参效果，如：传入 "?foo=bar"
        miniProgramObj.withShareTicket = _withShareTicket;
        WXMediaMessage msg = new WXMediaMessage(miniProgramObj);
        msg.title = title;                    // 小程序消息title
        msg.description = desc;               // 小程序消息desc
        msg.thumbData = getBitmapBytes(thumb, 32);                      // 小程序消息封面图片，小于128k

        SendMessageToWX.Req req = new SendMessageToWX.Req();
        req.transaction = buildTransaction("miniProgram");
        req.message = msg;
        req.scene = SendMessageToWX.Req.WXSceneSession;  // 目前只支持会话
        api.sendReq(req);
    }

    private void wakeUpApp(Context context) {
        Intent resolveIntent = new Intent(Intent.ACTION_MAIN, null);
        resolveIntent.addCategory(Intent.CATEGORY_LAUNCHER);
        resolveIntent.setPackage(context.getPackageName());
        List<ResolveInfo> apps = context.getPackageManager().queryIntentActivities(resolveIntent, 0);
        if (apps.size() != 0) {
            Intent intent = new Intent(Intent.ACTION_MAIN);
            ResolveInfo ri = apps.iterator().next();
            String packageName = ri.activityInfo.packageName;
            String className = ri.activityInfo.name;
            ComponentName cn = new ComponentName(packageName, className);
            intent.addCategory(Intent.CATEGORY_LAUNCHER);
            intent.setComponent(cn);
            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            intent.addFlags(Intent.FLAG_ACTIVITY_RESET_TASK_IF_NEEDED);
            context.startActivity(intent);
        }
    }

    private boolean checkIsNull(String name, String data) {
        if (data == null) {
            finish();
            Toast.makeText(this, name + "不能为空", Toast.LENGTH_SHORT).show();
            return true;
        } else {
            return false;
        }

    }

    public void sendImg(final String image, final int scene) {
        final Handler handler = new Handler() {
            public void handleMessage(android.os.Message msg) {
                Bitmap bmp = (Bitmap) msg.obj;
                sendImg(image, bmp, scene);
            }
        };
        // 如果是网络图片
        if (image.contains("http://") || image.contains("https://")) {
            // 开启子线程加载图片
            new Thread() {
                public void run() {
                    try {
                        Bitmap thumb = BitmapFactory.decodeStream(new URL(image).openStream());
                        handler.handleMessage(handler.obtainMessage(0, thumb));
                    } catch (Exception e) {
                        DoServiceContainer.getLogEngine().writeError("do_TencentWX_Model sendImgText \n\t", e);
                    }
                }

                ;
            }.start();
            // 如果是本地图片
        } else {
            Bitmap bmp = getLocalImage(image);
            sendImg(image, bmp, scene);
        }
    }

    private void sendImg(String image, Bitmap thumbBmp, int scene) {
        WXImageObject imgObj = new WXImageObject(thumbBmp);

        //设置图片
        WXMediaMessage msg = new WXMediaMessage();
        msg.mediaObject = imgObj;
        //设置缩略图
        if (thumbBmp != null) {
            msg.thumbData = getBitmapBytes(thumbBmp, 32);
        }
        SendMessageToWX.Req req = new SendMessageToWX.Req();
        req.transaction = buildTransaction("img");
        req.message = msg;
        req.scene = scene;
        api.sendReq(req);
        finish();
    }

    public void sendFile(final String filePath, final int scene, final String image, final String content) {
        final Handler handler = new Handler() {
            public void handleMessage(android.os.Message msg) {
                Bitmap bmp = (Bitmap) msg.obj;
                toFile(filePath, scene, bmp, content);
            }
        };
        // 如果是网络图片
        if (image.contains("http://") || image.contains("https://")) {
            // 开启子线程加载图片
            new Thread() {
                public void run() {
                    try {
                        Bitmap thumb = BitmapFactory.decodeStream(new URL(image).openStream());
                        handler.handleMessage(handler.obtainMessage(0, thumb));
                    } catch (Exception e) {
                        DoServiceContainer.getLogEngine().writeError("do_TencentWX_Model sendImgText \n\t", e);
                    }
                }

                ;
            }.start();
            // 如果是本地图片
        } else {
            Bitmap bmp = getLocalImage(image);
            toFile(filePath, scene, bmp, content);
        }
    }


    private void toFile(final String filePath, final int scene, final Bitmap thumbBmp, final String text) {
        WXFileObject fileObj = new WXFileObject();
//        fileObj.fileData = inputStreamToByte(filePath);//文件路径

//        File shareFile = new File(filePath);
//        if(!shareFile.exists()){
//            return;
//        }

        String extName = getExtensionName(filePath);
        String sharePath = Environment.getExternalStorageDirectory().getAbsolutePath();
        File shareFile = new File(sharePath + "/share" +extName);
        copyFile(filePath,shareFile.getAbsolutePath());
        fileObj.setFilePath(shareFile.getAbsolutePath());
        Log.d("fxh", filePath);
        //设置图片
        WXMediaMessage msg = new WXMediaMessage();
        msg.mediaObject = fileObj;
        msg.title = filePath.substring(filePath.lastIndexOf("/") + 1);
        msg.description = text;
        msg.messageExt = extName;
        //设置缩略图
        if (thumbBmp != null) {
            msg.thumbData = getBitmapBytes(thumbBmp, 32);
        }
        SendMessageToWX.Req req = new SendMessageToWX.Req();
        req.transaction = buildTransaction("file");
        req.message = msg;
        req.scene = scene;
        api.sendReq(req);
//        shareFile.delete();
        finish();
    }


    public void sendImgText(final String url, final String title, final String text, final String image, final int scene) {
        final Handler handler = new Handler() {
            public void handleMessage(android.os.Message msg) {
                Bitmap bmp = (Bitmap) msg.obj;
                toTextImg(url, title, text, bmp, scene);
            }
        };
        // 如果是网络图片
        if (image.contains("http://") || image.contains("https://")) {
            // 开启子线程加载图片
            new Thread() {
                public void run() {
                    try {
                        Bitmap thumb = BitmapFactory.decodeStream(new URL(image).openStream());
                        handler.handleMessage(handler.obtainMessage(0, thumb));
                    } catch (Exception e) {
                        DoServiceContainer.getLogEngine().writeError("do_TencentWX_Model sendImgText \n\t", e);
                    }
                }

                ;
            }.start();
            // 如果是本地图片
        } else {
            Bitmap bmp = getLocalImage(image);
            toTextImg(url, title, text, bmp, scene);
        }
    }

    public static String getExtensionName(String filename) {
        if ((filename != null) && (filename.length() > 0)) {
            int dot = filename.lastIndexOf('.');
            if ((dot > -1) && (dot < (filename.length() - 1))) {
                return filename.substring(dot + 1);
            }
        }
        return filename;
    }

    private void toTextImg(String url, String title, String text, Bitmap bmp, int scene) {
        WXWebpageObject localWXWebpageObject = new WXWebpageObject();
        localWXWebpageObject.webpageUrl = url;
        WXMediaMessage localWXMediaMessage = new WXMediaMessage(localWXWebpageObject);
        localWXMediaMessage.title = title;
        localWXMediaMessage.description = text;
        if (bmp != null) {
            localWXMediaMessage.thumbData = getBitmapBytes(bmp, 32);
        }
        SendMessageToWX.Req localReq = new SendMessageToWX.Req();
        localReq.transaction = buildTransaction("ImgText");
        localReq.message = localWXMediaMessage;
        localReq.scene = scene;
        api.sendReq(localReq);
        finish();
    }

    public void sendMusic(final String audio, final int scene, final String title, final String text, final String image) {
        final Handler handler = new Handler() {
            public void handleMessage(android.os.Message msgs) {
                Bitmap bmp = (Bitmap) msgs.obj;
                toMusicImg(audio, title, text, bmp, scene);
            }

            ;
        };

        // 如果是网络图片
        if (image.contains("http://") || image.contains("https://")) {
            // 开启子线程加载图片
            new Thread() {
                public void run() {
                    try {
                        Bitmap thumb = BitmapFactory.decodeStream(new URL(image).openStream());
                        handler.handleMessage(handler.obtainMessage(0, thumb));
                    } catch (Exception e) {
                        DoServiceContainer.getLogEngine().writeError("do_TencentWX_Model sendMusic \n\t", e);
                    }
                }

                ;
            }.start();

        } else {
            Bitmap bmp = getLocalImage(image);
            toMusicImg(audio, title, text, bmp, scene);
        }
    }

    private void toMusicImg(String audio, String title, String text, Bitmap bmp, int scene) {

        WXMusicObject music = new WXMusicObject();
        music.musicUrl = audio;

        WXMediaMessage msg = new WXMediaMessage();
        msg.mediaObject = music;
        msg.title = title;
        msg.description = text;
        if (bmp != null) {
            msg.thumbData = getBitmapBytes(bmp, 32);
        }
        SendMessageToWX.Req req = new SendMessageToWX.Req();
        req.transaction = buildTransaction("music");
        req.message = msg;
        req.scene = scene;
        api.sendReq(req);
        finish();

    }

    private String buildTransaction(final String type) {
        return (type == null) ? String.valueOf(System.currentTimeMillis()) : type + System.currentTimeMillis();
    }

    private Bitmap getLocalImage(String srcPath) {
        if (srcPath == null || !new File(srcPath).exists()) {
            return null;
        }
        BitmapFactory.Options newOpts = new BitmapFactory.Options();
        // 开始读入图片，此时把options.inJustDecodeBounds 设回true了
        newOpts.inJustDecodeBounds = true;
        Bitmap bitmap = BitmapFactory.decodeFile(srcPath, newOpts);// 此时返回bm为空

        newOpts.inJustDecodeBounds = false;
        int w = newOpts.outWidth;
        int h = newOpts.outHeight;
        // 现在主流手机比较多是800*480分辨率，所以高和宽我们设置为
        float hh = 800f;// 这里设置高度为800f
        float ww = 480f;// 这里设置宽度为480f
        // 缩放比。由于是固定比例缩放，只用高或者宽其中一个数据进行计算即可
        int be = 1;
        if (w > h && w > ww) {// 如果宽度大的话根据宽度固定大小缩放
            be = (int) (newOpts.outWidth / ww);
        } else if (w < h && h > hh) {// 如果高度高的话根据宽度固定大小缩放
            be = (int) (newOpts.outHeight / hh);
        }
        if (be <= 0)
            be = 1;
        newOpts.inSampleSize = be;// 设置缩放比例
        // 重新读入图片，注意此时已经把options.inJustDecodeBounds 设回false了
        bitmap = BitmapFactory.decodeFile(srcPath, newOpts);
        return compressImage(bitmap);// 压缩好比例大小后再进行质量压缩
    }

    // 压缩图片，使图片不大于100kb
    private Bitmap compressImage(Bitmap image) {

        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        image.compress(Bitmap.CompressFormat.JPEG, 100, baos);// 质量压缩方法，这里100表示不压缩，把压缩后的数据存放到baos中
        int options = 100;
        while (baos.toByteArray().length / 1024 > 100) { // 循环判断如果压缩后图片是否大于100kb,大于继续压缩
            baos.reset();// 重置baos即清空baos
            image.compress(Bitmap.CompressFormat.JPEG, options, baos);// 这里压缩options%，把压缩后的数据存放到baos中
            options -= 10;// 每次都减少10
        }
        ByteArrayInputStream isBm = new ByteArrayInputStream(baos.toByteArray());// 把压缩后的数据baos存放到ByteArrayInputStream中
        Bitmap bitmap = BitmapFactory.decodeStream(isBm, null, null);// 把ByteArrayInputStream数据生成图片
        return bitmap;
    }

    public static byte[] getBitmapBytes(Bitmap bitmap, int maxkb) {
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        bitmap.compress(Bitmap.CompressFormat.PNG, 100, output);
        int options = 100;
        while (output.toByteArray().length > maxkb && options != 10) {
            output.reset(); //清空output
            bitmap.compress(Bitmap.CompressFormat.JPEG, options, output);//这里压缩options%，把压缩后的数据存放到output中
            options -= 10;
        }
        return output.toByteArray();
    }

    public static byte[] inputStreamToByte(String path) {
        try {
            FileInputStream fis = new FileInputStream(path);
            ByteArrayOutputStream bytestream = new ByteArrayOutputStream();
            int ch;
            while ((ch = fis.read()) != -1) {
                bytestream.write(ch);
            }
            byte imgdata[] = bytestream.toByteArray();
            bytestream.close();
            return imgdata;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public boolean copyFile(String oldPath$Name, String newPath$Name) {
        try {
            File oldFile = new File(oldPath$Name);
            if (!oldFile.exists() || !oldFile.isFile() || !oldFile.canRead()) {
                return false;
            }
            File newFile = new File(newPath$Name);
            if(!newFile.getParentFile().exists()){
                newFile.getParentFile().mkdir();
            }
            if(newFile.exists()){
                newFile.delete();
            }
            newFile.createNewFile();
            FileInputStream fileInputStream = new FileInputStream(oldPath$Name);
            FileOutputStream fileOutputStream = new FileOutputStream(newPath$Name);
            byte[] buffer = new byte[1024];
            int byteRead;
            while (-1 != (byteRead = fileInputStream.read(buffer))) {
                fileOutputStream.write(buffer, 0, byteRead);
            }
            fileInputStream.close();
            fileOutputStream.flush();
            fileOutputStream.close();
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }
}