/**
 * 生日
 * 
 * @Author : Cydow
 * @Timestamp : 2017-11-28
 */
var deviceone = require("deviceone");
var do_global = deviceone.sm("do_Global");

var objectId = require("objectId");       //生成objectId
var tools = require("tools");             //工具
var constants = require("constants");     //常量
var dbcycle = require("dbcycle");         //周期
var widgetTools = require("widgetTools.ui"); // 小部件工具
var notificationTools = require("notificationTools"); //设置提醒
var calendar = require("calendar"); // 日期

// 数据库
var localdb = deviceone.mm("M0026_sqlite", "localdb", "app");

//信息类型
var infoCategoryBirthday = constants.getConstants("infoCategory_birthday");

//周期类型（每年：8）
var cycleId = 8;
//提前提醒时长（默认：正点提醒）
var remindTime = 0;

/**
 * 通过id从mytask表中获取指定的一条数据
 * 
 * @param _id
 * @returns
 */
function get_mytaskOneData(_id) {
	var sql = "select * from mytask where id=" + tools.formatSqlValue(_id)
			+ ";";
	var _data = localdb.querySync(sql);
	if (_data == null || _data.length <= 0) {
		return null;
	}
	return _data[0];
}

/**
 * 获取插入一条生日数据的sql语句
 * @param _changedData
 * @returns
 */
function getInsertBirthdaySql(_changedData){
	var _nowTime = new Date();
	if (_changedData.type == null) {
		_changedData.type = "0";
	}
	if (_changedData.update_time == null) {
		_changedData.update_time = _nowTime.getTime();
	}
	if (_changedData.create_time == null) {
		_changedData.create_time = _nowTime.getTime();
	}
	if (_changedData.sync_count == null) {
		_changedData.sync_count = 1;
	}
	if (_changedData.source == null) {
		_changedData.source = constants.getConstants("dataSource_create");
	}
	if (_changedData.deleted == null) {
		_changedData.deleted = 0;
	}
	if (_changedData.cycleid == null) {
		_changedData.cycleid = cycleId;
	}
	if (_changedData.cyclerecordid == null) {
		_changedData.cyclerecordid = objectId.getObjectId();
	}
	_changedData.sync_count = tools.getSyncSequence();
	_changedData.notifyid = tools.getNotifyId();
	var _sql_insert = "insert into mytask(id, infoid, commname, type, start, commtime, calendartype, remindtime, cycleid, cyclerecordid, infocategory, create_time, update_time, sync_count, notifyid, source, deleted) values("
			+ tools.formatSqlValue(_changedData.id) + ","
			+ tools.formatSqlValue(_changedData.infoid) + ","
			+ tools.formatSqlValue(_changedData.commname) + ","
			+ tools.formatSqlValue(_changedData.type) + ","
			+ _changedData.start + ","   // 开始时间（生日的显示时间），用于start字段升序查询数据
			+ _changedData.commtime + "," //生日
			+ tools.formatSqlValue(_changedData.calendartype) + "," // 生日历法类型
			+ _changedData.remindtime + ","
			+ _changedData.cycleid + ","
			+ tools.formatSqlValue(_changedData.cyclerecordid) + ","
			+ infoCategoryBirthday + ","
			+ _changedData.create_time + ","
			+ _changedData.update_time + ","
			+ _changedData.sync_count + ","
			+ _changedData.notifyid + ","
			+ tools.formatSqlValue(_changedData.source) + ","
			+ _changedData.deleted + ");";
	return _sql_insert;
}

/**
 * 插入一条生日卡片数据
 * @param _changedData
 * @param _sqlList
 * @returns
 */
function insertBirthdayOneData(_changedData, _sqlList) {
	var _sql = getInsertBirthdaySql(_changedData);
	_sqlList.push(_sql);
	//设置本地提醒
	notificationTools.setBirthdayNotification(_changedData, true, "add");
}

/**
 * 获取插入一条生日数据的所有sql语句
 * @param _changedData
 * @param _sqlList
 * @returns
 */
function getInsertBirthdayAllSqls(_changedData, _sqlList){
	var _nowTime = new Date();
	var _nowYear = _nowTime.getFullYear();
	var _commTime = new Date();
	_commTime.setTime(_changedData.commtime);
	var _commYear = _commTime.getFullYear();
	var _commStart = _commTime;
	if (_nowYear > _commYear) {
		//如果纪念年份小于今年，不管日期如何，均显示今年的卡片
		_commStart = _nowTime;
	}
	// 若历法类型值为空，则默认取值为公历
	if(null == _changedData.calendartype || 
			"" == _changedData.calendartype){
		_changedData.calendartype = constants.getConstants("calendartype_greg");
	}
	// 获取生日卡片显示时间
	_commStart = calendar.getCycleTimeOfComm({
		commtime: _commTime, 
		updtime: _commStart, 
		calendartype: _changedData.calendartype, 
		isnow: true
	});
	
	var _id = objectId.getObjectId();
	var _cycleRecordId = _changedData.id;
	var _cycleData = {
		id : _id,
		infoid: _cycleRecordId,
		commname : _changedData.commname,
		start : _commStart.getTime(),
		commtime : _changedData.commtime,
		calendartype: _changedData.calendartype,
		remindtime: remindTime,
		cycleid : cycleId,
		cyclerecordid : _cycleRecordId
	};
	var _sql = getInsertBirthdaySql(_cycleData);
	_sqlList.push(_sql);
	
	// 在周期性记录表增加一条记录
	var _cycleRecordData = {
		id : _cycleRecordId,
		taskid : _id,
		starttime : _cycleData.start,
		updtime : _cycleData.start,
		cycleid : cycleId,
		count : 0,
		infocategory : infoCategoryBirthday
	};
	_sql = dbcycle.getInsertCycleRecordSql(_cycleRecordData);
	_sqlList.push(_sql);
	
	//设置本地提醒
	notificationTools.setBirthdayNotification(_cycleData, true, "add");
}

// 获取插入一条生日数据的所有sql语句
module.exports.getInsertBirthdayAllSqls = function(_changedData, _sqlList) {
	getInsertBirthdayAllSqls(_changedData, _sqlList);
};

/**
 * 初始化并插入一条生日数据
 * 
 * @param _changedData
 */
function initAndInsertBirthdayData(_changedData) {
	var _sqlList = [];
	getInsertBirthdayAllSqls(_changedData, _sqlList);
	// 插入一条纪念日数据
	return localdb.executeSync1({sqls: _sqlList});
}

//插入一条生日数据
module.exports.insertBirthdayData = function(_changedData) {
	initAndInsertBirthdayData(_changedData);
	
	//时间流数据重新加载
	do_global.fire("taskDataChangeded");
};

/**
 * 获取增量生成新生日卡片的sql语句
 * @param _changedData
 * @param _cycleRecordData
 * @param _sqlList
 * @returns
 */
var commLengthTime = 30 * 24 * 3600 * 1000;
function getInsertBirthdayBySourceSql(_changedData, _cycleRecordData, _sqlList){
	var _sql = "";
	var _taskid = _cycleRecordData.taskid;
	var _count = parseInt(_cycleRecordData.count);
	var isChanged = false;
	var _nowTime = new Date();
	var _nowTimeTemp = _nowTime.getTime();
	var _commTime = new Date();
	_commTime.setTime(parseInt(_changedData.commtime));
	var _updTime = new Date();
	_updTime.setTime(parseInt(_cycleRecordData.updtime));
	// 获取下个周期（重复）显示时间
	_updTime = calendar.getCycleTimeOfComm({
		commtime: _commTime,
		updtime: _updTime,
		calendartype: _changedData.calendartype,
		isnow: false
	});
	var _updTimeTemp = _updTime.getTime() - commLengthTime;
	while (_nowTimeTemp >= _updTimeTemp) {
		_changedData.id = tools.createTaskId(_taskid, ++_count);
		_changedData.start = _updTime.getTime();

		// 插入一条纪念日数据
		insertBirthdayOneData(_changedData, _sqlList);

		// 获取下个周期（重复）显示时间
		_updTime = calendar.getCycleTimeOfComm({
			commtime: _commTime,
			updtime: _updTime,
			calendartype: _changedData.calendartype,
			isnow: false
		});
		_updTimeTemp = _updTime.getTime() - commLengthTime;
		isChanged = true;
	}

	if (isChanged) {
		var _cycleRecord = {
			id : _cycleRecordData.id,
			updtime : _changedData.start,
			count: _count
		};
		// 获取更新周期记录的sql语句
		_sql = dbcycle.getUpdCycleRecordSql(_cycleRecord);
		_sqlList.push(_sql);
	}
}

// 增量插入生日卡片数据（从周期记录表中获取生日卡片源数据id，若满足生成规则，则生成生日卡片）
function insertBirthdayBySource(_changedData, _cycleRecordData) {
	var _sqlList = [];
	getInsertBirthdayBySourceSql(_changedData, _cycleRecordData, _sqlList);
	localdb.executeSync1({sqls: _sqlList});
};

// 增量插入生日卡片数据（从周期记录表中获取生日卡片源数据id，若满足生成规则，则生成生日卡片）
module.exports.insertBirthdayBySource = function(_changedData, _cycleRecordData) {
	insertBirthdayBySource(_changedData, _cycleRecordData);
	// 刷新视图（桌面小部件）
	widgetTools.refresh();
};

/**
 * 获取更新生日卡片数据的sql语句
 * @param _changedData
 * @returns
 */
function getUpdBirthdayDataSql(_changedData){
	var _nowTime = new Date();
	_changedData.sync_count = tools.getSyncSequence();
	_changedData.update_time = _nowTime.getTime();
	var _setList = "";
	if (_changedData.commname != null) {
		if (_setList.length > 0)
			_setList += ",";
		_setList += "commname=" + tools.formatSqlValue(_changedData.commname);
	}
	if (_changedData.commtime != null) {
		if (_setList.length > 0)
			_setList += ",";
		_setList += "commtime=" + _changedData.commtime;
	}
	if (_changedData.calendartype != null) {
		if (_setList.length > 0)
			_setList += ",";
		_setList += "calendartype=" + tools.formatSqlValue(_changedData.calendartype);
	}
	if (_changedData.start != null) {
		if (_setList.length > 0)
			_setList += ",";
		_setList += "start=" + _changedData.start;
	}
	if (_changedData.update_time != null) {
		if (_setList.length > 0)
			_setList += ",";
		_setList += "update_time=" + _changedData.update_time;
	}
	if (_changedData.sync_count != null) {
		if (_setList.length > 0)
			_setList += ",";
		_setList += "sync_count=" + _changedData.sync_count;
	}
	if (_setList.length <= 0)
		return;
	var _sql_update = "update mytask set " + _setList + " where id="
			+ tools.formatSqlValue(_changedData.id) + ";";
	return _sql_update;
}

/**
 * 更新一条生日卡片数据（通过生日卡片id）
 * @param _changedData
 * @param _sqlList
 * @returns
 */
function updBirthdayOneData(_changedData, _sqlList) {
	var _sql = getUpdBirthdayDataSql(_changedData);
	_sqlList.push(_sql);
	//设置本地提醒
	notificationTools.setBirthdayNotification(_changedData, true, "update");
}

/**
 * 获取更新生日卡片的所有sql语句
 * @param _changedData
 * @param _sqlList
 * @returns
 */
function getUpdBirthdayAllSqls(_changedData, _sqlList){
	var _sql = "";
	//获取周期表里的数据
	var _cycleRecord = dbcycle.query_cycleRecordOneData(_changedData.id);
	// 获取更新前的数据
	var _oldData = get_mytaskOneData(_cycleRecord.taskid);
	var isSuccess = false;
	// 判断生日日期是否改变
	var _isChangedCommTime = true;
	if (_oldData.commtime == _changedData.commtime &&
			_oldData.calendartype == _changedData.calendartype) {
		_isChangedCommTime = false;
	}
	var _nowTime = new Date();
	//当前年
	var _nowYear = _nowTime.getFullYear();
	var _commTime = new Date();
	_commTime.setTime(parseInt(_changedData.commtime));
	//修改后的生日日期-年，月，日
	var _commYear = _commTime.getFullYear();
	var _commMonth = _commTime.getMonth() + 1;
	var _commDay = _commTime.getDate();
	var _commHours = _commTime.getHours();
	var _commMinutes = _commTime.getMinutes();
	var _commSeconds = _commTime.getSeconds();
	var _commMill = _commTime.getMilliseconds();
	var _commDataList = query_birthdayDataByCycleRecordId(_cycleRecord.id);
	//标识是否存在今年的卡片
	var haveCurrYearCard = false;
	var curYearCardId = "";
	var curYearCardNotifyId = "";
	//标识是否存在未来的卡片
	var futureYearCardId = "";
	var haveFutureYearCard = false;
	_nowTime.setTime(parseInt(_cycleRecord.updtime));
	_nowTime.setHours(_commHours, _commMinutes, _commSeconds, _commMill);
	// 周期（重复）时间
	var _updTime = _nowTime.getTime();
	// 当前重复卡片数
	var _currCount = parseInt(_cycleRecord.count);
	if (_commDataList != null) {
		for (var i = 0; i < _commDataList.length; i++) {
			var _commData = _commDataList[i];
			var _commTempStart = new Date();
			_commTempStart.setTime(parseInt(_commData.start));
			var _commDataYear = _commTempStart.getFullYear();
			if(_commDataYear == _nowYear){
				curYearCardId = _commData.id;
				curYearCardNotifyId = _commData.notifyid;
				haveCurrYearCard = true;
			}
			if(_commDataYear > _nowYear){
				futureYearCardId = _commData.id;
				haveFutureYearCard = true;
			}
		}
	}
	//遍历该纪念日期对应周期内的所有卡片
	if (_commDataList != null) {
		for (var i = 0; i < _commDataList.length; i++) {
			var _commData = _commDataList[i];
			if (_commData == null)
				continue;
			if (_isChangedCommTime) {
				var _commTempStart = new Date();
				//当前生日卡片的生日日期显示时间
				_commTempStart.setTime(parseInt(_commData.start));
				var _commTempYear = _commTempStart.getFullYear();
				_commTempStart.setHours(_commHours, _commMinutes, _commSeconds, _commMill);
				//如果修改后的年份小于等于今年
				if(_commYear <= _nowYear){
					if(_commTempYear <= _nowYear){
						//今年及今年以前的卡片月份和日期做更新
						_commTempStart = calendar.getCycleTimeOfComm({
							commtime: _commTime,
							updtime: _commTempStart,
							calendartype: _changedData.calendartype,
							isnow: true
						});
						_commData.commtime = _changedData.commtime;
						_commData.start = _commTempStart.getTime();
					}else{
						//看看以后的卡片，根据是不是主卡，分开处理
						if(_commData.id != _cycleRecord.taskid){
							//如果不是主卡，直接删除
							delBirthdayDataById(_commData.id,
									_commData.notifyid, _sqlList);
						}else{
							//如果是主卡，把它的年份变更为今年的
							_commTempStart = calendar.getCycleTimeOfComm({
								commtime: _commTime,
								updtime: (new Date()),
								calendartype: _changedData.calendartype,
								isnow: true
							});
							_commData.commtime = _changedData.commtime;
							_commData.start = _commTempStart.getTime();
							//判断原list中今年的卡片是否存在
							//若存在，就把它删除
							if(haveCurrYearCard){
								delBirthdayDataById(curYearCardId,
										curYearCardNotifyId, _sqlList);
							}
							//最近年等于当前年
							_updTime = _commData.start;
						}
					}
				}else{
					//如果修改后的年份大于今年
					if(_commTempYear <= _nowYear){
						//今年及今年以前的卡片,
						if(_commData.id != _cycleRecord.taskid){
							//如果不是主卡，直接更新即可
							_commTempStart = calendar.getCycleTimeOfComm({
								commtime: _commTime,
								updtime: _commTempStart,
								calendartype: _changedData.calendartype,
								isnow: true
							});
							_commData.commtime = _changedData.commtime;
							_commData.start = _commTempStart.getTime();
						}else{
							//如果是主卡
							//先复制一张主卡原来的年份的卡片(修改后的月和日）
							var _id = tools.createTaskId(_cycleRecord.taskid, ++_currCount);
							_commTempStart = calendar.getCycleTimeOfComm({
								commtime: _commTime,
								updtime: _commTempStart,
								calendartype: _changedData.calendartype,
								isnow: true
							});
							var _cycleData = {
									id : _id,
									infoid: _commData.infoid,
									commname : _commData.commname,
									//修改后的月，日
									start : _commTempStart.getTime(),
									type : _commData.type,
									//修改后的纪念日期
									commtime : _changedData.commtime,
									calendartype: _changedData.calendartype,
									remindtime : _commData.remindtime,
									cycleid : _commData.cycleid,
									cyclerecordid : _cycleRecord.id
							};
							insertBirthdayOneData(_cycleData, _sqlList);
							//新增完了以后，修改主卡的显示时间到未来年
							_commData.commtime = _changedData.commtime;
							_commData.start = _changedData.commtime;
							//最近年等于修改到的未来年
							_updTime = _commData.start;
						}
					}else{
						//未来的卡片
						if(_commData.id != _cycleRecord.taskid){
							//不是主卡，直接删除
							delBirthdayDataById(_commData.id,
									_commData.notifyid, _sqlList);
						}else{
							//是主卡的话，年月日全部更新，更新成新修改成的日期
							_commData.commtime = _changedData.commtime;
							_commData.start = _changedData.commtime;
							//最近年等于修改到的未来年
							_updTime = _commData.start;
						}
					}
				}
			}
			_commData.calendartype = _changedData.calendartype;
			_commData.commname = _changedData.commname;
			// 更新生日卡片信息
			updBirthdayOneData(_commData, _sqlList);
		}
		//只有时间变化了，才更新周期性记录表
		if(_isChangedCommTime){
			var _start = new Date();
			_start.setTime(parseInt(_cycleRecord.starttime));
			_start.setHours(_commHours, _commMinutes, _commSeconds, _commMill);
			_start = calendar.getCycleTimeOfComm({
				commtime: _commTime,
				updtime: _start,
				calendartype: _changedData.calendartype,
				isnow: true
			});
			var _cycleRecordData = {
				id : _cycleRecord.id,
				starttime : _start.getTime(),
				updtime : _updTime,
				count: _currCount
			};
			// 更新周期记录的数据
			_sql = dbcycle.getUpdCycleRecordSql(_cycleRecordData);
			_sqlList.push(_sql);
			
			_cycleRecord.starttime = _cycleRecordData.starttime;
			_cycleRecord.updtime = _cycleRecordData.updtime;
			_cycleRecord.count = _cycleRecordData.count;
		}
		var _commChangedData = {
			id : _oldData.id,
			infoid: _oldData.infoid,
			commname : _changedData.commname,
			commtime : _changedData.commtime,
			calendartype : _changedData.calendartype,
			start : _oldData.start,
			remindtime : _oldData.remindtime,
			cycleid : _oldData.cycleid,
			cyclerecordid : _oldData.cyclerecordid,
			infocategory : _oldData.infocategory,
			create_time : _oldData.create_time
		};
		// 获取增量生成生日卡片的sql语句
		getInsertBirthdayBySourceSql(_commChangedData, _cycleRecord, _sqlList);
	}
}

// 获取更新生日卡片的所有sql语句
module.exports.getUpdBirthdayAllSqls = function(_changedData, _sqlList) {
	var _data = query_birthdayDataByCycleRecordId(_changedData.id); 
	if(null != _data && _data.length > 0){
		getUpdBirthdayAllSqls(_changedData, _sqlList);
	} else {
		getInsertBirthdayAllSqls(_changedData, _sqlList);
	}
};

/**
 * 更新所有生日卡片数据
 * @param _changedData
 * @returns
 */
function updBirthdayAllData(_changedData) {
	var _sqlList = [];
	getUpdBirthdayAllSqls(_changedData, _sqlList);
}

// 更新生日卡片数据
module.exports.updBirthdayData = function(_changedData, _sqlList) {
	var _data = query_birthdayDataByCycleRecordId(_changedData.id); 
	if(null != _data && _data.length > 0){
		updBirthdayAllData(_changedData);
	} else {
		initAndInsertBirthdayData(_changedData);
	}
	//时间流数据重新加载
	do_global.fire("taskDataChangeded");
	//更新未来卡片页面
	do_global.fire("futureCardDataChanged");
};

/**
 * 获取删除联系人生日卡片的sql语句
 * @param _id
 * @returns
 */
function getDelBirthDataSql(_id){
	var _sql_del = "delete from mytask where id=" + tools.formatSqlValue(_id) + ";";
	return _sql_del;
}

/**
 * 删除指定id的生日卡片数据
 * @param _id
 * @param _notifyid
 * @param _sqlList
 * @returns
 */
function delBirthdayDataById(_id, _notifyid, _sqlList) {
	// 移除本地提醒
	notificationTools.removeLocalNotification([parseInt(_notifyid)], 
			[_id]);
	
	var _sql = getDelBirthDataSql(_id);
	_sqlList.push(_sql);
}

/**
 * 获取删除联系人所有生日卡片的sql语句
 * @param _cycleRecordId
 * @param _sqlList
 * @returns
 */
function getDelBirthAllDataSql(_cycleRecordId, _sqlList){
	var _sql = "";
	var _cycleRecord = dbcycle.query_taskidByCycleRecordId(_cycleRecordId);
	if (null != _cycleRecord) {
		var _birthDataList = query_birthdayDataIdByCycleRecordId(_cycleRecordId);
		if (null != _birthDataList) {
			var _notifyIds = [];
			var _taskIds = [];
			for (var i = 0; i < _birthDataList.length; i++) {
				var _id = _birthDataList[i].id;
				var _notifyId = _birthDataList[i].notifyid;
				if (null == _id) continue;
				_sql = getDelBirthDataSql(_id);
				_sqlList.push(_sql);
				_taskIds.push(_id);
				if(null == _notifyId || "" == _notifyId) continue;
				_notifyIds.push(_notifyId);
			}
			// 移除本地提醒
			notificationTools.removeLocalNotification(_notifyIds, _taskIds);
		}
		// 获取删除周期记录的sql语句
		_sql = dbcycle.getDelCycleRecordDataSql(_cycleRecordId);
		_sqlList.push(_sql);
	}
}

//获取删除联系人所有生日卡片的sql语句
module.exports.getDelBirthAllDataSql = function(_cycleRecordId, _sqlList){
	getDelBirthAllDataSql(_cycleRecordId, _sqlList);	
};

/**
 * 删除mytask表中指定id生日的所有数据
 * 
 * @param _cycleRecordId
 */
function delBirthdayAllData(_cycleRecordId) {
	var _sqlList = [];
	getDelBirthAllDataSql(_cycleRecordId, _sqlList);
	var isSuccess = localdb.executeSync1({sqls: _sqlList});
	return isSuccess;
}

// 删除生日卡片数据
module.exports.delBirthdayData = function(_cycleRecordId) {
	//删除对应的所有生日数据
	var _isSuccess = delBirthdayAllData(_cycleRecordId);
	
	//生日卡片删除成功后更新时间流首页面
	if(_isSuccess){
		//时间流数据重新加载
		do_global.fire("taskDataChangeded");
		//更新未来卡片页面
		do_global.fire("futureCardDataChanged");
	}
};

/**
 * 通过cyclerecordid从mytask表中获取对应的生日卡片数据
 * 
 * @param cyclerecordid
 * @returns
 */
function query_birthdayDataByCycleRecordId(_cycleRecordId) {
	var sql = "select * from mytask where cyclerecordid="
			+ tools.formatSqlValue(_cycleRecordId) + "order by start ASC;";
	var _data = localdb.querySync(sql);
	if (_data == null || _data.length <= 0) {
		return null;
	}
	return _data;
}

/**
 * 通过cyclerecordid从mytask表中获取对应的生日卡片id
 * 
 * @param cyclerecordid
 * @returns
 */
function query_birthdayDataIdByCycleRecordId(_cycleRecordId) {
	var sql = "select id, notifyid from mytask where cyclerecordid="
			+ tools.formatSqlValue(_cycleRecordId) + "order by start ASC;";
	var _data = localdb.querySync(sql);
	if (_data == null || _data.length <= 0) {
		return null;
	}
	return _data;
}

module.exports.query_birthdayDataIdByCycleRecordId = function(_cycleRecordId) {
	query_birthdayDataIdByCycleRecordId(_cycleRecordId);
};

/**
 * 异步查询指定条件和字段的卡片信息
 * @param _fields
 * @param _conditions
 * @param _callFunc
 * @returns
 */
function query_birthFieldsByCondition(_fields, _conditions, _callFunc){
	var _sql ="select " +  _fields + " from mytask where infocategory="
		+ infoCategoryBirthday + " and ";
	if(null != _conditions && _conditions.length > 0){
		_sql += _conditions;
		localdb.query({sql: _sql}, function(_data){
			_callFunc.call(this, _data);
		});
	} else {
		_callFunc.call(this, null);
	}
}

// 异步查询指定条件和字段的卡片信息
module.exports.query_birthFieldsByCondition = function(_fields, _condition, _callFunc){
	query_birthFieldsByCondition(_fields, _condition, _callFunc);
};