/**
 * 数据同步-云同步
 * 
 * @Author : Cydow
 * @Timestamp : 2019-10-08
 */
var deviceone = require("deviceone");
var tools = require("tools");

//数据库
var localdb = deviceone.mm("M0026_sqlite", "localdb", "app");
var objectId = require("objectId");       //生成objectId

// 同步状态标识
var syncState_no = 0; // 未同步
var syncState_yes = 1; // 已同步

//-------------
//-------------------- 更新记录表（datasync_cl_updrecord）
// ------------
/**
 * 获取插入更新记录的sql语句
 * @param _updRecord
 * @returns
 */
function getInsertUpdRecordDataSql(_updRecord){
	var _sql_insert = "insert into datasync_cl_updrecord(dataid, moduleid, " +
			"recordtime, recordtype, datatype, handlestate) values("
		+ tools.formatSqlValue(_updRecord.id) + ","
		+ tools.formatSqlValue(_updRecord.moduleid) + ","
		+ _updRecord.recordtime + ","
		+ _updRecord.recordtype + ","
		+ tools.formatSqlValue(_updRecord.datatype) + ","
		+ _updRecord.handlestate + ");";
	return _sql_insert;
}

//获取更新记录的sql语句
module.exports.getInsertUpdRecordDataSql = function(_updRecord){
	_updRecord["handlestate"] = syncState_yes;
	return getInsertUpdRecordDataSql(_updRecord);
};

//获取更新记录的sql语句
module.exports.getInsertUnHandleUpdRecordSql = function(_updRecord){
	_updRecord["handlestate"] = syncState_no;
	return getInsertUpdRecordDataSql(_updRecord);
};

/**
 * 向更新记录中插入数据
 * @param _updRecord
 */
function insertUpdRecordData(_updRecord){
	var _sql_updrecord_insert = getInsertUpdRecordDataSql(_updRecord);
	localdb.executeSync(_sql_updrecord_insert);
}

/**
 * 获取数据更新记录的sql语句
 * @param _updRecord
 * @returns
 */
function getUpdateUpdRecordDataSql(_updRecord){
	var _sql_update = "update datasync_cl_updrecord set " 
		+ "recordtime=" + _updRecord.recordtime 
		+ ", recordtype=" + _updRecord.recordtype 
		+ ", handlestate=" + _updRecord.handlestate
		+ " where dataid=" + tools.formatSqlValue(_updRecord.id) + ";";
	return _sql_update;
}

//获取更新记录的sql语句-已同步
module.exports.getUpdateUpdRecordSql = function(_updRecord){
	_updRecord.handlestate = syncState_yes;
	return getUpdateUpdRecordDataSql(_updRecord);
};

//获取更新记录的sql语句-未同步
module.exports.getUpdateUpdRecordUnHandleSql = function(_updRecord){
	_updRecord.handlestate = syncState_no;
	return getUpdateUpdRecordDataSql(_updRecord);
};

/**
 * 更新更新记录表中的数据
 * @param _updRecord
 */
function updateUpdRecordData(_updRecord){
	var _sql_updrecord_update = getUpdateUpdRecordDataSql(_updRecord);
	localdb.executeSync(_sql_updrecord_update);
}

/**
 * 获取更新记录的sql语句
 * @param _updRecord
 * @param _syncState
 * @returns
 */
function getUpdRecordSql(_updRecord, _syncState){
	var _sql = "";
	//记录更新（新增）
	var _recordData = queryUpdRecordDataById(_updRecord.id);
	var _recordTime = _updRecord.recordtime;
	if(tools.isNull(_recordTime)){
		_recordTime = (new Date()).getTime();
	}
	if(null == _recordData) {
		_sql = getInsertUpdRecordDataSql({
			id: _updRecord.id,
			moduleid: _updRecord.moduleid,
			recordtime: _recordTime,
			recordtype: _updRecord.recordtype,
			datatype: _updRecord.datatype,
			handlestate: _syncState
		});
	} else {
		_sql = getUpdateUpdRecordDataSql({
			id: _updRecord.id,
			recordtime: _recordTime,
			recordtype: _updRecord.recordtype,
			handlestate: _syncState
		});
	}
	return _sql;
}

// 获取更新记录的sql语句-已同步
module.exports.getUpdRecordSql = function(_updRecord){
	return getUpdRecordSql(_updRecord, syncState_yes);
};

//获取更新记录的sql语句-未同步
module.exports.getUpdRecordUnHandleSql = function(_updRecord){
	return getUpdRecordSql(_updRecord, syncState_no);
};

/**
 * 设置更新记录信息
 * @param _updRecord
 * @param _syncState
 */
function setUpdRecord(_updRecord, _syncState){
	var _sql = getUpdRecordSql(_updRecord, _syncState);
	localdb.executeSync(_sql);
}

// 设置更新记录信息-已同步
module.exports.setUpdRecord = function(_updRecord){
	setUpdRecord(_updRecord, syncState_yes);
};

//设置更新记录信息-未同步
module.exports.setUpdRecordUnHandle = function(_updRecord){
	setUpdRecord(_updRecord, syncState_no);
};

/**
 * 设置更新记录为未同步
 * @param _moduleId
 * @returns
 */
function setUpdRecordUnSync(_moduleId){
	var _sql_update = "update datasync_cl_updrecord set handlestate=" + syncState_no
		+ " where moduleid=" + tools.formatSqlValue(_moduleId) + ";";
	localdb.executeSync(_sql_update);
}

// 设置更新记录为未同步
module.exports.setUpdRecordUnSync = function(_moduleId){
	setUpdRecordUnSync(_moduleId);
};

/**
 * 设置指定id更新记录为未同步
 * @param _id
 * @returns
 */
function setUpdRecordUnSyncById(_id){
	var _sql_update = "update datasync_cl_updrecord set handlestate=" + syncState_no
		+ " where dataid=" + tools.formatSqlValue(_id) + ";";
	localdb.executeSync(_sql_update);
}

// 设置指定id更新记录为未同步
module.exports.setUpdRecordUnSyncById = function(_id){
	setUpdRecordUnSyncById(_id);
};

/**
 * 设置一些id对应的更新记录为同步
 * @param _ids
 * @returns
 */
function setUpdRecordSyncByIds(_ids){
	var _idStr = "";
	for(var i=0; i<_ids.length; i++){
		if(null == _ids[i] ||
				"" == _ids[i]){
			continue;
		}
		if(_idStr.length > 0){
			_idStr += ",";
		}
		_idStr += tools.formatSqlValue(_ids[i]);
	}
	if(_idStr.length > 0){
		var _sql_update = "update datasync_cl_updrecord set handlestate=" + syncState_yes
			+ " where dataid in (" + _idStr + ");";
		localdb.executeSync(_sql_update);
	}
}

// 设置一些id对应的更新记录为同步
module.exports.setUpdRecordSyncByIds = function(_ids){
	setUpdRecordSyncByIds(_ids);
};

/**
 * 通过id查询更新记录表中的数据
 * @param _id
 * @returns
 */
function queryUpdRecordDataById(_id){
	var _sql_updrecord_query = "select * from datasync_cl_updrecord where dataid=" 
		+ tools.formatSqlValue(_id) + ";";
	var _data_updrecord = localdb.querySync(_sql_updrecord_query);
	if(null == _data_updrecord || _data_updrecord.length <= 0){
		return null;
	}
	
	return _data_updrecord[0];
}

//通过id查询更新记录
module.exports.queryUpdRecordDataById = function(_id){
	return queryUpdRecordDataById(_id);
}

/**
 * 查询所有未同步的更新记录
 * @param _moduleId
 * @returns
 */
function queryUpdRecordUnSync(_moduleId){
	var _sql_query = "select * from datasync_cl_updrecord where moduleid=" 
		+ tools.formatSqlValue(_moduleId) + 
		" and handlestate=" + syncState_no + ";";
	var _data = localdb.querySync(_sql_query);
	return _data;
}

// 查询所有未同步的更新记录
module.exports.queryUpdRecordUnSync = function(_moduleId){
	return queryUpdRecordUnSync(_moduleId);
}


//-------------
//-------------------- 同步状态信息表（datasync_cl_syncinfo）
//------------
/**
 * 获取插入清零计数的sql语句
 * @param _syncInfo
 * @returns
 */
function getInsertClearCountSql(_syncInfo){
	var _sql_insert = "insert into datasync_cl_syncinfo(moduleid, " +
			"clearcount, cleartime) values("
		+ tools.formatSqlValue(_syncInfo.moduleid) + ","
		+ _syncInfo.clearcount + ","
		+ _syncInfo.cleartime + ");";
	return _sql_insert;
}

/**
 * 插入清零计数的数据
 * @param _syncInfo
 */
function insertClearCountData(_syncInfo){
	var _sql_insert = getInsertClearCountSql(_syncInfo);
	localdb.executeSync(_sql_insert);
}

/**
 * 获取插入同步计数的sql语句
 * @param _syncInfo
 * @returns
 */
function getInsertSyncCountSql(_syncInfo){
	var _id = objectId.getObjectId();
	var _sql_insert = "insert into datasync_cl_syncinfo(moduleid, synccount, " +
			"synctime) values("
		+ tools.formatSqlValue(_syncInfo.moduleid) + ","
		+ _syncInfo.synccount + ","
		+ _syncInfo.synctime + ");";
	return _sql_insert;
}

/**
 * 插入同步计数的数据
 * @param _syncInfo
 */
function insertSyncCountData(_syncInfo){
	var _sql_insert = getInsertSyncCountSql(_syncInfo);
	localdb.executeSync(_sql_insert);
}

/**
 * 获取更新同步信息的sql语句
 * @param _syncInfo
 * @returns
 */
function getUpdateSyncInfoSql(_syncInfo){
	var _setSqls = "";
	if (_syncInfo.synccount != null) {
		if (_setSqls.length > 0)
			_setSqls += ",";
		_setSqls += "synccount=" + _syncInfo.synccount;
	}
	if (_syncInfo.synctime != null) {
		if (_setSqls.length > 0)
			_setSqls += ",";
		_setSqls += "synctime=" + _syncInfo.synctime;
	}
	if (_syncInfo.clearcount != null) {
		if (_setSqls.length > 0)
			_setSqls += ",";
		_setSqls += "clearcount=" + _syncInfo.clearcount;
	}
	if (_syncInfo.cleartime != null) {
		if (_setSqls.length > 0)
			_setSqls += ",";
		_setSqls += "cleartime=" + _syncInfo.cleartime;
	}
	var _sql_update = "update datasync_cl_syncinfo set " + _setSqls
		+ " where moduleid=" + tools.formatSqlValue(_syncInfo.moduleid) + ";";
	return _sql_update;
}

/**
 * 更新同步信息
 * @param _syncInfo
 */
function updateSyncInfo(_syncInfo){
	var _sql_update = getUpdateSyncInfoSql(_syncInfo);
	localdb.executeSync(_sql_update);
}

// 更新同步信息
module.exports.updateSyncInfo = function(_syncInfo){
	updateSyncInfo(_syncInfo);
};

/**
 * 设置清零计数
 * @param _syncInfo
 */
function setClearCount(_syncInfo){
	//记录更新（新增）
	var _syncStateData = querySyncInfoByModuleId(_syncInfo.moduleid);
	if(null == _syncStateData) {
		insertClearCountData(_syncInfo);
	} else {
		updateSyncInfo(_syncInfo);
	}
}

// 设置清零计数
module.exports.setClearCount = function(_syncInfo){
	setClearCount(_syncInfo);
};

/**
 * 设置同步计数
 * @param _syncInfo
 */
function setSyncCount(_syncInfo){
	//记录更新（新增）
	var _syncStateData = querySyncInfoByModuleId(_syncInfo.moduleid);
	if(null == _syncStateData) {
		insertSyncCountData(_syncInfo);
	} else {
		updateSyncInfo(_syncInfo);
	}
}

// 设置同步计数
module.exports.setSyncCount = function(_syncInfo){
	setSyncCount(_syncInfo);
};

/**
 * 通过moduleid查询同步信息
 * @param _moduleId
 * @returns
 */
function querySyncInfoByModuleId(_moduleId){
	var _sql_query = "select * from datasync_cl_syncinfo where moduleid=" 
		+ tools.formatSqlValue(_moduleId) + ";";
	var _data = localdb.querySync(_sql_query);
	if(null == _data || _data.length <= 0){
		return null;
	}
	return _data[0];
}

// 通过moduleid查询同步信息
module.exports.querySyncInfoByModuleId = function(_moduleId){
	return querySyncInfoByModuleId(_moduleId);
}


//-------------
//-------------------- 失败记录表（datasync_cl_failrecord）
//------------
/**
 * 获取插入失败记录的sql语句
 * @param _failRecord
 * @param _moduleId
 * @returns
 */
function getInsertFailRecordSql(_failRecord, _moduleId){
	var _sql_insert = "insert into datasync_cl_failrecord(dataid, moduleid, " +
			"recordtime, recordtype, datatype) values("
		+ tools.formatSqlValue(_failRecord.dataid) + ","
		+ tools.formatSqlValue(_moduleId) + ","
		+ _failRecord.recordtime + ","
		+ _failRecord.recordtype + ","
		+ tools.formatSqlValue(_failRecord.datatype) + ");";
	return _sql_insert;
}

/**
 * 插入失败记录
 * @param _failRecord
 * @param _moduleId
 * @returns
 */
function insertFailRecordData(_failRecord, _moduleId){
	var _sql_insert = getInsertFailRecordSql(_failRecord, _moduleId);
	localdb.executeSync(_sql_insert);
}

/**
 * 获取更新失败记录的sql语句
 * @param _failRecord
 * @returns
 */
function getUpdateFailRecordSql(_failRecord){
	var _setSqls = "";
	if (_syncInfo.recordtime != null) {
		if (_setSqls.length > 0)
			_setSqls += ",";
		_setSqls += "recordtime=" + _failRecord.recordtime;
	}
	if (_failRecord.recordtype != null) {
		if (_setSqls.length > 0)
			_setSqls += ",";
		_setSqls += "recordtype=" + _failRecord.recordtype;
	}
	var _sql_update = "update datasync_cl_failrecord set " 
		+ _setSqls + " where dataid=" 
		+ tools.formatSqlValue(_failRecord.dataid) + ";";
	return _sql_update;
}

/**
 * 更新失败记录
 * @param _failRecord
 * @returns
 */
function updateFailRecordData(_failRecord){
	var _sql_update = getUpdateFailRecordSql(_failRecord);
	localdb.executeSync(_sql_update);
}

/**
 * 插入大量失败记录
 * @param _failRecords
 * @param _moduleId
 * @param _callFunc
 * @returns
 */
function insertMultipleFailRecord(_failRecords, _moduleId, _callFunc){
	var _sql_insert = "insert into datasync_cl_failrecord(dataid," +
			" moduleid, recordtime, recordtype, datatype) values";
	var _sql = "";
	var _id = "";
	for(var i=0; i<_failRecords.length; i++){
		if(null == _failRecords[i]) continue;
		if(_sql.length > 0){
			_sql += ",";
		}
		_sql += "(";
		_sql += tools.formatSqlValue(_failRecords[i].dataid) + ",";
		_sql += tools.formatSqlValue(_moduleId) + ",";
		_sql += _failRecords[i].recordtime + ",";
		_sql += _failRecords[i].recordtype + ",";
		_sql += tools.formatSqlValue(_failRecords[i].datatype) + ")";
	}
	if(_sql.length > 0){
		_sql_insert += _sql + ";";
		localdb.execute(_sql_insert, function(){
			_callFunc.call(this);
		});
	} else {
		_callFunc.call(this);
	}
}

/**
 * 设置多个失败记录数据
 * @param _failRecord
 * @param _moduleId
 * @param _callFunc
 * @returns
 */
function setMultipleFailReocrd(_failRecord, _moduleId, _callFunc){
	// 删除指定模块的所有失败记录
	delFailRecordByModuleId(_moduleId, function(ret){
		// 插入大量失败记录
		insertMultipleFailRecord(_failRecord, _moduleId, _callFunc);
	});
}

// 设置多个失败记录数据
module.exports.setMultipleFailReocrd = function(_failRecord, _moduleId, _callFunc){
	setMultipleFailReocrd(_failRecord, _moduleId, _callFunc);
};

/**
 * 通过id查询失败记录
 * @param _id
 * @returns
 */
function queryFailRecordById(_id){
	var _sql_query = "select * from datasync_cl_failrecord where dataid=" 
		+ tools.formatSqlValue(_id) + ";";
	var _data = localdb.querySync(_sql_query);
	if(null == _data || _data.length <= 0){
		return null;
	}
	return _data[0];
}

// 通过id查询失败记录
module.exports.queryFailRecordById = function(_id){
	return queryFailRecordById(_id);
}

/**
 * 查询所有失败记录
 * @param _moduleId
 * @returns
 */
function queryAllFailRecord(_moduleId){
	var _sql_query = "select * from datasync_cl_failrecord where " +
			"moduleid=" + tools.formatSqlValue(_moduleId) + ";";
	var _data = localdb.querySync(_sql_query);
	return _data;
}

// 查询所有失败记录
module.exports.queryAllFailRecord = function(_moduleId){
	return queryAllFailRecord(_moduleId);
}

/**
 * 通过id删除失败记录
 * @param _id
 * @returns
 */
function delFailRecordById(_id){
	var _sql_del = "delete from datasync_cl_failrecord where dataid=" 
		+ tools.formatSqlValue(_id) + ";";
	var _result = localdb.executeSync(_sql_del);
	return _result;
}

// 通过id删除失败记录
module.exports.delFailRecordById = function(_id){
	return delFailRecordById(_id);
}

/**
 * 删除指定模块的所有失败记录
 * @param _moduleId
 * @param _callFunc
 * @returns
 */
function delFailRecordByModuleId(_moduleId, _callFunc){
	var _sql_del = "delete from datasync_cl_failrecord where moduleid=" 
		+ tools.formatSqlValue(_moduleId) + ";";
	localdb.execute(_sql_del, function(_result){
		_callFunc.call(this, _result);
	});
}

// 删除指定模块的所有失败记录
module.exports.delFailRecordByModuleId = function(_moduleId, _callFunc){
	return delFailRecordByModuleId(moduleId, _callFunc);
}

/**
 * 删除所有失败记录
 * @returns
 */
function delAllFailRecord(){
	var _sql_del = "delete from datasync_cl_failrecord;";
	var _result = localdb.executeSync(_sql_del);
	return _result;
}

// 删除所有失败记录
module.exports.delAllFailRecord = function(){
	return delAllFailRecord();
}
