/**
 * 电子名片
 * 
 * @Author : Cydow
 * @Timestamp : 2017-08-09
 */
//引入组件库
var deviceone = require("deviceone");
var do_storage = deviceone.sm("do_Storage");
var do_global = sm("do_Global");

var objectId = require("objectId");      //生成objectId
var tools = require("tools");            //工具
var constants = require("constants");    //常量
var dbclsyncdata = require("dbclsyncdata");  //数据同步
var dbcontactproperties = require("dbcontactproperties"); //联系人属性值
var dbbirthday = require("dbbirthday");

//小部件工具
var widgetTools = require("widgetTools.ui");

//数据库
var localdb = deviceone.mm("M0026_sqlite", "localdb", "app");

//业务模块ID
var moduleId = constants.getConstants("moduleType_contact");

/**
 * 时间是否为空
 * @param _time
 * @returns
 */
function isNullTime(_time){
	if(null == _time || 
			('string' == (typeof _time) && "" == _time)){
		return true;
	}
	return false;
}

/**
 * 获取插入名片数据的sql语句
 * @param _changedData
 * @returns
 */
function getInsertVCardDataSql(_changedData){
	var _nowTime = new Date();
	if (_changedData.id == null){
		_changedData.id = objectId.getObjectId();
	}
	if (_changedData.infoid == null){
		_changedData.infoid = "";
	}
	if(_changedData.headurl == null){
		_changedData.headurl = "";
	}
	if(_changedData.name == null){
		_changedData.name  = "";
	}
	if(_changedData.gender == null){
		_changedData.gender  = "";
	}
	if(_changedData.company == null){
		_changedData.company  = "";
	}
	if(_changedData.department == null){
		_changedData.department  = "";
	}
	if(_changedData.position == null){
		_changedData.position  = "";
	}
	if(_changedData.website == null){
		_changedData.website  = "";
	}
	if(_changedData.remarks == null){
		_changedData.remarks  = "";
	}
	if(_changedData.defaultflag == null){
		_changedData.defaultflag = "0";
	}
	if(_changedData.origin == null){
		_changedData.origin = "";
	}
	if(_changedData.presrc == null){
		_changedData.presrc = "";
	}
	if(_changedData.morefields == null){
		_changedData.morefields = "";
	}
	if(isNullTime(_changedData.starttime)){
		_changedData.starttime = null;
	}
	if(isNullTime(_changedData.endtime)){ 
		_changedData.endtime = null;
	}
	if(_changedData.createtime == null){
		_changedData.createtime  = _nowTime.getTime();
	}
	if(_changedData.updatetime == null){
		_changedData.updatetime  = _nowTime.getTime();
	}
	var _sql_insert = "insert into contacts_vcard(id, infoid, headurl," +
			" name, gender, company, department, position, website, ";
	if(!isNullTime(_changedData.birthday)){
		_sql_insert += "birthday, ";
	}
	if(_changedData.calendartype != null && 
			_changedData.calendartype != ""){
		_sql_insert += "calendartype, ";
	}
	_sql_insert += "remarks, defaultflag, origin, presrc, morefields, starttime, " +
			"endtime, createtime, updatetime)values("
		+ tools.formatSqlValue(_changedData.id) + ","
		+ tools.formatSqlValue(_changedData.infoid) + ","
		+ tools.formatSqlValue(_changedData.headurl) + ","
		+ tools.formatSqlValue(_changedData.name) + ","
		+ tools.formatSqlValue(_changedData.gender) + ","
		+ tools.formatSqlValue(_changedData.company) + ","
		+ tools.formatSqlValue(_changedData.department) + ","
		+ tools.formatSqlValue(_changedData.position) + ","
		+ tools.formatSqlValue(_changedData.website) + ",";
	if(!isNullTime(_changedData.birthday)){
		_sql_insert += _changedData.birthday + ",";
	}
	if(_changedData.calendartype != null && 
			_changedData.calendartype != ""){
		_sql_insert += tools.formatSqlValue(_changedData.calendartype) + ",";
	}
		
	_sql_insert += tools.formatSqlValue(_changedData.remarks) + ","
		+ tools.formatSqlValue(_changedData.defaultflag) + ","
		+ tools.formatSqlValue(_changedData.origin) + ","
		+ tools.formatSqlValue(_changedData.presrc) + ","
		+ tools.formatSqlValue(_changedData.morefields) + ","
		+ _changedData.starttime + ","
		+ _changedData.endtime + ","
		+ _changedData.createtime + ","
		+ _changedData.updatetime + ");";
	return _sql_insert;
}

//获取插入名片数据的sql语句
module.exports.getInsertVCardDataSql = function(_changedData){
	return getInsertVCardDataSql(_changedData);	
};

/**
 * 获取插入名片数据的所有sql语句
 * @param _changedData
 * @param _sqlList
 * @param _isSync
 * @param _isRecord
 * @returns
 */
function getInsertVCardAllSqls(_changedData, _sqlList, _isSync, _isRecord){
	// 获取插入名片数据的sql语句
	var _sql = getInsertVCardDataSql(_changedData);
	_sqlList.push(_sql);
	
	// 设置名片属性
	if(null != _changedData.properties &&
			_changedData.properties.length > 0) {
		// 获取插入联系人属性数据的sql语句
		_sql = dbcontactproperties.getInsertContactPropertySql(_changedData.id,
				_changedData.properties);
		_sqlList.push(_sql);
	}
	
	// 若为默认名片，且存在生日的时候，则在时间流里面创建生日卡片
	if('1' == _changedData.defaultflag && 
			!isNullTime(_changedData.birthday)){
		// 获取插入名片生日数据的所有sql语句
		dbbirthday.getInsertBirthdayAllSqls({
			id: _changedData.id,
			commname: _changedData.name,
			commtime: _changedData.birthday,
			calendartype: _changedData.calendartype
		}, _sqlList);
	}
	
	if(_isRecord){
		// 更新记录
		var _updRecord = {
				id: _changedData.id,
				moduleid: moduleId,
				recordtype: constants.getConstants("recordType_create"), 
				datatype: constants.getConstants("datatype_contactcard")
			};
		// 设置更新记录
		if(_isSync){
			_sql = dbclsyncdata.getUpdRecordUnHandleSql(_updRecord);
		} else {
			_sql = dbclsyncdata.getUpdRecordSql(_updRecord);
		}
		_sqlList.push(_sql);
	}
}

// 获取插入名片相关数据的所有sql语句
module.exports.getInsertVCardAllSqls = function(_changedData,
		_sqlList, _isSync, _isRecord){
	getInsertVCardAllSqls(_changedData, _sqlList, _isSync, _isRecord);	
};

/**
 * 插入名片数据
 * @param _changedData
 * @param _sqlList
 * @param _callFunc
 * @returns
 */
function insertVCardData(_changedData, _sqlList, _callFunc){
	getInsertVCardAllSqls(_changedData, _sqlList, true, true);
	// 执行多条sql语句
	localdb.execute1({sqls: _sqlList}, function(_result){
		var _isSuc = false;
		if(_result > 0){
			_isSuc = true;
			_sqlList = null;
			// 若名片生日不为空，则刷新时间流主页面
			if('1' == _changedData.defaultflag && 
					!isNullTime(_changedData.birthday)){
				//时间流数据重新加载
				do_global.fire("taskDataChangeded");
				// 刷新视图（桌面小部件）
				widgetTools.refresh();
			}
		}
		// 回调函数
		_callFunc(this, _isSuc);
	});
}

// 插入名片数据
module.exports.insertVCardData = function(_changedData, _callFunc){
	insertVCardData(_changedData, [], _callFunc);	
};

/**
 * 插入名片数据--数据同步
 * @param _changedData
 * @param _callFunc
 * @returns
 */
function insertVCardSyncData(_changedData, _callFunc){
	var _sqlList = [];
	// 若为默认名片，则查询已有默认名片
	if("1" == _changedData.defaultflag){
		// 查询默认名片信息
		var _vcardData = query_contactCardDataByDefaultFlag("1");
		if(null != _vcardData){
			// 若新的默认名片的更新时间大于已有默认名片的更新时间，
			// 则将已有默认名片改为非默认的；否则，将新的默认名片改为非默认的
			if(parseInt(_changedData.updatetime) > parseInt(_vcardData.updatetime)){
				var _sql = "";
				// 更新默认名片标志的sql语句
				_sql = getUpdDefaultVCardSql(_vcardData.id, "0");
				_sqlList.push(_sql);
				
				// 若存在生日，则在时间流里删除生日卡片
				if(!isNullTime(_vcardData.birthday)){
					dbbirthday.getDelBirthAllDataSql(_vcardData.id, _sqlList);
				}
			} else {
				_changedData.defaultflag = "0";
			}
		}
	}
	// 获取插入名片数据的所有sql语句
	getInsertVCardAllSqls(_changedData, _sqlList, false, true);
	// 执行多条sql语句
	localdb.execute1({sqls: _sqlList}, function(_result){
		var _isSuc = false;
		if(_result > 0){
			_isSuc = true;
		}
		// 回调函数
		_callFunc(this, _isSuc);
	});
}

// 插入名片数据--数据同步
module.exports.insertVCardSyncData = function(_changedData, _callFunc){
	insertVCardSyncData(_changedData, _callFunc);
};

/**
 * 插入名片数据--信息库同步
 * @param _changedData
 * @param _record
 * @param _callFunc
 * @returns
 */
function insertVCardExpSync(_changedData, _record, _callFunc){
	var _sqlList = [];
	// 获取插入名片数据的所有sql语句
	getInsertVCardAllSqls(_changedData, _sqlList, false, false);
	// 设置更新记录-已同步
	var _sql = dbclsyncdata.getUpdRecordSql({
		id: _changedData.id,
		moduleid: moduleId,
		recordtime: _record.recordtime,
		recordtype: constants.getConstants("recordType_create"), 
		datatype: constants.getConstants("datatype_contactcard")
	});
	_sqlList.push(_sql);
	// 执行多条sql语句
	localdb.execute1({sqls: _sqlList}, function(_result){
		var _isSuc = false;
		if(_result > 0){
			_isSuc = true;
		}
		// 回调函数
		_callFunc(this, _isSuc);
	});
}

// 插入名片数据--信息库同步
module.exports.insertVCardExpSync = function(_changedData, _record, _callFunc){
	insertVCardExpSync(_changedData, _record, _callFunc);
};

/**
 * 获取更新名片数据的sql语句
 * @param _changedData
 * @returns
 */
function getUpdVCardDataSql(_changedData){
	print("_changedData " + JSON.stringify(_changedData));
	var _nowTime = new Date();
	var _setList="";
	if(_changedData.infoid != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "infoid=" + tools.formatSqlValue(_changedData.infoid);
	}
	if(_changedData.headurl != null && _changedData.headurl != ""){
		if (_setList.length > 0) _setList+=",";
		_setList += "headurl=" + tools.formatSqlValue(_changedData.headurl);
	}
	if(_changedData.name != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "name=" + tools.formatSqlValue(_changedData.name);
	}
	if(_changedData.gender != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "gender=" + tools.formatSqlValue(_changedData.gender);
	}
	if(_changedData.company != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "company=" + tools.formatSqlValue(_changedData.company);
	}
	if (_changedData.department != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "department=" + tools.formatSqlValue(_changedData.department);
	}
	if (_changedData.position != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "position=" + tools.formatSqlValue(_changedData.position);
	}
	if(_changedData.website != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "website=" + tools.formatSqlValue(_changedData.website);
	}
	if(isNullTime(_changedData.birthday)){
		_changedData.birthday = null;
	}
	if(_setList.length > 0) _setList+=",";
	_setList += "birthday=" + _changedData.birthday;
	if (_changedData.calendartype != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "calendartype=" + tools.formatSqlValue(_changedData.calendartype);
	}
	if (_changedData.remarks != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "remarks=" + tools.formatSqlValue(_changedData.remarks);
	}
	if(_changedData.defaultflag != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "defaultflag=" + tools.formatSqlValue(_changedData.defaultflag);
	}
	if(_changedData.morefields != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "morefields=" + tools.formatSqlValue(_changedData.morefields);
	}
	if(isNullTime(_changedData.starttime)){
		_changedData.starttime = null;
	}
	if(_setList.length > 0) _setList+=",";
	_setList += "starttime=" + _changedData.starttime;
	if (isNullTime(_changedData.endtime)){
		_changedData.endtime = null;
	}
	if(_setList.length > 0) _setList+=",";
	_setList += "endtime=" + _changedData.endtime;
	if(_changedData.deleted!=null){
		if (_setList.length > 0) _setList+=",";
		_setList += "deleted=" + _changedData.deleted;
	}
	if(_changedData.updatetime!=null){
		if (_setList.length > 0) _setList+=",";
		_setList += "updatetime=" + _changedData.updatetime;
	}
	var _sql = "";
	if (_setList.length <= 0) return _sql;
	_sql = "update contacts_vcard set " + _setList + " where id="
		+ tools.formatSqlValue(_changedData.id) + ";";
	print("_sql " + _sql);
	return _sql;
}

// 获取更新名片数据的sql语句
module.exports.getUpdVCardDataSql = function(_changedData){
	return getUpdVCardDataSql(_changedData);	
};

/**
 * 获取更新名片数据的所有sql语句
 * @param _changedData
 * @param _sqlList
 * @param _isRecord
 * @returns
 */
function getUpdVCardAllSqls(_changedData, _sqlList, _isSync, _isRecord){
	// 获取更新名片数据的sql语句
	var _sql = getUpdVCardDataSql(_changedData);
	_sqlList.push(_sql);
	
	// 获取更新属性数据的sql语句
	dbcontactproperties.getUpdContactPropAllSqls(_changedData.id,
			_changedData.properties, _sqlList);
	
	// 若为默认名片，则处理时间流里面创建的生日卡片信息；
	// 否则，删除名片的生日卡片
	if('1' == _changedData.defaultflag){
		// 删除或更新时间流里面名片对应的生日卡片信息
		if(isNullTime(_changedData.birthday)){
			// 获取删除名片生日卡片的所有sql语句
			dbbirthday.getDelBirthAllDataSql(_changedData.id, _sqlList);
		} else {
			// 获取更新生日卡片的所有sql语句
			dbbirthday.getUpdBirthdayAllSqls({
				id: _changedData.id,
				commname: _changedData.name,
				commtime: _changedData.birthday,
				calendartype: _changedData.calendartype
			}, _sqlList);
		}
	} else {
		// 获取删除名片生日卡片的所有sql语句
		dbbirthday.getDelBirthAllDataSql(_changedData.id, _sqlList);
	}
	if(_isRecord){
		// 更新记录
		var _updRecord = {
				id: _changedData.id,
				moduleid: moduleId,
				recordtype: constants.getConstants("recordType_update"), 
				datatype: constants.getConstants("datatype_contactcard")
			};
		// 设置更新记录
		if(_isSync){
			_sql = dbclsyncdata.getUpdRecordUnHandleSql(_updRecord);
		} else {
			_sql = dbclsyncdata.getUpdRecordSql(_updRecord);
		}
		_sqlList.push(_sql);
	}
}

// 获取更新名片相关数据的所有sql语句
module.exports.getUpdVCardAllSqls = function(_changedData,
		_sqlList, _isSync, _isRecord){
	getUpdVCardAllSqls(_changedData, _sqlList, _isSync, _isRecord);
};

/**
 * 更新名片数据
 * @param _changedData
 * @param _sqlList
 * @param _callFunc
 * @returns
 */
function updContactCardData(_changedData, _sqlList, _callFunc){
	getUpdVCardAllSqls(_changedData, _sqlList, true, true);
	// 执行多条sql语句
	localdb.execute1({sqls: _sqlList}, function(_result){
		var _isSuc = false;
		if(_result > 0){
			_isSuc = true;
			_sqlList = null;
			// 若名片生日不为空，则刷新时间流主页面
			if('1' == _changedData.defaultflag){
				//时间流数据重新加载
				do_global.fire("taskDataChangeded");
				// 刷新视图（桌面小部件）
				widgetTools.refresh();
			}
		}
		// 回调函数
		_callFunc(this, _isSuc);
	});
}

// 更新名片数据
module.exports.updContactCardData = function(_changedData, _callFunc){
	updContactCardData(_changedData, [], _callFunc);	
};

/**
 * 更新名片数据-数据同步
 * @param _changedData
 * @param _callFunc
 * @returns
 */
function updVCardSyncData(_changedData, _callFunc){
	var _sqlList = [];
	// 若为默认名片，则查询已有默认名片
	if("1" == _changedData.defaultflag){
		// 查询默认名片信息
		var _vcardData = query_contactCardDataByDefaultFlag("1");
		if(null != _vcardData){
			// 若新的名片和已有默认名片标识（id）不一样，则说明已更改默认
			// 名片，需要重新设置默认名片
			if(_changedData.id != _vcardData.id ){
				// 若新的默认名片的更新时间大于已有默认名片的更新时间，
				// 则将已有默认名片改为非默认的；否则，将新的默认名片改为非默认的
				if(parseInt(_changedData.updatetime) > parseInt(_vcardData.updatetime)){
					var _sql = "";
					// 更新默认名片标志的sql语句
					_sql = getUpdDefaultVCardSql(_vcardData.id, "0");
					_sqlList.push(_sql);
					
					// 若存在生日，则在时间流里删除生日卡片
					if(!isNullTime(_vcardData.birthday)){
						dbbirthday.getDelBirthAllDataSql(_vcardData.id, _sqlList);
					}
				} else {
					_changedData.defaultflag = "0";
				}
			}
		}
	}
	// 获取更新名片数据的所有sql语句
	getUpdVCardAllSqls(_changedData, _sqlList, false, true);
	// 执行多条sql语句
	localdb.execute1({sqls: _sqlList}, function(_result){
		var _isSuc = false;
		if(_result > 0){
			_isSuc = true;
		}
		// 回调函数
		_callFunc(this, _isSuc);
	});
}

// 更新名片信息-数据同步
module.exports.updVCardSyncData = function(_changedData, _callFunc){
	updVCardSyncData(_changedData, _callFunc);
};

/**
 * 更新名片数据-信息库同步
 * @param _changedData
 * @param _record
 * @param _callFunc
 * @returns
 */
function updVCardExpSync(_changedData, _record, _callFunc){
	var _sqlList = [];
	// 获取更新名片数据的所有sql语句
	getUpdVCardAllSqls(_changedData, _sqlList, false, false);
	// 设置更新记录-已同步
	var _updRecord = {
		id: _changedData.id,
		moduleid: moduleId,
		recordtime: _record.recordtime,
		recordtype: constants.getConstants("recordType_update"), 
		datatype: constants.getConstants("datatype_contactcard")
	};
	// 设置更新记录
	var _sql = "";
	if(_record.handlestate){
		// 设置更新记录-已同步
		_sql = dbclsyncdata.getUpdRecordSql(_updRecord);
	} else {
		// 设置更新记录-未同步
		_sql = dbclsyncdata.getUpdRecordUnHandleSql(_updRecord);
	}
	if(null != _sql && "" != _sql){
		_sqlList.push(_sql);
	}
	// 执行多条sql语句
	localdb.execute1({sqls: _sqlList}, function(_result){
		var _isSuc = false;
		if(_result > 0){
			_isSuc = true;
		}
		// 回调函数
		_callFunc(this, _isSuc);
	});
}

// 更新名片信息-信息库同步
module.exports.updVCardExpSync = function(_changedData, _record, _callFunc){
	updVCardExpSync(_changedData, _record, _callFunc);
};

/**
 * 更新名片头像数据
 * @param _changedData
 * @returns
 */
function update_vcardHeadData(_changedData){
	var _sql_update = "update contacts_vcard set headurl=" + 
		tools.formatSqlValue(_changedData.headurl) + " where id=" +
		tools.formatSqlValue(_changedData.id) + ";";
	localdb.execute(_sql_update, function(){
		_changedData.callFunc.call(this);
	});
}

// 更新名片头像数据
module.exports.update_vcardHeadData = function(_changedData){
	update_vcardHeadData(_changedData);	
};

/**
 * 获取删除名片的sql语句
 * @param _id
 * @returns
 */
function getDelVCardDataSql(_id){
	//删除名片的sql语句
	var _sql = "delete from contacts_vcard where id=" + tools.formatSqlValue(_id) + ";";
	return _sql;
}

/**
 * 获取删除名片涉及的所有sql语句
 * @param _id
 * @param _sqlList
 * @param _isSync
 * @param _isRecord
 * @returns
 */
function getDelVCardDataAllSqls(_id, _sqlList, _isSync, _isRecord){
	//删除名片的sql语句
	var _sql = getDelVCardDataSql(_id);
	_sqlList.push(_sql);
	
	//删除名片属性sql语句
	_sql = dbcontactproperties.getDelContactPropDataSql(_id);
	_sqlList.push(_sql);
	
	//删除名片生日卡片sql语句
	dbbirthday.getDelBirthAllDataSql(_id, _sqlList);
	
	if(_isRecord){
		// 更新记录
		var _updRecord = {
				id: _id,
				moduleid: moduleId,
				recordtype: constants.getConstants("recordType_delete"), 
				datatype: constants.getConstants("datatype_contactcard")
			};
		// 设置更新记录
		if(_isSync){
			_sql = dbclsyncdata.getUpdRecordUnHandleSql(_updRecord);
		} else {
			_sql = dbclsyncdata.getUpdRecordSql(_updRecord);
		}
		_sqlList.push(_sql);
	}
}

/**
 * 检测默认卡片并设置新的默认卡片
 * @param _id
 * @param _sqlList
 * @param _isRecord
 * @returns
 */
function checkAndSetDefVCard(_id, _sqlList, _isRecord){
	var _isRefresh = false;
	// 查询指定名片信息
	var _cardData = query_contactCardDataById(_id);
	// 若存在生日的时候，则删除生日卡片后，需要刷新时间流主页面
	if(!isNullTime(_cardData.birthday)){
		_isRefresh = true;
	}
	// 若名片信息不为空，且为默认名片，则设置最近创建的名片为默认名片
	if(null != _cardData && 
			"1" == _cardData.defaultflag){
		// 查询最近创建的非默认名片信息
		var _sql = "select * from contacts_vcard " +  
		"where createtime=(select max(createtime) from contacts_vcard where defaultflag='0');";  
		var _data = localdb.querySync(_sql);
		if(_data != null || _data.length > 0){
			_cardData = _data[0];
			if(null != _cardData){
				// 获取更新默认名片标志的所有sql语句
				getUpdDefVCardAllSqls(_cardData.id, "1",
						_sqlList, _isRecord);
				// 若存在生日的时候，则在时间流里面创建生日卡片，
				// 并需要刷新时间流主页面
				if(!isNullTime(_cardData.birthday)){
					// 获取插入名片生日数据的所有sql语句
					dbbirthday.getInsertBirthdayAllSqls({
						id: _cardData.id,
						commname: _cardData.name,
						commtime: _cardData.birthday,
						calendartype: _cardData.calendartype
					}, _sqlList);
					
					_isRefresh = true;
				}
			}
		}
	}
	return _isRefresh;
}

/**
 * 删除名片数据
 * @param _id
 */
function delVCardData(_id){
	var _sqlList = [];
	// 获取删除名片的所有sql语句
	getDelVCardDataAllSqls(_id, _sqlList, true, true);
	// 检测并设置默认名片
	var _isRefresh = checkAndSetDefVCard(_id, _sqlList, true);
	var _isSuc = localdb.executeSync1({sqls: _sqlList});
	print("_isSuc " + _isSuc);
	if(_isSuc){
		//删除电子名片头像
		do_storage.deleteDir("data://head/vcard/" + _id);
		// 是否刷新时间流主页面
		if(_isRefresh){
			//时间流数据重新加载
			do_global.fire("taskDataChangeded");
			// 刷新视图（桌面小部件）
			widgetTools.refresh();
		}
	}
	return _isSuc;
}

//删除名片数据
module.exports.delVCardData = function(_id){
	return delVCardData(_id);
};

/**
 * 删除名片
 * @param _id
 * @param _callFunc
 * @returns
 */
function delVCardSyncData(_id, _callFunc){
	var _sqlList = [];
	// 获取删除名片的所有sql语句
	getDelVCardDataAllSqls(_id, _sqlList, false, true);
	// 执行多条sql语句
	localdb.execute1({sqls: _sqlList}, function(_result){
		var _isSuc = false;
		if(_result > 0){
			_isSuc = true;
			//删除名片头像
			do_storage.deleteDir("data://head/vcard/" + _id);
		}
		// 回调函数
		_callFunc(this, _isSuc);
	});
}

//删除名片数据--数据同步
module.exports.delVCardSyncData = function(_id, _callFunc){
	delVCardSyncData(_id, _callFunc);
};

/**
 * 删除名片 - 信息库同步
 * @param _id
 * @param _record
 * @param _callFunc
 * @returns
 */
function delVCardExpSync(_id, _record, _callFunc){
	var _sqlList = [];
	// 获取删除名片的所有sql语句
	getDelVCardDataAllSqls(_id, _sqlList, false, false);
	// 设置更新记录-已同步
	var _sql = dbclsyncdata.getUpdRecordSql({
		id: _id,
		moduleid: moduleId,
		recordtime: _record.recordtime,
		recordtype: constants.getConstants("recordType_delete"), 
		datatype: constants.getConstants("datatype_contactcard")
	});
	_sqlList.push(_sql);
	// 执行多条sql语句
	localdb.execute1({sqls: _sqlList}, function(_result){
		var _isSuc = false;
		if(_result > 0){
			_isSuc = true;
			//删除名片头像
			do_storage.deleteDir("data://head/vcard/" + _id);
		}
		// 回调函数
		_callFunc(this, _isSuc);
	});
}

//删除名片数据--信息库同步
module.exports.delVCardExpSync = function(_id, _record, _callFunc){
	delVCardExpSync(_id, _record, _callFunc);
};

/**
 * 获取更新默认名片标志
 * @param _id
 * @param _defaultFlag
 * @returns
 */
function getUpdDefaultVCardSql(_id, _defaultFlag){
	var _nowTime = new Date();
	var _sql = "update contacts_vcard set defaultflag=" 
		+ tools.formatSqlValue(_defaultFlag) + ",updatetime="
		+ _nowTime.getTime()
		+ " where id=" + tools.formatSqlValue(_id) + ";";
	return _sql;
}

/**
 * 获取更新默认名片标志的所有sql语句
 * @param _id
 * @param _defaultFlag
 * @param _sqlList
 * @param _isRecord
 * @returns
 */
function getUpdDefVCardAllSqls(_id, _defaultFlag, _sqlList, _isRecord){
	var _sql = getUpdDefaultVCardSql(_id, _defaultFlag);
	print("getUpdDefaultVCardSql _sql " + _sql);
	_sqlList.push(_sql);
	
	if(_isRecord){
		// 更新记录
		var _updRecord = {
				id: _id,
				moduleid: moduleId,
				recordtype: constants.getConstants("recordType_update"), 
				datatype: constants.getConstants("datatype_contactcard")
			};
		_sql = dbclsyncdata.getUpdRecordUnHandleSql(_updRecord);
		_sqlList.push(_sql);
	}
}

/**
 * 更新名片默认标志
 * @param _id
 * @param _defaultFlag
 * @returns
 */
function updateDefaultVCard(_id, _defaultFlag, _isRecord){
	var _sqlList = [];
	getUpdDefVCardAllSqls(_id, _defaultFlag, _sqlList, true)
	localdb.executeSync1({sqls: _sqlList});
}

/**
 * 设置默认名片
 * @param _changedData
 * @returns
 */
function setDefaultVCard(_changedData){
	var _sqlList = [];
	// 查询已有的默认名片信息
	var _data = query_contactCardDataByDefaultFlag("1");
	if(null != _data){
		// 获取更新默认名片标志的所有sql语句
		getUpdDefVCardAllSqls(_data.id, "0",
				_sqlList, true);
		//删除名片生日卡片sql语句
		dbbirthday.getDelBirthAllDataSql(_data.id, _sqlList);
		localdb.executeSync1({sqls: _sqlList});
		_sqlList = [];
	}
	// 获取更新默认名片标志的所有sql语句
	getUpdDefVCardAllSqls(_changedData.id, "1",
			_sqlList, true);
	// 若存在生日的时候，则在时间流里面创建生日卡片
	if(!isNullTime(_changedData.birthday)){
		// 获取插入名片生日数据的所有sql语句
		dbbirthday.getInsertBirthdayAllSqls({
			id: _changedData.id,
			commname: _changedData.name,
			commtime: _changedData.birthday,
			calendartype: _changedData.calendartype
		}, _sqlList);
		
		//时间流数据重新加载
		do_global.fire("taskDataChangeded");
		// 刷新视图（桌面小部件）
		widgetTools.refresh();
	}
	return localdb.executeSync1({sqls: _sqlList});
}

// 设置默认名片
module.exports.setDefaultVCard = function(_changedData){
	return setDefaultVCard(_changedData);
};

/**
 * 设置默认名片-数据同步
 * @param _remoteDefaultVCard
 * @returns
 */
function setDefVCardSyncData(){
	// 查询默认名片
	var _defaultVCard = query_contactCardDataByDefaultFlag('1');
	// 若没有默认名片，则设置最新创建的一张非默认名片为默认名片；
	// 否则不做处理
	if(null == _defaultVCard){
		// 查询最近创建的非默认名片信息
		var _sql = "select * from contacts_vcard " +  
		"where createtime=(select max(createtime) from contacts_vcard where defaultflag='0');";  
		var _data = localdb.querySync(_sql);
		if(_data != null || _data.length > 0){
			_cardData = _data[0];
			if(null != _cardData){
				var _sqlList = [];
				// 获取更新默认名片标志的所有sql语句
				getUpdDefVCardAllSqls(_cardData.id, "1",
						_sqlList, true);
				// 若存在生日的时候，则在时间流里面创建生日卡片
				if(!isNullTime(_cardData.birthday)){
					// 获取插入名片生日数据的所有sql语句
					dbbirthday.getInsertBirthdayAllSqls({
						id: _cardData.id,
						commname: _cardData.name,
						commtime: _cardData.birthday,
						calendartype: _cardData.calendartype
					}, _sqlList);
				}
				localdb.executeSync1({sqls: _sqlList});
			}
		}
	}
}

// 设置默认名片-数据同步
module.exports.setDefVCardSyncData = function(){
	setDefVCardSyncData();
};

/**
 * 查询指定条件（_id）下电子名片数据 
 * @param _id
 * @returns
 */
function query_contactCardDataById(_id){
	var sql ="select * from contacts_vcard where id="
		+ tools.formatSqlValue(_id) + ";";
	var _data = localdb.querySync(sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data[0];
}

//查询指定条件（_id）下电子名片数据
module.exports.query_contactCardDataById = function(_id){
	return query_contactCardDataById(_id);
};

/**
 * 查询指定条件（_id）下电子名片数据-异步查询
 * @param _id
 * @returns
 */
function queryAsyncContactCardDataById(_id, _callFunc){
	var _sql ="select * from contacts_vcard where id=?;";
	localdb.query(_sql, [_id], function(_data){
		if(_data == null || _data.length <= 0){
			_callFunc.call(this, null);
		} else {
			_callFunc.call(this, _data[0]);
		}
	});
}

//查询指定条件（_id）下电子名片数据-异步查询
module.exports.queryAsyncContactCardDataById = function(_id, _callFunc){
	queryAsyncContactCardDataById(_id, _callFunc);
};

/**
 * 查询默认标记（defaultflag）下电子名片数据
 * @param _defaultFlag
 * @returns
 */
function query_contactCardDataByDefaultFlag(_defaultFlag){
	var sql ="select * from contacts_vcard where defaultflag="
		+ tools.formatSqlValue(_defaultFlag) + ";";
	var _data = localdb.querySync(sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data[0];
}

//查询默认标记（_defaultFlag）下电子名片数据
module.exports.query_contactCardDataByDefaultFlag = function(_defaultFlag){
	return query_contactCardDataByDefaultFlag(_defaultFlag);
};

/**
 * 查询指定条件（_condition）下电子名片数据
 * @param _id
 * @returns
 */
function query_contactCardDataByCondition(_condition){
	var sql = "select " + _condition + " from contacts_vcard order by createtime DESC;";
	var _data = localdb.querySync(sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data;
}

//查询指定条件（_condition）下电子名片数据
module.exports.query_contactCardDataByCondition = function(_condition){
	return query_contactCardDataByCondition(_condition);
};

/**
 * 查询最近新建的电子名片数据
 * @returns
 */
function query_recentlyContactCardData(){
	var sql = "select * from contacts_vcard " +  
	"where  createtime=(select max(createtime) from contacts_vcard );";  
	var _data = localdb.querySync(sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data[0];
}

//查询最近新建的电子名片数据
module.exports.query_recentlyContactCardData = function(){
	return query_recentlyContactCardData();
};

/**
 * 查询所有电子名片数据
 * @returns
 */
function query_allCardData(){
	var sql = "select * from contacts_vcard order by createtime DESC;";  
	var _data = localdb.querySync(sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data;
}

//查询所有电子名片数据
module.exports.query_allCardData = function(){
	return query_allCardData();
};

/**
 * 查询大量名片数据
 * @param _ids
 * @returns
 */
function queryMoreVCardByIds(_ids){
	var _sqlTmp = "";
	for(var i=0; i<_ids.length; i++){
		if(null == _ids[i]) continue;
		if(_sqlTmp.length > 0){
			_sqlTmp += ",";
		}
		_sqlTmp += tools.formatSqlValue(_ids[i].id);
	}
	if(_sqlTmp.length > 0){
		var _sql ="select id,updatetime from contacts_vcard where id in (";
		_sql += _sqlTmp;
		_sql += ");";
		
		var _data = localdb.querySync(_sql);
		if(_data == null || 
				_data.length <= 0){
			_data = null;
		}
		return _data;
	} else {
		return null;
	}
}

// 查询大量名片数据
module.exports.queryMoreVCardByIds = function(_ids){
	return queryMoreVCardByIds(_ids);
};

/**
 * 检测名片是否存在（通过id）
 * @param _id
 * @returns
 */
function isExistVCardDataById(_id){
	var sql ="select id from contacts_vcard where id="
		+ tools.formatSqlValue(_id) + ";";
	var _data = localdb.querySync(sql);
	if(_data == null || _data.length <= 0){
		return false;
	}
	return true;
}

// 检测名片是否存在（通过id）
module.exports.isExistVCardDataById = function(_id){
	return isExistVCardDataById(_id);
};

/**
 * 检测名片是否存在（通过id）-异步查询
 * @param _id
 * @param _callFunc
 * @returns
 */
function isExistVCardDataByIdAsync(_id, _callFunc){
	var _sql ="select id from contacts_vcard where id=?;";
	localdb.query(_sql, [_id], function(_data){
		var _isExist = true;
		if(_data == null || _data.length <= 0){
			_isExist = false;
		}
		_callFunc.call(this, _isExist);
	});
}

// 检测名片是否存在（通过id）-异步查询
module.exports.isExistVCardDataByIdAsync = function(_id, _callFunc){
	isExistVCardDataByIdAsync(_id, _callFunc);
};

/**
 * 检测是否存在名片
 * @returns
 */
function isExistVCardData(){
	var sql ="select id from contacts_vcard;";
	var _data = localdb.querySync(sql);
	if(_data == null || _data.length <= 0){
		return false;
	}
	return true;
}

// 检测是否存在名片
module.exports.isExistVCardData = function(){
	return isExistVCardData();
};
