/**
 * 联系人群组
 * 
 * @Author : Cydow
 * @Timestamp : 2018-03-19
 */
//引入组件库
var deviceone = require("deviceone");
var do_storage = deviceone.sm("do_Storage");
var do_global = deviceone.sm("do_Global");

var objectId = require("objectId");      //生成objectId
var tools = require("tools");            //工具
var constants = require("constants");    //常量
var dbclsyncdata = require("dbclsyncdata");  //数据同步
var dbcontactgroupsrel = require("dbcontactgroupsrel"); //联系人群组关系

//数据库
var localdb = deviceone.mm("M0026_sqlite", "localdb", "app");

//业务模块ID
var moduleId = constants.getConstants("moduleType_contact");

/**
 * 获取插入群组数据的sql语句
 * @param _changedData
 * @returns
 */
function getInsertGroupDataSql(_changedData){
	var _nowTime = new Date();
	if (_changedData.id == null){
		_changedData.id = objectId.getObjectId();
	}
	if (_changedData.name == null){
		_changedData.name = "";
	}
	if (_changedData.morefields == null){
		_changedData.morefields = "";
	}
	if(_changedData.createtime == null){
		_changedData.createtime  = _nowTime.getTime();
	}
	if(_changedData.updatetime == null){
		_changedData.updatetime  = _nowTime.getTime();
	}
	var _sql_insert = "insert into contacts_group(id, name, morefields, " +
			"createtime, updatetime) values("
		+ tools.formatSqlValue(_changedData.id) + ","
		+ tools.formatSqlValue(_changedData.name) + ","
		+ tools.formatSqlValue(_changedData.morefields) + ","
		+ _changedData.createtime + ","
		+ _changedData.updatetime + ");";
	return _sql_insert;
}

/**
 * 获取插入群组数据的所有sql语句
 * @param _changedData
 * @param _sqlList
 * @param _isRecord
 * @returns
 */
function getInsertGroupDataAllSqls(_changedData, _sqlList, _isRecord){
	// 群组数据
	var _sql = getInsertGroupDataSql(_changedData);
	_sqlList.push(_sql);
	// 更新记录
	var _updRecord = {
			id: _changedData.id,
			moduleid: moduleId,
			recordtype: constants.getConstants("recordType_create"), 
			datatype: constants.getConstants("datatype_contactgroups")
		};
	// 设置更新记录
	if(_isRecord){
		_sql = dbclsyncdata.getUpdRecordUnHandleSql(_updRecord);
	} else {
		_sql = dbclsyncdata.getUpdRecordSql(_updRecord);
	}
	_sqlList.push(_sql);
}

/**
 * 插入群组数据
 * @param _changedData
 * @param _isRecord
 * @returns
 */
function insertGroupData(_changedData, _isRecord){
	var _sqlList = [];
	// 获取插入群组数据的所有sql语句
	getInsertGroupDataAllSqls(_changedData, _sqlList, _isRecord);
	// 执行多条sql语句
	var _isSuc = localdb.executeSync1({sqls: _sqlList});
	return _isSuc;
}

// 插入群组数据
module.exports.insertGroupData = function(_changedData){
	return insertGroupData(_changedData, true);	
};

// 插入群组数据-数据同步
module.exports.insertGroupSyncData = function(_changedData){
	return insertGroupData(_changedData, false);	
};

/**
 * 获取更新群组数据的sql语句
 * @param _changedData
 * @returns
 */
function getUpdGroupDataSql(_changedData){
	var _nowTime = new Date();
	if(null == _changedData.updatetime){
		_changedData.updatetime = _nowTime.getTime();
	}
	var _setList="";
	if (_changedData.name != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "name=" + tools.formatSqlValue(_changedData.name);
	}
	if (_changedData.morefields != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "morefields=" + tools.formatSqlValue(_changedData.morefields);
	}
	if (_changedData.updatetime != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "updatetime=" + _changedData.updatetime;
	}
	var _sql = "";
	if (_setList.length <= 0) return _sql;
	_sql = "update contacts_group set " + _setList + " where id="
		+ tools.formatSqlValue(_changedData.id) + ";";
	return _sql;
}

/**
 * 获取更新群组数据的所有sql语句
 * @param _changedData
 * @param _sqlList
 * @param _isRecord
 * @returns
 */
function getUpdGroupDataAllSqls(_changedData, _sqlList, _isRecord){
	// 获取更新群组数据的sql语句
	var _sql = getUpdGroupDataSql(_changedData);
	_sqlList.push(_sql);
	// 更新记录
	var _updRecord = {
			id: _changedData.id,
			moduleid: moduleId,
			recordtype: constants.getConstants("recordType_update"), 
			datatype: constants.getConstants("datatype_contactgroups")
		};
	// 设置更新记录
	if(_isRecord){
		_sql = dbclsyncdata.getUpdRecordUnHandleSql(_updRecord);
	} else {
		_sql = dbclsyncdata.getUpdRecordSql(_updRecord);
	}
	_sqlList.push(_sql);
}

/**
 * 更新群组数据
 * @param _changedData
 * @param _isRecord
 * @returns
 */
function updGroupData(_changedData, _isRecord){
	var _sqlList = [];
	getUpdGroupDataAllSqls(_changedData, _sqlList, _isRecord);
	var isSuc = localdb.executeSync1({sqls: _sqlList});
	return isSuc;
}

// 更新群组数据
module.exports.updGroupData = function(_changedData){
	return updGroupData(_changedData, true);	
};

// 更新群组数据-数据同步
module.exports.updGroupSyncData = function(_changedData){
	return updGroupData(_changedData, false);	
};

/**
 * 获取删除群组sql语句
 * @param _id
 * @returns
 */
function getDelGroupDataSql(_id){
	// 删除联系人群组sql语句
	var _sql = "delete from contacts_group where id=" + tools.formatSqlValue(_id) + ";";
	return _sql;
}

/**
 * 获取删除群组涉及的所有sql语句
 * @param _id
 * @param _sqlList
 * @param _isRecord
 * @returns
 */
function getDelGroupDataAllSqls(_id, _sqlList, _isRecord){
	//删除联系人群组sql语句
	var _sql = getDelGroupDataSql(_id);
	_sqlList.push(_sql);
	// 更新记录
	var _updRecord = {
			id: _id,
			moduleid: moduleId,
			recordtype: constants.getConstants("recordType_delete"), 
			datatype: constants.getConstants("datatype_contactgroups")
		};
	// 设置更新记录
	if(_isRecord){
		_sql = dbclsyncdata.getUpdRecordUnHandleSql(_updRecord);
	} else {
		_sql = dbclsyncdata.getUpdRecordSql(_updRecord);
	}
	_sqlList.push(_sql);
}

/**
 * 删除群组数据
 * @param _changedData
 * @returns
 */
function delGroupData(_changedData){
	var _sqlList = [];
	var _dataList = _changedData.dataList;
	if(!tools.isNullOrEmpty(_dataList)){
		for(var i=0; i<_dataList.length; i++){
			var _oneData = _dataList[i];
			if(null == _oneData) continue;
			// 获取删除群组数据的全部sql语句
			getDelGroupDataAllSqls(_oneData.id, _sqlList, true);
			// 获取删除群组关系的全部sql语句
			dbcontactgroupsrel.getDelGroupRelAllSqlsByGroupId(_oneData.id,
					_sqlList, true);
		}
	}
	//执行sql语句，并返回
	if(!tools.isNullOrEmpty(_sqlList)){
		localdb.execute1({sqls: _sqlList}, function(data){
			_changedData.callFunc.call(this);
		});
	} else {
		_changedData.callFunc.call(this);
	}
}

//删除群组数据
module.exports.delGroupData = function(_changedData){
	delGroupData(_changedData);
};

/**
 * 删除群组数据--数据同步
 * @param _id
 * @param _callFunc
 * @returns
 */
function delGroupSyncData(_id, _callFunc){
	var _sqlList = [];
	// 获取删除群组数据的全部sql语句
	getDelGroupDataAllSqls(_id, _sqlList, true);
	// 获取删除群组关系的全部sql语句
	dbcontactgroupsrel.getDelGroupRelAllSqlsByGroupId(_id,
			_sqlList, true);
	localdb.execute1({sqls: _sqlList}, function(_result){
		var _isSuc = false;
		if(_result > 0){
			_isSuc = true;
		}
		// 回调函数
		_callFunc.call(this, _isSuc);
	});
}

// 删除群组数据--数据同步
module.exports.delGroupSyncData = function(_id, _callFunc){
	return delGroupSyncData(_id, _callFunc);	
};

/**
 * 查询全部联系人群组数据
 * @param _id
 * @returns
 */
function query_groupsAllData(){
	var _sql = "select contacts_group.id as groupid, name, contacts_group_relation.id as id, " +
			"contacts_group_relation.removed, contacts_group_relation.contactid " +
			"from contacts_group left join contacts_group_relation on " +
			"contacts_group.id = contacts_group_relation.groupid;";
	var _data = localdb.querySync(_sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data;
}

//查询联系人群组数据
module.exports.query_groupsAllData = function(){
	return query_groupsAllData();
};

/**
 * 查询指定条件（_id）下联系人群组数据
 * @param _id
 * @returns
 */
function query_groupsDataById(_id){
	var _sql = "select * from contacts_group where id="
		+ tools.formatSqlValue(_id) + ";";
	var _data = localdb.querySync(_sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data[0];
}

//查询指定条件（_id）下联系人群组数据
module.exports.query_groupsDataById = function(_id){
	return query_groupsDataById(_id);
};

/**
 * 查询指定条件（_contactId）下联系人群组数据
 * @param _contactId
 * @returns
 */
function query_groupsDataByContactId(_contactId){
	var _sql = "select * from contacts_group where id in (select groupid from contacts_group_relation where contactid=" + 
		tools.formatSqlValue(_contactId) + " and removed=0);";
	var _data = localdb.querySync(_sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data;
}

//查询指定条件（_contactId）下联系人群组数据
module.exports.query_groupsDataByContactId = function(_contactId){
	return query_groupsDataByContactId(_contactId);
};

/**
 * 查询指定条件（_condition）联系人群组数据
 * @param _id
 * @returns
 */
function query_groupsDataByCondition(_condition){
	var _sql = "select " + _condition + " from contacts_group;";
	var _data = localdb.querySync(_sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data;
}

//查询指定条件（_condition）联系人群组数据
module.exports.query_groupsDataByCondition = function(_condition){
	return query_groupsDataByCondition(_condition);
};

/**
 * 查询指定条件（群组名称）下联系人群组数据，用于群组重名判断
 * @param _name
 * @returns
 */
function query_groupsDataByName(_name){
	var _sql = "select id from contacts_group where name=" 
		+ tools.formatSqlValue(_name); 
	var _data = localdb.querySync(_sql);
	if(tools.isNullOrEmpty(_data)){
		return null;
	}
	return _data[0];
}

//查询指定条件（群组名称）下联系人群组数据，用于群组重名判断
module.exports.query_groupsDataByName = function(_name){
	return query_groupsDataByName(_name);
};

/**
 * 查询联系人群组数量
 * @returns
 */
function query_groupNums(){
	var _sql = "select count(*) as num from contacts_group;";
	var _data = localdb.querySync(_sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data[0].num;
}

//查询联系人群组数量
module.exports.query_groupNums = function(){
	return query_groupNums();
};

/**
 * 检测群组是否存在（通过id）
 * @param _id
 * @returns
 */
function isExistGroupDataById(_id){
	var _sql = "select id from contacts_group where id="
		+ tools.formatSqlValue(_id) + ";";
	var _data = localdb.querySync(_sql);
	if(_data == null || _data.length <= 0){
		return false;
	}
	return true;
}

// 检测群组是否存在（通过id）
module.exports.isExistGroupDataById = function(_id){
	return isExistGroupDataById(_id);
};
