/**
 * 联系人群组关系表
 * 
 * @Author : Cydow
 * @Timestamp : 2018-03-19
 */
//引入组件库
var deviceone = require("deviceone");
var do_storage = deviceone.sm("do_Storage");
var do_global = deviceone.sm("do_Global");

var objectId = require("objectId");      //生成objectId
var tools = require("tools");            //工具
var constants = require("constants");    //常量
var dbclsyncdata = require("dbclsyncdata");  //数据同步

//数据库
var localdb = deviceone.mm("M0026_sqlite", "localdb", "app");

//业务模块ID
var moduleId = constants.getConstants("moduleType_contact");

/**
 * 获取插入一条群组关系的sql语句
 * @param _changedData
 * @returns
 */
function getInsertOneGroupRelSql(_changedData){
	var _nowTime = new Date();
	if (_changedData.id == null){
		_changedData.id = objectId.getObjectId();
	}
	if(_changedData.removed == null){
		_changedData.removed = 0;
	}
	if(_changedData.updatetime == null){
		_changedData.updatetime  = _nowTime.getTime();
	}
	var _sql_insert = "insert into contacts_group_relation(id, groupid, contactid, removed, " +
			"updatetime) values("
		+ tools.formatSqlValue(_changedData.id) + ","
		+ tools.formatSqlValue(_changedData.groupid) + ","
		+ tools.formatSqlValue(_changedData.contactid) + ","
		+ _changedData.removed + ","
		+ _changedData.updatetime + ");";
	return _sql_insert;
}

/**
 * 获取插入一条群组关系的所有sql语句
 * @param _changedData
 * @param _sqlList
 * @param _isRecord
 * @returns
 */
function getInsertOneGroupRelAllSqls(_changedData, _sqlList, _isRecord){
	// 群组关系
	var _sql = getInsertOneGroupRelSql(_changedData);
	_sqlList.push(_sql);
	
	var _updRecord = {
			id: _changedData.id,
			moduleid: moduleId,
			recordtype: constants.getConstants("recordType_create"), 
			datatype: constants.getConstants("datatype_groupsrelation")
		};
	// 设置更新记录
	if(_isRecord){
		_sql = dbclsyncdata.getUpdRecordUnHandleSql(_updRecord);
	} else {
		_sql = dbclsyncdata.getUpdRecordSql(_updRecord);
	}
	_sqlList.push(_sql);
}

/**
 * 插入一条联系人群组关系数据
 * @param _changedData
 * @param _isRecord
 */
function insertOneGroupsRelData(_changedData, _isRecord){
	var _sqlList = [];
	getInsertOneGroupRelAllSqls(_changedData, _sqlList, _isRecord);
	var isSuccess = localdb.executeSync1({sqls: _sqlList});
	return isSuccess;
}

/**
 * 插入群组关系数据
 * @param _changedData
 */
function insertGroupRelData(_changedData){
	if(!tools.isNull(_changedData)){
		var _id = _changedData.id;
		var _dataList = _changedData.dataList;
		if(!tools.isNullOrEmpty(_dataList)){
			for(var i=0; i<_dataList.length; i++){
				var _data = _dataList[i];
				if(tools.isNullOrEmpty(_data)) continue;
				insertOneGroupsRelData({
					groupid: _id,
					contactid: _data
				}, true);
			}
		}
	}
}

// 插入群组关系数据
module.exports.insertGroupRelData = function(_changedData){
	insertGroupRelData(_changedData);	
};

// 插入群组关系数据--数据同步
module.exports.insertGroupsRelSyncData = function(_changedData){
	return insertOneGroupsRelData(_changedData, false);	
};

/**
 * 获取更新一条群组关系的sql语句
 * @param _changedData
 * @returns
 */
function getUpdOneGroupRelSql(_changedData){
	var _nowTime = new Date();
	if(null == _changedData.updatetime){
		_changedData.updatetime = _nowTime.getTime();
	}
	var _setList="";
	if (_changedData.id != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "id=" + tools.formatSqlValue(_changedData.id);
	}
	if (_changedData.removed != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "removed=" + _changedData.removed;
	}
	if (_setList.length > 0) _setList+=",";
	_setList += "updatetime=" + _changedData.updatetime;
	if (_setList.length <= 0) return;
	var _sql_update = "update contacts_group_relation set " + _setList 
		+ " where contactid=" + tools.formatSqlValue(_changedData.contactid)
		+ " and groupid=" + tools.formatSqlValue(_changedData.groupid) + ";";
	return _sql_update;
}

/**
 * 获取更新一条群组关系的所有sql语句
 * @param _changedData
 * @param _sqlList
 * @param _isRecord
 * @returns
 */
function getUpdOneGroupRelAllSqls(_changedData, _sqlList, _isRecord){
	// 群组关系
	var _sql = getUpdOneGroupRelSql(_changedData);
	_sqlList.push(_sql);
	
	var _updRecord = {
			id: _changedData.id,
			moduleid: moduleId,
			recordtype: constants.getConstants("recordType_update"), 
			datatype: constants.getConstants("datatype_groupsrelation")
		};
	// 设置更新记录
	if(_isRecord){
		_sql = dbclsyncdata.getUpdRecordUnHandleSql(_updRecord);
	} else {
		_sql = dbclsyncdata.getUpdRecordSql(_updRecord);
	}
	_sqlList.push(_sql);
}

/**
 * 更新一条群组关系数据
 * @param _changedData
 * @param _isRecord
 * @returns
 */
function updOneGroupRelData(_changedData, _isRecord){
	var _sqlList = [];
	getUpdOneGroupRelAllSqls(_changedData, _sqlList, _isRecord);
	var _isSuc = localdb.executeSync1({sqls: _sqlList});
	return _isSuc;
}

// 更新一条群组关系数据
module.exports.updOneGroupRelData = function(_changedData){
	updOneGroupRelData(_changedData, true);	
};

// 更新一条群组关系数据-数据同步
module.exports.updGroupRelSyncData = function(_changedData){
	return updOneGroupRelData(_changedData, false);	
};

/**
 * 获取更新群组关系的所有sql语句-指定联系人
 * @param _changedData
 * @param _sqlList
 * @param _isRecord
 * @returns
 */
function getUpdGroupRelAllSqlsByContactId(_changedData, _sqlList, _isRecord){
	print("getUpdGroupRelAllSqlsByContactId------: " + JSON.stringify(_changedData));
	var _id = _changedData.id;
	var _groupList = _changedData.groupList;
	// 移出旧的群组成员
	var _delGroupIdList = _groupList.delGroupIdList;
	if(!tools.isNullOrEmpty(_delGroupIdList)){
		for(var i=0; i<_delGroupIdList.length; i++){
			var _data = _delGroupIdList[i];
			if(tools.isNullOrEmpty(_data)) continue;
			var _groupRelData = queryGroupRelByGroupAndContactId(_data, _id);
			if(!tools.isNull(_groupRelData)){
				getUpdOneGroupRelAllSqls({
					id: _groupRelData.id,
					groupid: _data,
					contactid: _id,
					removed: 1
				}, _sqlList, _isRecord);
			}
		}
	}
	// 添加新的联系人群组
	var _addGroupIdList = _groupList.addGroupIdList;
	if(!tools.isNullOrEmpty(_addGroupIdList)){
		for(var j=0; j<_addGroupIdList.length; j++){
			var _data = _addGroupIdList[j];
			if(tools.isNullOrEmpty(_data)) continue;
			var _groupRelData = queryGroupRelByGroupAndContactId(_data, _id);
			if(!tools.isNull(_groupRelData)){
				getUpdOneGroupRelAllSqls({
					id: _groupRelData.id,
					groupid: _data,
					contactid: _id,
					removed: 0
				}, _sqlList, _isRecord);
			} else {
				getInsertOneGroupRelAllSqls({
					groupid: _data,
					contactid: _id
				}, _sqlList, _isRecord);
			}
		}
	}
}

// 获取更新群组关系的所有sql语句-指定联系人
module.exports.getUpdGroupRelAllSqlsByContactId = function(_changedData, _sqlList, _isRecord){
	getUpdGroupRelAllSqlsByContactId(_changedData, _sqlList, _isRecord);	
};

/**
 * 更新群组关系数据
 * @param _changedData
 * @param _isRecord
 */
function updGroupRelDataByContactId(_changedData, _isRecord){
	print("updGroupRelDataByContactId------: " + JSON.stringify(_changedData));
	var _sqlList = [];
	getUpdGroupRelAllSqlsByContactId(_changedData, _sqlList, _isRecord);
	var isSuccess = localdb.executeSync1({sqls: _sqlList});
	return isSuccess;
}

// 更新群组关系数据-指定联系人
module.exports.updGroupRelDataByContactId = function(_changedData, _isRecord){
	updGroupRelDataByContactId(_changedData, _isRecord);	
};

/**
 * 获取更新群组关系的所有sql语句-指定群组
 * @param _changedData
 * @param _sqlList
 * @param _isRecord
 * @returns
 */
function getUpdGroupRelAllSqlsByGroupId(_changedData, _sqlList, _isRecord){
	print("getUpdGroupRelAllSqlsByGroupId------: " + JSON.stringify(_changedData));
	if(!tools.isNull(_changedData)){
		var _id = _changedData.id;
		// 移出旧的群组成员
		var _delGroupMemIdList = _changedData.delGroupMemIdList;
		if(!tools.isNullOrEmpty(_delGroupMemIdList)){
			for(var i=0; i<_delGroupMemIdList.length; i++){
				var _data = _delGroupMemIdList[i];
				if(tools.isNullOrEmpty(_data)) continue;
				var _groupRelData = queryGroupRelByGroupAndContactId(_id, _data);
				if(!tools.isNull(_groupRelData)){
					getUpdOneGroupRelAllSqls({
						id: _groupRelData.id,
						groupid: _id,
						contactid: _data,
						removed: 1
					}, _sqlList, _isRecord);
				}
			}
		}
		// 添加新的联系人群组
		var _addGroupMemIdList = _changedData.addGroupMemIdList;
		if(!tools.isNullOrEmpty(_addGroupMemIdList)){
			for(var j=0; j<_addGroupMemIdList.length; j++){
				var _data = _addGroupMemIdList[j];
				if(tools.isNullOrEmpty(_data)) continue;
				var _groupRelData = queryGroupRelByGroupAndContactId(_id, _data);
				if(!tools.isNull(_groupRelData)){
					getUpdOneGroupRelAllSqls({
						id: _groupRelData.id,
						groupid: _id,
						contactid: _data,
						removed: 0
					}, _sqlList, _isRecord);
				} else {
					getInsertOneGroupRelAllSqls({
						groupid: _id,
						contactid: _data
					}, _sqlList, _isRecord);
				}
			}
		}
	}
}

// 获取更新群组关系的所有sql语句-指定联系人-指定群组
module.exports.getUpdGroupRelAllSqlsByGroupId = function(_changedData, _sqlList, _isRecord){
	getUpdGroupRelAllSqlsByGroupId(_changedData, _sqlList, _isRecord);	
};

/**
 * 更新群组关系数据-指定群组
 * @param _changedData
 * @param _isRecord
 * @returns
 */
function updGroupRelDataByGroupId(_changedData, _isRecord){
	deviceone.print("updGroupRelDataByGroupId------: " + JSON.stringify(_changedData));
	var _sqlList = [];
	getUpdGroupRelAllSqlsByGroupId(_changedData, _sqlList, _isRecord);
	var isSuccess = localdb.executeSync1({sqls: _sqlList});
	return isSuccess;
}

// 更新群组关系数据-指定群组
module.exports.updGroupRelDataByGroupId = function(_changedData, _isRecord){
	updGroupRelDataByGroupId(_changedData, _isRecord);	
};

/**
 * 获取删除群组关系sql语句
 * @param _id
 * @returns
 */
function getDelGroupRelDataSql(_id){
	//删除群组关系sql语句
	var _sql = "delete from contacts_group_relation where id=" 
		+ tools.formatSqlValue(_id) + ";";
	return _sql;
}

/**
 * 获取删除群组关系的所有sql语句
 * @param _id
 * @param _sqlList
 * @param _isSync
 * @param _isRecord
 * @returns
 */
function getDelGroupRelAllSqls(_id, _sqlList, _isSync, _isRecord){
	//删除联系人群组关系sql语句
	var _sql = getDelGroupRelDataSql(_id);
	_sqlList.push(_sql);
	
	if(_isRecord){
		// 更新记录
		var _updRecord = {
				id: _id,
				moduleid: moduleId,
				recordtype: constants.getConstants("recordType_delete"), 
				datatype: constants.getConstants("datatype_groupsrelation")
			};
		// 设置更新记录
		if(_isSync){
			_sql = dbclsyncdata.getUpdRecordUnHandleSql(_updRecord);
		} else {
			_sql = dbclsyncdata.getUpdRecordSql(_updRecord);
		}
		_sqlList.push(_sql);
	}
}

/**
 * 删除群组关系
 * @param _id
 * @param _isSync
 * @returns
 */
function delGroupRelData(_id, _isSync){
	var _sqlList = [];
	getDelGroupRelAllSqls(_id, _sqlList, _isSync, true);
	return localdb.executeSync1({sqls: _sqlList});
}

// 删除群组关系数据
module.exports.delGroupRelData = function(_id){
	return delGroupRelData(_id, true);
};

// 删除群组关系数据-数据同步
module.exports.delGroupRelSyncData = function(_id){
	return delGroupRelData(_id, false);
};

/**
 * 获取删除群组关系的所有sql语句-指定群组
 * @param _groupId
 * @param _sqlList
 * @param _isSync
 * @returns
 */
function getDelGroupRelAllSqlsByGroupId(_groupId, _sqlList, _isSync){
	// 查询指定群组的所有群组关系id标识
	var _sql = "select id from contacts_group_relation where groupid="
		+ tools.formatSqlValue(_groupId) + ";";
	var _groupRelList = localdb.querySync(_sql);
	
	// 遍历所有群组关系数据
	if(null != _groupRelList &&
			_groupRelList.length > 0){
		for(var i=0; i<_groupRelList.length; i++){
			if(null == _groupRelList[i]) continue;
			getDelGroupRelAllSqls(_groupRelList[i].id,
					_sqlList, _isSync, true);
		}
	}
}

// 获取删除群组关系的全部sql语句-指定群组
module.exports.getDelGroupRelAllSqlsByGroupId = function(_groupId, _sqlList, _isRecord){
	getDelGroupRelAllSqlsByGroupId(_groupId, _sqlList, _isRecord);
};

/**
 * 删除群组关系-指定群组
 * @param _groupId
 * @param _isRecord
 * @returns
 */
function delGroupRelByGroupId(_groupId, _isRecord){
	var _sqlList = [];
	getDelGroupRelAllSqlsByGroupId(_groupId, _sqlList, _isRecord);
	return localdb.executeSync1({sqls: _sqlList});
}

// 删除指定联系人群组（_groupId）关系数据
module.exports.delGroupRelByGroupId = function(_groupId){
	return delGroupRelByGroupId(_groupId, true);
};

/**
 * 获取删除群组关系的所有sql语句-指定联系人
 * @param _contactId
 * @param _sqlList
 * @param _isSync
 * @param _isRecord
 * @returns
 */
function getDelGroupRelAllSqlsByContactId(_contactId, _sqlList, _isSync, _isRecord){
	// 查询指定群组的所有群组关系id标识
	var _sql = "select id from contacts_group_relation where contactid="
		+ tools.formatSqlValue(_contactId) + ";";
	var _groupRelList = localdb.querySync(_sql);
	
	// 遍历所有群组关系数据
	if(null != _groupRelList &&
			_groupRelList.length > 0){
		for(var i=0; i<_groupRelList.length; i++){
			if(null == _groupRelList[i]) continue;
			getDelGroupRelAllSqls(_groupRelList[i].id, 
					_sqlList, _isSync, _isRecord);
		}
	}
}

// 获取删除群组关系的全部sql语句-指定联系人
module.exports.getDelGroupRelAllSqlsByContactId = function(_contactId, _sqlList, _isRecord){
	getDelGroupRelAllSqlsByContactId(_contactId, _sqlList, _isRecord, true);
};

// 获取删除群组关系的全部sql语句-数据同步
module.exports.getDelGroupRelByContactIdSync = function(_contactId, _sqlList){
	getDelGroupRelAllSqlsByContactId(_contactId, _sqlList, true, false);
};

/**
 * 删除群组关系-指定联系人
 * @param _contactId
 * @param _isRecord
 * @returns
 */
function delGroupRelByContactId(_contactId, _isRecord){
	var _sqlList = [];
	getDelGroupRelAllSqlsByContactId(_contactId, _sqlList, _isRecord, true);
	return localdb.executeSync1({sqls: _sqlList});
}

// 删除群组关系-指定联系人
module.exports.delGroupRelByContactId = function(_contactId){
	return delGroupRelByContactId(_contactId, true);
};

//删除群组关系（指定联系人）-数据同步
module.exports.delGroupRelByContactId = function(_contactId){
	return delGroupRelByContactId(_contactId, false);
};

/**
 * 查询指定条件（_id）下联系人群组关系数据
 * @param _id
 * @returns
 */
function queryGroupRelDataById(_id){
	var _sql = "select * from contacts_group_relation where id="
		+ tools.formatSqlValue(_id) + ";";
	var _data = localdb.querySync(_sql);
	if(tools.isNullOrEmpty(_data)){
		return null;
	}
	return _data[0];
}

//查询指定条件（_id）下联系人群组关系数据
module.exports.queryGroupRelDataById = function(_id){
	return queryGroupRelDataById(_id);
};

/**
 * 查询指定群组（groupid）的群组关系数据
 * @param _groupId
 * @returns
 */
function query_groupsRelDataByGroupId(_groupId){
	var _sql = "select * from contacts_group_relation where groupid="
		+ tools.formatSqlValue(_groupId) + " and removed=0;";
	var _data = localdb.querySync(_sql);
	if(tools.isNullOrEmpty(_data)){
		return null;
	}
	return _data;
}

//查询指定群组（groupid）的群组关系数据
module.exports.query_groupsRelDataByGroupId = function(_groupId){
	return query_groupsRelDataByGroupId(_groupId);
};

/**
 * 查询指定联系人（_contactId）的群组关系数据
 * @param _contactId
 * @returns
 */
function query_groupsRelDataByContactId(_contactId){
	var _sql = "select * from contacts_group_relation where contactid="
		+ tools.formatSqlValue(_contactId) + " and removed=0;";
	var _data = localdb.querySync(_sql);
	if(tools.isNullOrEmpty(_data)){
		return null;
	}
	return _data;
}

// 查询指定联系人（_contactId）的群组关系数据
module.exports.query_groupsRelDataByContactId = function(_contactId){
	return query_groupsRelDataByContactId(_contactId);
};

/**
 * 查询指定联系人（_groupId、_contactId）的群组关系数据
 * @param _groupId
 * @param _contactId
 * @returns
 */
function queryGroupRelByGroupAndContactId(_groupId, _contactId){
	var _sql = "select * from contacts_group_relation where groupid="
		+ tools.formatSqlValue(_groupId) + " and contactid="
		+ tools.formatSqlValue(_contactId) + ";";
	var _data = localdb.querySync(_sql);
	if(tools.isNullOrEmpty(_data)){
		return null;
	}
	return _data[0];
}

// 查询指定联系人（_groupId、_contactId）的群组关系数据
module.exports.queryGroupRelByGroupAndContactId = function(_groupId, _contactId){
	return queryGroupRelByGroupAndContactId(_groupId, _contactId);
};

/**
 * 查询指定条件（_condition）的群组关系数据
 * @param _condition
 * @returns
 */
function queryGroupRelByCondition(_condition){
	// 若参数为空，则直接返回空
	if(tools.isNullOrEmpty(_condition)) return null;
	// 查询群组关系sql语句
	var _sql = "select * from contacts_group_relation where ";
	// 拼接查询sql语句
	for(var i=0; i<_condition.length; i++){
		var _oneData = _condition[i];
		if(tools.isNull(_oneData)) continue;
		if(i > 0){
			_sql += " and ";
		}
		_sql += _oneData.field;
		_sql += "=";
		_sql += tools.formatSqlValue(_oneData.value);
	}
	_sql += ";";
	// 执行查询语句
	var _data = localdb.querySync(_sql);
	// 若查询结果为空或没有值，则返回空
	if(tools.isNullOrEmpty(_data)){
		return null;
	}
	return _data[0];
}

// 查询指定条件（_condition）的群组关系数据
module.exports.queryGroupRelByCondition = function(_condition){
	return queryGroupRelByCondition(_condition);
};
