/**
 * 联系人导入导出记录
 * 
 * @Author : Cydow
 * @Timestamp : 2017-08-09
 */
//引入组件库
var deviceone = require("deviceone");

var tools = require("tools");            //工具
var constants = require("constants");    //常量

//数据库
var localdb = deviceone.mm("M0026_sqlite", "localdb", "app");

//-----------------------------------------------------------
//导出记录
/**
 * 向contacts_impexprecord表中插入一条数据
 * @param _changedData
 */
function insert_contactExportData(_changedData){
	var _nowTime = new Date();
	if(_changedData.createtime == null){
		_changedData.createtime  = _nowTime.getTime();
	}
	if(_changedData.updatetime == null){
		_changedData.updatetime  = _nowTime.getTime();
	}
	var _sql_insert = "insert into contacts_impexprecord(contactid, phonebookid, " +
			"type, createtime, updatetime)values("
		+ tools.formatSqlValue(_changedData.contactid) + ","
		+ tools.formatSqlValue(_changedData.phonebookid) + ","
		+ tools.formatSqlValue(_changedData.type) + ","
		+ _changedData.createtime + ","
		+ _changedData.updatetime + ");";
	var isSuccess = localdb.executeSync(_sql_insert);
	return isSuccess;
}

//向contacts_impexprecord表中插入一条数据
module.exports.insert_contactExportData = function(_changedData){
	insert_contactExportData(_changedData);	
};

/**
 * 获取更新导出联系人记录sql语句
 * @param _changedData
 */
function getUpdExportContactSql(_changedData){
	var _nowTime = new Date();
	var _setList="";
	if (_changedData.type != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "type=" + _changedData.type;
	}
	if (_changedData.updatetime != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "updatetime=" + _changedData.updatetime;
	}
	if (_setList.length <= 0) return "";
	var _sql_update = "update contacts_impexprecord set " + _setList + " where contactid="
		+ tools.formatSqlValue(_changedData.contactid) + ";";
	return _sql_update;
}

// 获取更新导出联系人记录sql语句
module.exports.getUpdExportContactSql = function(_changedData){
	return getUpdExportContactSql(_changedData);	
};

/**
 * 更新contacts_impexprecord表中的数据
 * @param _changedData
 */
function update_contactExportData(_changedData){
	var _nowTime = new Date();
	var _setList="";
	if (_changedData.type != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "type=" + _changedData.type;
	}
	if (_changedData.updatetime != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "updatetime=" + _changedData.updatetime;
	}
	if (_setList.length <= 0) return;
	var _sql_update = "update contacts_impexprecord set " + _setList + " where contactid="
		+ tools.formatSqlValue(_changedData.contactid) + ";";
	var isSuccess = localdb.executeSync(_sql_update);
	return isSuccess;
}

// 更新contacts_impexprecord表中的数据
module.exports.update_contactExportData = function(_changedData){
	return update_contactExportData(_changedData);	
};

/**
 * 查询指定条件（_id）下导出记录数据
 * @param _id
 * @returns
 */
function query_contactExportDataById(_id){
	var sql ="select * from contacts_impexprecord where contactid="
		+ tools.formatSqlValue(_id) + ";";
	var _data = localdb.querySync(sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data[0];
}

//查询指定条件（_id）下导出记录数据
module.exports.query_contactExportDataById = function(_id){
	return query_contactExportDataById(_id);
};

/**
 * 查询指定条件（type）下导出记录数据
 * @param _type
 * @returns
 */
function query_contactExportDataByType(_type){
	var sql ="select * from contacts_impexprecord where type="
		+ tools.formatSqlValue(_type) + ";";
	var _data = localdb.querySync(sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data;
}

//查询指定条件（type）下导出记录数据
module.exports.query_contactExportDataByType = function(_type){
	return query_contactExportDataByType(_type);
};

//----------------------------------------------------
//导入记录

/**
 * 获取联系人导入记录sql语句
 * @param _changedData
 */
function getImportContactSql(_changedData){
	var _nowTime = new Date();
	if(_changedData.createtime == null){
		_changedData.createtime  = _nowTime.getTime();
	}
	if(_changedData.updatetime == null){
		_changedData.updatetime  = _nowTime.getTime();
	}
	var _sql_insert = "insert into contacts_impexprecord(contactid, phonebookid, " +
			"type, createtime, updatetime)values("
		+ tools.formatSqlValue(_changedData.contactid) + ","
		+ tools.formatSqlValue(_changedData.phonebookid) + ","
		+ tools.formatSqlValue(_changedData.type) + ","
		+ _changedData.createtime + ","
		+ _changedData.updatetime + ");";
	return _sql_insert;
}

// 获取联系人导入记录sql语句
module.exports.getImportContactSql = function(_changedData){
	return getImportContactSql(_changedData);	
};

/**
 * 向contacts_impexprecord表中插入一条数据
 * @param _changedData
 */
function insert_contactImportData(_changedData){
	var _sql_insert = getImportContactSql(_changedData);
	var isSuccess = localdb.executeSync(_sql_insert);
	return isSuccess;
}

//向contacts_impexprecord表中插入一条数据
module.exports.insert_contactImportData = function(_changedData){
	insert_contactImportData(_changedData);	
};

/**
 * 更新contacts_impexprecord表中的数据
 * @param _changedData
 */
function update_contactImportData(_changedData){
	var _nowTime = new Date();
	var _setList="";
	if (_changedData.outflag!=null){
		if (_setList.length > 0) _setList+=",";
		_setList += "type=" + _changedData.type;
	}
	if (_changedData.updatetime!=null){
		if (_setList.length > 0) _setList+=",";
		_setList += "updatetime=" + _changedData.updatetime;
	}
	if (_setList.length <= 0) return;
	var _sql_update = "update contacts_impexprecord set " + _setList + " where phonebookid="
		+ tools.formatSqlValue(_changedData.phonebookid) + ";";
	var isSuccess = localdb.executeSync(_sql_update);
	return isSuccess;
}

//更新导入记录数据
module.exports.update_contactImportData = function(_changedData){
	return update_contactImportData(_changedData);	
};

/**
 * 获取更新联系人记录sql语句
 * @param _changedData
 */
function getUpdPhoneBookIdSql(_changedData){
	var _sql_update = "update contacts_impexprecord set phonebookid=" + 
		tools.formatSqlValue(_changedData.phonebookid) + 
		" where contactid=" + tools.formatSqlValue(_changedData.contactid) + ";";
	return _sql_update;
}

// 获取更新联系人记录sql语句
module.exports.getUpdPhoneBookIdSql = function(_changedData){
	return getUpdPhoneBookIdSql(_changedData);	
};

/**
 * 获取删除联系人导入导出记录的sql语句
 * @param _id
 * @returns
 */
function getDelContactImExportDataSql(_id){
	var _sql_del = "delete from contacts_impexprecord where contactid="
		+ tools.formatSqlValue(_id) + ";";
	return _sql_del;
}

//获取删除联系人导入导出记录的sql语句
module.exports.getDelContactImExportDataSql = function(_id){
	return getDelContactImExportDataSql(_id);	
};

/**
 * 删除contacts_impexprecord表中指定id的数据
 * @param _id
 */
function delete_contactImExportData(_id){
	var _sql_del = getDelContactImExportDataSql(_id);
	var isSuccess = localdb.executeSync(_sql_del);
	return isSuccess;
}

//删除contacts_impexprecord表中指定id的数据
module.exports.delete_contactImExportData = function(_id){
	return delete_contactImExportData(_id);	
};

/**
 * 删除联系人导入导出记录表中指定phonebookid的记录（批量）
 * @param _ids
 */
function delete_contactImExportRecords(_ids){
	var _sql_del = "delete from contacts_impexprecord where phonebookid in (" + _ids + ");";
	var isSuccess = localdb.executeSync(_sql_del);
	return isSuccess;
}

// 删除联系人导入导出记录表中指定phonebookid的记录（批量）
module.exports.delete_contactImExportRecords = function(_ids){
	return delete_contactImExportRecords(_ids);
};

/**
 * 查询指定条件（_id）下导入记录数据
 * @param _id
 * @returns
 */
function query_contactImportDataById(_id){
	var sql ="select * from contacts_impexprecord where phonebookid="
		+ tools.formatSqlValue(_id) + ";";
	var _data = localdb.querySync(sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data[0];
}

//查询指定条件（_id）下导入记录数据
module.exports.query_contactImportDataById = function(_id){
	return query_contactImportDataById(_id);
};

/**
 * 查询指定条件（type）下导入导出记录数据
 * @param _type
 * @returns
 */
function query_contactImExportDataByType(_type){
	var sql ="select * from contacts_impexprecord where type="
		+ tools.formatSqlValue(_type) + ";";
	var _data = localdb.querySync(sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data;
}

//查询指定条件（type）下导入导出记录数据
module.exports.query_contactImExportDataByType = function(_type){
	return query_contactImExportDataByType(_type);
};

/**
 * 查询全部导入导出记录数据
 * @returns
 */
function query_contactAllImExportData(){
	var sql ="select * from contacts_impexprecord;";
	var _data = localdb.querySync(sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data;
}

//查询全部导入导出记录数据
module.exports.query_contactAllImExportData = function(){
	return query_contactAllImExportData();
};