/**
 * 联系人
 * 
 * @Author : Cydow
 * @Timestamp : 2017-08-09
 */
//引入组件库
var deviceone = require("deviceone");
var do_storage = deviceone.sm("do_Storage");
var do_global = deviceone.sm("do_Global");

var objectId = require("objectId");      //生成objectId
var tools = require("tools");            //工具
var constants = require("constants");    //常量
var chinesePinYin = require("chinesePinYin"); // 拼音
var typeconstants = require("typeconstants.ui"); //信息类型常量
var dbclsyncdata = require("dbclsyncdata");  //数据同步
var dbcontactimexport = require("dbcontactimexport"); //联系人导入导出记录
var dbcontactproperties = require("dbcontactproperties"); //联系人属性值
var dbbirthday = require("dbbirthday"); //生日
var dbcontactgroupsrel = require("dbcontactgroupsrel"); //联系人群组关系

//小部件工具
var widgetTools = require("widgetTools.ui");

//数据库
var localdb = deviceone.mm("M0026_sqlite", "localdb", "app");

//业务模块ID
var moduleId = constants.getConstants("moduleType_contact");

/**
 * 时间是否为空
 * @param _time
 * @returns
 */
function isNullTime(_time){
	if(null == _time || 
			('string' == (typeof _time) && "" == _time)){
		return true;
	}
	return false;
}

/**
 * 获取插入联系人数据的sql语句
 * @param _changedData
 * @returns
 */
function getInsertContactDataSql(_changedData){
	var _nowTime = new Date();
	if (_changedData.id == null){
		_changedData.id = objectId.getObjectId();
	}
	if (_changedData.infoid == null){
		_changedData.infoid = "";
	}
	if (_changedData.type == null){
		_changedData.type = typeconstants.typeConstants["contactType_common"];
	}
	if(_changedData.headurl == null){
		_changedData.headurl = "";
	}
	if(_changedData.name == null){
		_changedData.name  = "";
	}
	if(_changedData.gender == null){
		_changedData.gender  = "";
	}
	if(_changedData.company == null){
		_changedData.company  = "";
	}
	if(_changedData.department == null){
		_changedData.department  = "";
	}
	if(_changedData.interests == null){
		_changedData.interests  = "";
	}
	if(_changedData.meetoccasion == null){
		_changedData.meetoccasion  = "";
	}
	if(_changedData.remarks == null){
		_changedData.remarks  = "";
	}
	if(_changedData.origin == null){
		_changedData.origin  = "";
	}
	if(_changedData.presrc == null){
		_changedData.presrc  = "";
	}
	if(_changedData.morefields == null){
		_changedData.morefields  = "";
	}
	if(_changedData.createtime == null || 
			_changedData.createtime == ""){
		_changedData.createtime  = _nowTime.getTime();
	}
	if(_changedData.updatetime == null ||
			_changedData.updatetime == ""){
		_changedData.updatetime  = _nowTime.getTime();
	}
	var _pinyin = "";
	var _birthday = null;
	var _calendarType = null;
	var _sql_insert = "";
	if(typeconstants.typeConstants["contactType_children"] == _changedData.type){
		// 姓名拼音
		_pinyin = chinesePinYin.GetQP(_changedData.name);
		_sql_insert = "insert into contacts_contact(id, infoid, type, " +
			"headurl, name, gender, ";
		if(!isNullTime(_changedData.birthday)){
			_sql_insert += "birthday, ";
			_birthday = _changedData.birthday;
		}
		if(_changedData.calendartype != null && _changedData.calendartype != ""){
			_sql_insert += "calendartype, ";
			_calendarType = _changedData.calendartype;
		}
		_sql_insert += "company, department, interests, meetoccasion," +
				" remarks, pinyin, origin, presrc, morefields," +
				" createtime, updatetime)values("
			+ tools.formatSqlValue(_changedData.id) + ","
			+ tools.formatSqlValue(_changedData.infoid) + ","
			+ tools.formatSqlValue(_changedData.type) + ","
			+ tools.formatSqlValue(_changedData.headurl) + ","
			+ tools.formatSqlValue(_changedData.name) + ","
			+ tools.formatSqlValue(_changedData.gender) + ",";
		if(!isNullTime(_changedData.birthday)){
			_sql_insert += _changedData.birthday + ",";
		}
		if(_changedData.calendartype != null && _changedData.calendartype != ""){
			_sql_insert += tools.formatSqlValue(_changedData.calendartype) + ",";
		}
	} else if(typeconstants.typeConstants["contactType_goods"] == _changedData.type){
		if(_changedData.goodsname == null){
			_changedData.goodsname = "";
		} else {
			// 姓名拼音
			if("" == _changedData.name){
				_pinyin = chinesePinYin.GetQP(_changedData.goodsname);
			} else {
				_pinyin = chinesePinYin.GetQP(_changedData.name);
			}
		}
		if(_changedData.position == null){
			_changedData.position  = "";
		}
		if(_changedData.comwebsite == null){
			_changedData.comwebsite  = "";
		}
		_sql_insert = "insert into contacts_contact(id, infoid, type, " +
				"headurl, goodsname, name, gender, position, comwebsite, " +
				"company, department, interests, meetoccasion, " +
				"remarks, pinyin, origin, presrc, morefields, createtime, " +
				"updatetime)values("
			+ tools.formatSqlValue(_changedData.id) + ","
			+ tools.formatSqlValue(_changedData.infoid) + ","
			+ tools.formatSqlValue(_changedData.type) + ","
			+ tools.formatSqlValue(_changedData.headurl) + ","
			+ tools.formatSqlValue(_changedData.goodsname) + ","
			+ tools.formatSqlValue(_changedData.name) + ","
			+ tools.formatSqlValue(_changedData.gender) + ","
			+ tools.formatSqlValue(_changedData.position) + ","
			+ tools.formatSqlValue(_changedData.comwebsite) + "," 
	} else {
		// 姓名拼音
		if(null == _changedData.pinyin){
			_pinyin = chinesePinYin.GetQP(_changedData.name);
		} else {
			_pinyin = _changedData.pinyin;
		}
		if(_changedData.position == null){
			_changedData.position  = "";
		}
		if(_changedData.comwebsite == null){
			_changedData.comwebsite  = "";
		}
		if(_changedData.nativeplace == null){
			_changedData.nativeplace  = "";
		}
		if(_changedData.edubackground == null){
			_changedData.edubackground  = "";
		}
		if(_changedData.degree == null){
			_changedData.degree  = "";
		}
		_sql_insert = "insert into contacts_contact(id, infoid, type, " +
			"headurl, name, gender, ";
		if(!isNullTime(_changedData.birthday)){
			_sql_insert += "birthday, ";
			_birthday = _changedData.birthday;
		}
		if(_changedData.calendartype != null && _changedData.calendartype != ""){
			_sql_insert += "calendartype, ";
			_calendarType = _changedData.calendartype;
		}
		_sql_insert += "position, comwebsite, nativeplace, edubackground, " +
				"degree, company, department, interests, " +
				"meetoccasion, remarks, pinyin, origin, presrc, morefields, " +
				"createtime, updatetime)values("
			+ tools.formatSqlValue(_changedData.id) + ","
			+ tools.formatSqlValue(_changedData.infoid) + ","
			+ tools.formatSqlValue(_changedData.type) + ","
			+ tools.formatSqlValue(_changedData.headurl) + ","
			+ tools.formatSqlValue(_changedData.name) + ","
			+ tools.formatSqlValue(_changedData.gender) + ",";
		if(!isNullTime(_changedData.birthday)){
			_sql_insert += _changedData.birthday + ",";
		}
		if(_changedData.calendartype != null && _changedData.calendartype != ""){
			_sql_insert += tools.formatSqlValue(_changedData.calendartype) + ",";
		}
		_sql_insert += tools.formatSqlValue(_changedData.position) + ","
			+ tools.formatSqlValue(_changedData.comwebsite) + "," 
			+ tools.formatSqlValue(_changedData.nativeplace) + ","
			+ tools.formatSqlValue(_changedData.edubackground) + ","
			+ tools.formatSqlValue(_changedData.degree) + ","
	}
	_sql_insert	+= tools.formatSqlValue(_changedData.company) + ","
		+ tools.formatSqlValue(_changedData.department) + ","
		+ tools.formatSqlValue(_changedData.interests) + ","
		+ tools.formatSqlValue(_changedData.meetoccasion) + ","
		+ tools.formatSqlValue(_changedData.remarks) + ","
		+ tools.formatSqlValue(_pinyin) + ","
		+ tools.formatSqlValue(_changedData.origin) + ","
		+ tools.formatSqlValue(_changedData.presrc) + ","
		+ tools.formatSqlValue(_changedData.morefields) + ","
		+ _changedData.createtime + ","
		+ _changedData.updatetime + ");";
	return _sql_insert;
}

// 获取插入联系人数据的sql语句
module.exports.getInsertContactDataSql = function(_changedData){
	return getInsertContactDataSql(_changedData);	
};

/**
 * 获取插入联系人数据的所有sql语句
 * @param _changedData
 * @param _sqlList
 * @param _isSync
 * @param _isRecord
 * @returns
 */
function getInsertContactAllSqls(_changedData, _sqlList, _isSync, _isRecord){
	// 获取插入联系人数据的sql语句
	var _sql = getInsertContactDataSql(_changedData);
	_sqlList.push(_sql);
	// 设置联系人属性
	if(null != _changedData.properties &&
			_changedData.properties.length > 0) {
		// 获取插入联系人属性数据的sql语句
		_sql = dbcontactproperties.getInsertContactPropertySql(_changedData.id,
				_changedData.properties);
		_sqlList.push(_sql);
	}
	// 设置联系人所在群组
	if(!tools.isNull(_changedData.groupList)) {
		// 获取插入群组关系数据的所有sql语句
		dbcontactgroupsrel.getUpdGroupRelAllSqlsByContactId(_changedData,
				_sqlList, true);
	}
	// 生成生日卡片
	if(!isNullTime(_changedData.birthday)){
		// 获取插入联系人生日数据的所有sql语句
		dbbirthday.getInsertBirthdayAllSqls({
			id: _changedData.id,
			commname: _changedData.name,
			commtime: _changedData.birthday,
			calendartype: _changedData.calendartype
		}, _sqlList);
	}
	if(_isRecord){
		// 更新记录
		var _updRecord = {
				id: _changedData.id,
				moduleid: moduleId,
				recordtype: constants.getConstants("recordType_create"), 
				datatype: constants.getConstants("datatype_contact")
			};
		// 设置更新记录
		if(_isSync){
			_sql = dbclsyncdata.getUpdRecordUnHandleSql(_updRecord);
		} else {
			_sql = dbclsyncdata.getUpdRecordSql(_updRecord);
		}
		_sqlList.push(_sql);
	}
	print("_sqlList " + JSON.stringify(_sqlList));
}

// 获取插入联系人相关数据的所有sql语句
module.exports.getInsertContactAllSqls = function(_changedData, _sqlList, _isSync, _isRecord){
	getInsertContactAllSqls(_changedData, _sqlList, _isSync, _isRecord);	
};

/**
 * 插入一条联系人数据
 * @param _changedData
 * @param _callFunc
 * @returns
 */
function insertContactData(_changedData, _callFunc){
	var _sqlList = [];
	// 获取插入联系人数据的所有sql语句
	getInsertContactAllSqls(_changedData, _sqlList, true, true);
	// 异步执行多条sql语句
	localdb.execute1({sqls: _sqlList}, function(_result){
		var _isSuc = false;
		if(_result > 0){
			_isSuc = true;
			// 若联系人生日不为空，则刷新时间流主页面
			if(!isNullTime(_changedData.birthday)){
				//时间流数据重新加载
				do_global.fire("taskDataChangeded");
				// 刷新视图（桌面小部件）
				widgetTools.refresh();
			}
		}
		// 回调函数
		_callFunc.call(this, _isSuc);
	});
}

// 插入一条联系人数据
module.exports.insertContactData = function(_changedData, _callFunc){
	insertContactData(_changedData, _callFunc);	
};

/**
 * 插入一条联系人数据-数据同步
 * @param _changedData
 * @param _callFunc
 * @returns
 */
function insertContactSyncData(_changedData, _callFunc){
	var _sqlList = [];
	// 获取插入联系人数据的所有sql语句
	getInsertContactAllSqls(_changedData, _sqlList, false, true);
	// 异步执行多条sql语句
	localdb.execute1({sqls: _sqlList}, function(_result){
		var _isSuc = false;
		if(_result > 0){
			_isSuc = true;
		}
		// 回调函数
		_callFunc.call(this, _isSuc);
	});
}

// 插入一条联系人数据-数据同步
module.exports.insertContactSyncData = function(_changedData, _callFunc){
	insertContactSyncData(_changedData, _callFunc);	
};

/**
 * 插入一条联系人数据-信息库同步
 * @param _changedData
 * @param _record
 * @param _callFunc
 * @returns
 */
function insertContactExpSync(_changedData, _record, _callFunc){
	var _sqlList = [];
	// 获取插入联系人数据的所有sql语句
	getInsertContactAllSqls(_changedData, _sqlList, false, false);
	// 设置更新记录-已同步
	var _sql = dbclsyncdata.getUpdRecordSql({
		id: _changedData.id,
		moduleid: moduleId,
		recordtime: _record.recordtime,
		recordtype: constants.getConstants("recordType_create"), 
		datatype: constants.getConstants("datatype_contact")
	});
	_sqlList.push(_sql);
	// 异步执行多条sql语句
	localdb.execute1({sqls: _sqlList}, function(_result){
		var _isSuc = false;
		if(_result > 0){
			_isSuc = true;
		}
		// 回调函数
		_callFunc.call(this, _isSuc);
	});
}

// 插入一条联系人数据-信息库同步
module.exports.insertContactExpSync = function(_changedData, _record, _callFunc){
	insertContactExpSync(_changedData, _record, _callFunc);	
};

/**
 * 批量保存联系人数据
 * @param _sqls
 * @param _isTransaction
 * @param _callFunc
 * @returns
 */
function saveAllContactData(_sqls, _isTransaction, _callFunc){
//	print("saveAllContactData " + _sqls);
	localdb.execute1(_sqls, _isTransaction, function(_data){
		// 回调函数
		_callFunc.call(this);
	});
}

// 批量保存联系人数据
module.exports.saveAllContactData = function(_sqls, _isTransaction, _callFunc){
	saveAllContactData(_sqls, _isTransaction, _callFunc);	
};

/**
 * 获取更新联系人数据的sql语句
 * @param _changedData
 * @returns
 */
function getUpdContactDataSql(_changedData){
	var _nowTime = new Date();
	var _setList="";
	if (_changedData.infoid != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "infoid=" + tools.formatSqlValue(_changedData.infoid);
	}
	if (_changedData.headurl != null && _changedData.headurl != ""){
		if (_setList.length > 0) _setList+=",";
		_setList += "headurl=" + tools.formatSqlValue(_changedData.headurl);
	}
	if (_changedData.type != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "type=" + tools.formatSqlValue(_changedData.type);
	}
	if (_changedData.goodsname != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "goodsname=" + tools.formatSqlValue(_changedData.goodsname);
	}
	if (_changedData.name != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "name=" + tools.formatSqlValue(_changedData.name);
	}
	if (_changedData.gender != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "gender=" + tools.formatSqlValue(_changedData.gender);
	}
	if (isNullTime(_changedData.birthday)){
		_changedData.birthday = null;
	}
	if (_setList.length > 0) _setList+=",";
	_setList += "birthday=" + _changedData.birthday;
	if (_changedData.calendartype != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "calendartype=" + tools.formatSqlValue(_changedData.calendartype);
	}
	if (_changedData.company != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "company=" + tools.formatSqlValue(_changedData.company);
	}
	if (_changedData.department != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "department=" + tools.formatSqlValue(_changedData.department);
	}
	if (_changedData.position != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "position=" + tools.formatSqlValue(_changedData.position);
	}
	if (_changedData.comwebsite != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "comwebsite=" + tools.formatSqlValue(_changedData.comwebsite);
	}
	if (_changedData.nativeplace != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "nativeplace=" + tools.formatSqlValue(_changedData.nativeplace);
	}
	if (_changedData.edubackground != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "edubackground=" + tools.formatSqlValue(_changedData.edubackground);
	}
	if (_changedData.degree != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "degree=" + tools.formatSqlValue(_changedData.degree);
	}
	if (_changedData.interests != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "interests=" + tools.formatSqlValue(_changedData.interests);
	}
	if (_changedData.meetoccasion != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "meetoccasion=" + tools.formatSqlValue(_changedData.meetoccasion);
	}
	if (_changedData.remarks != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "remarks=" + tools.formatSqlValue(_changedData.remarks);
	}
	if (_changedData.deleted != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "deleted=" + _changedData.deleted;
	}
	if (_changedData.morefields != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "morefields=" + tools.formatSqlValue(_changedData.morefields);
	}
	if (_changedData.updatetime != null){
		if (_setList.length > 0) _setList+=",";
		_setList += "updatetime=" + _changedData.updatetime;
	}
	// 姓名拼音
	var _pinyin = "";
	if(typeconstants.typeConstants["contactType_goods"] == _changedData.type){
		if(null != _changedData.name && "" != _changedData.name){
			_pinyin = chinesePinYin.GetQP(_changedData.name);
		} else {
			_pinyin = chinesePinYin.GetQP(_changedData.goodsname);
		}
	} else {
		if(null != _changedData.pinyin){
			_pinyin = _changedData.pinyin;
		} else {
			if(null != _changedData.name){
				_pinyin = chinesePinYin.GetQP(_changedData.name);
			}
		}
	}
	if (_setList.length > 0) _setList+=",";
	_setList += "pinyin=" + tools.formatSqlValue(_pinyin);
	
	var _sql_update = "";
	if (_setList.length > 0){
		_sql_update = "update contacts_contact set " + _setList 
			+ " where id=" + tools.formatSqlValue(_changedData.id) + ";";
	}
	return _sql_update;
}

// 获取更新联系人数据的sql语句
module.exports.getUpdContactDataSql = function(_changedData){
	return getUpdContactDataSql(_changedData);	
};

/**
 * 获取更新联系人数据的所有sql语句
 * @param _changedData
 * @param _sqlList
 * @param _isSync
 * @param _isRecord
 * @param _isExp
 * @returns
 */
function getUpdContactAllSqls(_changedData, _sqlList, _isSync, _isRecord, _isExp){
	// 获取更新联系人数据的sql语句
	var _sql = getUpdContactDataSql(_changedData);
	_sqlList.push(_sql);
	// 获取更新联系人属性的sql语句
	if(!_isExp){
		dbcontactproperties.getUpdContactPropAllSqls(_changedData.id,
				_changedData.properties, _sqlList);
	} else {
		//获取删除除过传真和慧影号的联系人属性数据的sql语句
		var _condition = "id=" + tools.formatSqlValue(_changedData.id) + 
			" and type not like '%fax%'" + " and type not like '%cydowid%'";
		_sql = dbcontactproperties.getDelContactPropSqlByCond(_condition);
		_sqlList.push(_sql);
		
		//获取插入联系人属性的sql语句
		_sql = dbcontactproperties.getInsertContactPropertySql(_changedData.id, 
				_changedData.properties);
		if("" != _sql){
			_sqlList.push(_sql);
		}
	}
	// 设置联系人所在群组
	if(!tools.isNull(_changedData.groupList)) {
		// 获取更新群组关系的所有sql语句
		dbcontactgroupsrel.getUpdGroupRelAllSqlsByContactId(_changedData,
				_sqlList, true);
	}
	// 更新生日卡片
	if(isNullTime(_changedData.birthday)){
		// 获取删除联系人生日卡片的所有sql语句
		dbbirthday.getDelBirthAllDataSql(_changedData.id, _sqlList);
	} else {
		// 获取更新生日卡片的所有sql语句
		dbbirthday.getUpdBirthdayAllSqls({
			id: _changedData.id,
			commname: _changedData.name,
			commtime: _changedData.birthday,
			calendartype: _changedData.calendartype
		}, _sqlList);
	}
	if(_isRecord){
		// 更新记录
		var _updRecord = {
				id: _changedData.id,
				moduleid: moduleId,
				recordtype: constants.getConstants("recordType_update"), 
				datatype: constants.getConstants("datatype_contact")
			};
		// 设置更新记录
		if(_isSync){
			_sql = dbclsyncdata.getUpdRecordUnHandleSql(_updRecord);
		} else {
			_sql = dbclsyncdata.getUpdRecordSql(_updRecord);
		}
		_sqlList.push(_sql);
	}
}

/**
 * 更新联系人数据
 * @param _changedData
 * @param _callFunc
 * @returns
 */
function updContactData(_changedData, _callFunc){
	var _sqlList = [];
	getUpdContactAllSqls(_changedData, _sqlList, true, true, false);
	// 异步执行多条sql语句
	localdb.execute1({sqls: _sqlList}, function(_result){
		var _isSuc = false;
		if(_result > 0){
			_isSuc = true;
			//时间流数据重新加载
			do_global.fire("taskDataChangeded");
			// 刷新视图（桌面小部件）
			widgetTools.refresh();
		}
		// 回调函数
		_callFunc.call(this, _isSuc);
	});
}

// 更新联系人数据
module.exports.updContactData = function(_changedData, _callFunc){
	updContactData(_changedData, _callFunc);	
};

/**
 * 更新联系人数据-信息库
 * @param _changedData
 * @param _callFunc
 * @returns
 */
function updContactDataFromExp(_changedData, _callFunc){
	var _sqlList = [];
	getUpdContactAllSqls(_changedData, _sqlList, true, true, true);
	// 异步执行多条sql语句
	localdb.execute1({sqls: _sqlList}, function(_result){
		var _isSuc = false;
		if(_result > 0){
			_isSuc = true;
			//时间流数据重新加载
			do_global.fire("taskDataChangeded");
			// 刷新视图（桌面小部件）
			widgetTools.refresh();
		}
		// 回调函数
		_callFunc.call(this, _isSuc);
	});
}

//更新联系人数据-信息库
module.exports.updContactDataFromExp = function(_changedData, _callFunc){
	updContactDataFromExp(_changedData, _callFunc);	
};

/**
 * 更新联系人数据--数据同步
 * @param _changedData
 * @param _callFunc
 * @returns
 */
function updContactSyncData(_changedData, _callFunc){
	var _sqlList = [];
	getUpdContactAllSqls(_changedData, _sqlList, false, true, false);
	// 异步执行多条sql语句
	localdb.execute1({sqls: _sqlList}, function(_result){
		var _isSuc = false;
		if(_result > 0){
			_isSuc = true;
		}
		// 回调函数
		_callFunc.call(this, _isSuc);
	});
}

// 更新联系人数据--数据同步
module.exports.updContactSyncData = function(_changedData, _callFunc){
	updContactSyncData(_changedData, _callFunc);	
};

/**
 * 更新联系人数据--信息库同步
 * @param _changedData
 * @param _record
 * @param _callFunc
 * @returns
 */
function updContactExpSync(_changedData, _record, _callFunc){
	var _sqlList = [];
	getUpdContactAllSqls(_changedData, _sqlList, false, false, true);
	var _updRecord = {
			id: _changedData.id,
			moduleid: moduleId,
			recordtime: _record.recordtime,
			recordtype: constants.getConstants("recordType_update"), 
			datatype: constants.getConstants("datatype_contact")
		};
	// 设置更新记录
	var _sql = "";
	if(_record.handlestate){
		// 设置更新记录-已同步
		_sql = dbclsyncdata.getUpdRecordSql(_updRecord);
	} else {
		// 设置更新记录-未同步
		_sql = dbclsyncdata.getUpdRecordUnHandleSql(_updRecord);
	}
	if(null != _sql && "" != _sql){
		_sqlList.push(_sql);
	}
	// 异步执行多条sql语句
	localdb.execute1({sqls: _sqlList}, function(_result){
		var _isSuc = false;
		if(_result > 0){
			_isSuc = true;
		}
		// 回调函数
		_callFunc.call(this, _isSuc);
	});
}

// 更新联系人数据--信息库同步
module.exports.updContactExpSync = function(_changedData, _record, _callFunc){
	updContactExpSync(_changedData, _record, _callFunc);	
};

/**
 * 更新联系人源信息字段
 * @param _changedData
 * @returns
 */
function updContactOriginData(_changedData){
	var _nowTime = new Date();
	var _sql = "update contacts_contact set origin=" 
		+ tools.formatSqlValue(_changedData.origin) + ","
		+ ",updatetime=" + _nowTime.getTime()
		+ " where id=" + tools.formatSqlValue(_changedData.id) + ";";
	_sqlList.push(_sql);
	
	// 更新记录
	var _updRecord = {
			id: _changedData.id,
			moduleid: moduleId,
			recordtype: constants.getConstants("recordType_update"), 
			datatype: constants.getConstants("datatype_contact")
		};
	// 设置更新记录
	_sql = dbclsyncdata.getUpdRecordUnHandleSql(_updRecord);
	_sqlList.push(_sql);
	
	localdb.executeSync1({sqls: _sqlList});
}

// 更新联系人源信息字段
module.exports.updContactOriginData = function(_changedData){
	updContactOriginData(_changedData);	
};

/**
 * 更新联系人头像数据
 * @param _changedData
 * @returns
 */
function update_contactHeadData(_changedData){
	var _sql_update = "update contacts_contact set headurl=" + 
		tools.formatSqlValue(_changedData.headurl) + " where id=" +
		tools.formatSqlValue(_changedData.id) + ";";
	localdb.execute(_sql_update, function(){
		_changedData.callFunc.call(this);
	});
}

// 更新联系人头像数据
module.exports.update_contactHeadData = function(_changedData){
	update_contactHeadData(_changedData);	
};

/**
 * 获取删除联系人sql语句
 * @param _id
 * @returns
 */
function getDelContactDataSql(_id){
	//删除联系人sql语句
	var _sql = "delete from contacts_contact where id=" + tools.formatSqlValue(_id) + ";";
	return _sql;
}

/**
 * 获取删除联系人涉及的所有sql语句
 * @param _id
 * @param _sqlList
 * @param _isSync
 * @param _isRecord
 * @returns
 */
function getDelContactDataAllSqls(_id, _sqlList, _isSync, _isRecord){
	//删除联系人sql语句
	var _sql = getDelContactDataSql(_id);
	_sqlList.push(_sql);
	
	//删除联系人属性sql语句
	_sql = dbcontactproperties.getDelContactPropDataSql(_id);
	_sqlList.push(_sql);
	
	// 删除联系人导入导出记录表对应数据的sql语句
	_sql = dbcontactimexport.getDelContactImExportDataSql(_id);
	_sqlList.push(_sql);
	
	//删除联系人生日卡片sql语句
	dbbirthday.getDelBirthAllDataSql(_id, _sqlList);
	
	if(_isRecord){
		// 更新记录
		var _updRecord = {
				id: _id,
				moduleid: moduleId,
				recordtype: constants.getConstants("recordType_delete"), 
				datatype: constants.getConstants("datatype_contact")
			};
		// 设置更新记录
		if(_isSync){
			_sql = dbclsyncdata.getUpdRecordUnHandleSql(_updRecord);
		} else {
			_sql = dbclsyncdata.getUpdRecordSql(_updRecord);
		}
		_sqlList.push(_sql);
	}
}

/**
 * 删除联系人相关数据
 * @param _changedData
 * @returns
 */
function delContactData(_changedData){
	var _sqlList = [];
	var _dataList = _changedData.dataList;
	for(var i=0; i<_dataList.length; i++){
		if(null == _dataList[i]) continue;
		var _id = _dataList[i].id;
		// 获取删除指定联系人的所有sql语句
		getDelContactDataAllSqls(_id, _sqlList, true, true);
		// 删除联系人所在群组关系
		dbcontactgroupsrel.getDelGroupRelAllSqlsByContactId(_id,
				_sqlList, true);
		//删除联系人头像缓存
		do_storage.deleteDir("data://head/contact/" + _id);
	}
	//执行sql语句，并返回
	if(!tools.isNullOrEmpty(_sqlList)){
		localdb.execute1({sqls: _sqlList}, function(_data){
			//时间流数据重新加载
			do_global.fire("taskDataChangeded");
			// 刷新视图（桌面小部件）
			widgetTools.refresh();
			// 回调函数
			_changedData.callFunc.call(this);
		});
	} else {
		// 回调函数
		_changedData.callFunc.call(this);
	}
}

// 删除联系人数据
module.exports.delContactData = function(_changedData){
	delContactData(_changedData);
};

/**
 * 删除联系人数据--数据同步
 * @param _id
 * @param _callFunc
 * @returns
 */
function delContactSyncData(_id, _callFunc){
	var _sqlList = [];
	// 获取删除指定联系人的所有sql语句
	getDelContactDataAllSqls(_id, _sqlList, false, true);
	// 删除联系人所在群组关系
	dbcontactgroupsrel.getDelGroupRelAllSqlsByContactId(_id,
			_sqlList, true);
	localdb.execute1({sqls: _sqlList}, function(_result){
		var _isSuc = false;
		if(_result > 0){
			_isSuc = true;
			//删除联系人头像
			do_storage.deleteDir("data://head/contact/" + _id);
			// 回调函数
			_callFunc.call(this, _isSuc);
		}
	});
}

// 删除联系人数据--数据同步
module.exports.delContactSyncData = function(_id, _callFunc){
	delContactSyncData(_id, _callFunc);	
};

/**
 * 删除联系人数据--信息库同步
 * @param _id
 * @param _record
 * @param _callFunc
 * @returns
 */
function delContactExpSync(_id, _record, _callFunc){
	var _sqlList = [];
	// 获取删除指定联系人的所有sql语句
	getDelContactDataAllSqls(_id, _sqlList, false, false);
	// 删除联系人所在群组关系
	dbcontactgroupsrel.getDelGroupRelByContactIdSync(_id, _sqlList);
	// 设置更新记录-已同步
	var _sql = dbclsyncdata.getUpdRecordSql({
		id: _id,
		moduleid: moduleId,
		recordtime: _record.recordtime,
		recordtype: constants.getConstants("recordType_delete"), 
		datatype: constants.getConstants("datatype_contact")
	});
	_sqlList.push(_sql);
	
	localdb.execute1({sqls: _sqlList}, function(_result){
		var _isSuc = false;
		if(_result > 0){
			_isSuc = true;
			//删除联系人头像
			do_storage.deleteDir("data://head/contact/" + _id);
			// 回调函数
			_callFunc.call(this, _isSuc);
		}
	});
}

//删除联系人数据--信息库同步
module.exports.delContactExpSync = function(_id, _record, _callFunc){
	delContactExpSync(_id, _record, _callFunc);	
};

/**
 * 查询指定条件（_id）下联系人数据
 * @param _id
 * @returns
 */
function query_contactDataById(_id){
	var _sql ="select * from contacts_contact where id="
		+ tools.formatSqlValue(_id) + ";";
	var _data = localdb.querySync(_sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data[0];
}

//查询指定条件（_id）下联系人数据
module.exports.query_contactDataById = function(_id){
	return query_contactDataById(_id);
};

/**
 * 查询指定联系人字段（_condition）数据
 * @param _id
 * @returns
 */
function query_contactDataByCondition(_condition){
	var _sql ="select " + _condition + " from contacts_contact;";
	var _data = localdb.querySync(_sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data;
}

//查询指定联系人字段（_condition）数据
module.exports.query_contactDataByCondition = function(_condition){
	return query_contactDataByCondition(_condition);
};

/**
 * 查询指定条件（类型和姓名）下联系人数据--用于同一个类型下重名判断
 * @param _type
 * @param _name
 * @returns
 */
function query_contactDataByTypeName(_type, _name){
	var _sql ="select id from contacts_contact where type=" 
		+ tools.formatSqlValue(_type); 
	if(_type == typeconstants.typeConstants["contactType_goods"]){
		_sql += " and goodsname=" + tools.formatSqlValue(_name) + ";";
	} else {
		_sql += " and name=" + tools.formatSqlValue(_name) + ";";
	}
	var _data = localdb.querySync(_sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data[0];
}

//查询指定条件（类型和姓名或物品名称）下联系人数据--用于同一个类型下重名判断
module.exports.query_contactDataByTypeName = function(_type, _name){
	return query_contactDataByTypeName(_type, _name);
};

/**
 * 查询指定条件（唯一标识、类型和姓名）下联系人数据--用于同一个类型下重名判断
 * @param _id
 * @param _type
 * @param _name
 * @returns
 */
function query_contactDataByIdTypeName(_id, _type, _name){
	var _sql ="select id from contacts_contact where id!=" 
		+ tools.formatSqlValue(_id) + " and type="
		+ tools.formatSqlValue(_type); 
	if(_type == typeconstants.typeConstants["contactType_goods"]){
		_sql += " and goodsname=" + tools.formatSqlValue(_name) + ";";
	} else {
		_sql += " and name=" + tools.formatSqlValue(_name) + ";";
	}
	var _data = localdb.querySync(_sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data[0];
}

//查询指定条件（唯一标识、类型和姓名或物品名称）下联系人数据--用于同一个类型下重名判断
module.exports.query_contactDataByIdTypeName = function(_id, _type, _name){
	return query_contactDataByIdTypeName(_id, _type, _name);
};

/**
 * 检测联系人是否存在（通过id）
 * @param _id
 * @returns
 */
function isExistContactDataById(_id){
	var _sql ="select id from contacts_contact where id="
		+ tools.formatSqlValue(_id) + ";";
	var _data = localdb.querySync(_sql);
	if(_data == null || _data.length <= 0){
		return false;
	}
	return true;
}

// 检测联系人是否存在（通过id）
module.exports.isExistContactDataById = function(_id){
	return isExistContactDataById(_id);
};

/**
 * 检测联系人是否存在（通过id）-异步查询
 * @param _id
 * @param _callFunc
 * @returns
 */
function isExistContactDataByIdAsync(_id, _callFunc){
	var _sql ="select id from contacts_contact where id=?;";
	localdb.query(_sql, [_id], function(_data){
		var _isExist = true;
		if(_data == null || _data.length <= 0){
			_isExist = false;
		}
		_callFunc.call(this, _isExist);
	});
}

// 检测联系人是否存在（通过id）-异步查询
module.exports.isExistContactDataByIdAsync = function(_id, _callFunc){
	isExistContactDataByIdAsync(_id, _callFunc);
};

/**
 * 检测联系人是否存在（通过origin）
 * @param _origin
 * @returns
 */
function queryContactByOrigin(_origin){
	var _sql ="select id,type,origin from contacts_contact where origin like "
		+ tools.formatSqlValue("%" + _origin.dataid + "%") + " origin like"
		+ tools.formatSqlValue("%" + _origin.userid + "%") + ";";
	var _data = localdb.querySync(_sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data[0];
}

// 检测联系人是否存在（通过origin）
module.exports.queryContactByOrigin = function(_origin){
	return queryContactByOrigin(_origin);
};

/**
 * 查询大量联系人数据
 * @param _ids
 * @returns
 */
function queryMoreContactByIds(_ids){
	var _sqlTmp = "";
	for(var i=0; i<_ids.length; i++){
		if(null == _ids[i]) continue;
		if(_sqlTmp.length > 0){
			_sqlTmp += ",";
		}
		_sqlTmp += tools.formatSqlValue(_ids[i].id);
	}
	if(_sqlTmp.length > 0){
		var _sql ="select id,type,updatetime from contacts_contact where id in (";
		_sql += _sqlTmp;
		_sql += ");";
		
		var _data = localdb.querySync(_sql);
		if(_data == null || 
				_data.length <= 0){
			_data = null;
		}
		return _data;
	} else {
		return null;
	}
}

// 查询大量联系人数据
module.exports.queryMoreContactByIds = function(_ids){
	return queryMoreContactByIds(_ids);
};

/**
 * 查询大量联系人指定字段的数据
 * @param _fields
 * @param _ids
 * @returns
 */
function queryMoreFieldByIds(_fields, _ids){
	var _sql = "select " + _fields + 
		" from contacts_contact where id in ('" + 
		_ids.join("','") + "');";
	var _data = localdb.querySync(_sql);
	if(_data == null || 
			_data.length <= 0){
		_data = null;
	}
	return _data;
}

// 查询大量联系人指定字段的数据
module.exports.queryMoreFieldByIds = function(_fields, _ids){
	return queryMoreFieldByIds(_fields, _ids);
};

/**
 * 查询联系人数据
 * @param _fields 查询的字段
 * @param _condition 检索条件，where关键字后面的语句，不包含where
 * @param _callFunc 回调函数
 */
function queryContactData(_fields, _condition, _callFunc){
	if(tools.isNullOrEmpty(_fields)){
		_fields = "*";
	}
	var _sql = "select " + _fields + " from contacts_contact ";
	if(!tools.isNullOrEmpty(_condition)){
		_sql += " where " + _condition;
	}
	_sql += ";";
	// 执行异步查询语句
	localdb.query(_sql, function(_data){
		if(tools.isNullOrEmpty(_data)){
			_data = null;
		}
		_callFunc.call(this, _data);
	});
}

//查询大量联系人指定字段的数据
module.exports.queryContactData = function(_fields, _condition, _callFunc){
	queryContactData(_fields, _condition, _callFunc);
};

/**
 * 检测是否存在联系人（除过“大慧小影”）
 * @returns
 */
function isExistContactData(){
	var _id = constants.getConstants("cydowServiceId");
	var _sql ="select id from contacts_contact where id!="
		+ tools.formatSqlValue(_id) + ";";
	var _data = localdb.querySync(_sql);
	if(_data == null || _data.length <= 0){
		return false;
	}
	return true;
}

// 检测是否存在联系人（除过“大慧小影”）
module.exports.isExistContactData = function(){
	return isExistContactData();
};
