/**
 * 周期
 * 
 * @Author : Cydow
 * @Timestamp : 2017-02-22
 */
var deviceone = require("deviceone");

var objectId = require("objectId"); // 生成objectId
var tools = require("tools");         //工具
var constants = require("constants"); // 常量

//数据库
var localdb = deviceone.mm("M0026_sqlite", "localdb", "app");

/**
 * 初始化周期类型表（cycletype）中的数据
 * @returns
 */
function insert_cycleTypeData(){
	var _cycleType = 1;
	var _cycleNames = ["不重复", "每天", "每周", "每两周", "每月", "每季度", "每半年", "每年"];
	var _cycleTimes = [0, 86400000, 604800000, 1209600000, 2592000000, 7776000000, 31536000000, 63072000000];
	for(var i=0; i<8; i++){
		var _id = i + 1;
		var _sql_cycletype_insert = "insert into cycletype (id, name, timelength, type) values("
			+ _id + ", " 
			+ tools.formatSqlValue(_cycleNames[i]) + ", " 
			+ _cycleTimes[i] + ", " +
			_cycleType + ");"
		localdb.executeSync(_sql_cycletype_insert);
	}
}

//初始化周期类型表（cycletype）中的数据
module.exports.insert_cycleType = function(){
	return insert_cycleTypeData();
};

/**
 * 查询周期类型表（cycletype）中的数据
 * @returns
 */
function query_cycleTypeData(){
	var _sql_cycletype_query = "select * from cycletype;";
	var _data_cycletype = localdb.querySync(_sql_cycletype_query);
	if(_data_cycletype == null || _data_cycletype.length <= 0){
		return null;
	}
	return _data_cycletype;
}

//查询周期类型表（cycletype）中的所有数据
module.exports.query_cycleTypeAllData = function(){
	return query_cycleTypeData();
};

/**
 * 通过id从周期类型表（cycletype）中获取指定的一条数据
 * @param _id
 * @returns
 */
function query_cycleTypeOneData(_id){
	var sql = "select * from cycletype where id=" + _id + ";";
	var _data = localdb.querySync(sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data[0];
}

//通过id从周期类型表（cycletype）中获取指定的一条数据
module.exports.query_cycleTypeOneDataById = function(_id){
	return query_cycleTypeOneData(_id);
};

/**
 * 获取插入一条周期性记录数据的sql语句
 * @param _cycleRecord
 * @returns
 */
function getInsertCycleRecordSql(_cycleRecord){
	if(null == _cycleRecord.count){
		_cycleRecord.count = 0;
	}
	if(null == _cycleRecord.endtime){
		_cycleRecord.endtime = 0;
	}
	if(null == _cycleRecord.week){
		_cycleRecord.week = "";
	}
	if(null == _cycleRecord.type){
		_cycleRecord.type = 1;
	}
	if(null == _cycleRecord.frequency){
		_cycleRecord.frequency = 1;
	}
	if(null == _cycleRecord.selection){
		_cycleRecord.selection = "";
	}
	var _sql_insert = "insert into cyclerecord(id, taskid, starttime, endtime, updtime, cycleid, week, type, frequency, selection, count, infocategory) values(" 
		+ tools.formatSqlValue(_cycleRecord.id) + ","
		+ tools.formatSqlValue(_cycleRecord.taskid) + ","
		+ _cycleRecord.starttime + ","
		+ _cycleRecord.endtime + ","
		+ _cycleRecord.updtime + ","
		+ _cycleRecord.cycleid + ","
		+ tools.formatSqlValue(_cycleRecord.week) + ","
		+ _cycleRecord.type + ","
		+ _cycleRecord.frequency + ","
		+ tools.formatSqlValue(_cycleRecord.selection) + ","
		+ _cycleRecord.count + ","
		+ _cycleRecord.infocategory + ");";
	return _sql_insert;
}

// 获取插入一条周期性记录数据的sql语句
module.exports.getInsertCycleRecordSql = function(_cycleRecord){
	return getInsertCycleRecordSql(_cycleRecord);
};

/**
 * 插入一条数据到周期记录表（cyclerecord）中
 * @param _cycleRecord
 */
function insert_cycleRecordData(_cycleRecord){
	var _sql_insert = getInsertCycleRecordSql(_cycleRecord);
	localdb.executeSync(_sql_insert);
}

//插入一条数据到周期记录表（cyclerecord）中
module.exports.insert_cycleRecordData = function(_cycleRecord){
	return insert_cycleRecordData(_cycleRecord);
};

/**
 * 获取更新周期记录的sql语句
 * @param _changedData
 * @returns
 */
function getUpdCycleRecordSql(_changedData){
	var _setList = "";
	if (_changedData.taskid!=null){
		if (_setList.length > 0) _setList+=",";
		_setList += "taskid=" + tools.formatSqlValue(_changedData.taskid);
	}
	if (_changedData.starttime!=null){
		if (_setList.length > 0) _setList+=",";
		_setList += "starttime=" + _changedData.starttime;
	}
	if (_changedData.endtime!=null){
		if (_setList.length > 0) _setList+=",";
		_setList += "endtime=" + _changedData.endtime;
	}
	if (_changedData.updtime!=null){
		if (_setList.length > 0) _setList+=",";
		_setList += "updtime=" + _changedData.updtime;
	}
	if (_changedData.cycleid!=null){
		if (_setList.length > 0) _setList+=",";
		_setList += "cycleid=" + _changedData.cycleid;
	}
	if (_changedData.week!=null){
		if (_setList.length > 0) _setList+=",";
		_setList += "week=" + tools.formatSqlValue(_changedData.week);
	}
	if (_changedData.weekday!=null){
		if (_setList.length > 0) _setList+=",";
		_setList += "weekday=" + tools.formatSqlValue(_changedData.weekday);
	}
	if (_changedData.type!=null){
		if (_setList.length > 0) _setList+=",";
		_setList += "type=" + _changedData.type;
	}
	if (_changedData.frequency!=null){
		if (_setList.length > 0) _setList+=",";
		_setList += "frequency=" + _changedData.frequency;
	}
	if (_changedData.selection!=null){
		if (_setList.length > 0) _setList+=",";
		_setList += "selection=" + tools.formatSqlValue(_changedData.selection);
	}
	if (_changedData.count!=null){
		if (_setList.length > 0) _setList+=",";
		_setList += "count=" + _changedData.count;
	}
	if (_changedData.infocategory!=null){
		if (_setList.length > 0) _setList+=",";
		_setList += "infocategory=" + _changedData.infocategory;
	}
	var _sql_update= "";
	if(_setList == "") return _sql_update;
	_sql_update = "update cyclerecord set " 
		+ _setList + " where id=" + tools.formatSqlValue(_changedData.id) + ";";
	return _sql_update;
}

// 获取更新周期记录的sql语句
module.exports.getUpdCycleRecordSql = function(_changedData){
	return getUpdCycleRecordSql(_changedData);
};

/**
 * 更新周期记录的数据
 * @param _changedData
 * @returns
 */
function update_cycleRecordData(_changedData){
	var _sql = getUpdCycleRecordSql(_changedData);
	localdb.executeSync(_sql);
}

// 更新周期记录的数据
module.exports.update_cycleRecordData = function(_changedData){
	return update_cycleRecordData(_changedData);
};

/**
 * 获取删除周期记录的sql语句
 * @param _id
 * @returns
 */
function getDelCycleRecordDataSql(_id){
	var _sql_del = "delete from cyclerecord where " 
		+ "id=" + tools.formatSqlValue(_id) + ";";
	return _sql_del;
}

//获取删除周期记录的sql语句
module.exports.getDelCycleRecordDataSql = function(_id){
	return getDelCycleRecordDataSql(_id);	
};

/**
 * 通过id删除周期记录表（cyclerecord）中的一条数据
 * @param _cycleRecordId
 */
function delete_cyclerecordDataById(_cycleRecordId){
	var _sql_cyclerecord_del = getDelCycleRecordDataSql(_cycleRecordId);
	localdb.executeSync(_sql_cyclerecord_del);
}

//通过id删除周期记录表（cyclerecord）中的一条数据
module.exports.delete_cyclerecordDataById = function(_cycleRecordId) {
	delete_cyclerecordDataById(_cycleRecordId);
};

/**
 * 通过id（周期记录id）从周期记录表（cyclerecord）中获取指定的一条数据
 * @param _id
 * @returns
 */
function query_cycleRecordOneData(_id){
	var sql = "select * from cyclerecord where id=" + tools.formatSqlValue(_id) + ";";
	var _data = localdb.querySync(sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data[0];
}

//通过id（周期记录id）从周期记录表（cyclerecord）中获取指定的一条数据
module.exports.query_cycleRecordOneData = function(_id){
	return query_cycleRecordOneData(_id);
};

/**
 * 通过taskid（源数据id）从周期记录表（cyclerecord）中获取指定的一条数据
 * @param _id
 * @returns
 */
function query_cycleRecordOneDataByTaskId(_id){
	var sql = "select * from cyclerecord where taskid=" + tools.formatSqlValue(_id) + ";";
	var _data = localdb.querySync(sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data[0];
}

//通过taskid（源数据id）从周期记录表（cyclerecord）中获取指定的一条数据
module.exports.query_cycleRecordOneDataByTaskId = function(_id){
	return query_cycleRecordOneDataByTaskId(_id);
};

/**
 * 通过id从周期记录表（cyclerecord）中获取taskid
 * @param _id
 * @returns
 */
function query_taskidByCycleRecordId(_id){
	var sql = "select taskid from cyclerecord where id=" + tools.formatSqlValue(_id) + ";";
	var _data = localdb.querySync(sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data[0];
}

//通过id从周期记录表（cyclerecord）中获取taskid
module.exports.query_taskidByCycleRecordId = function(_id){
	return query_taskidByCycleRecordId(_id);
};

/**
 * 根据条件查询重复记录数据
 * @returns
 */
function query_cycleRecordByCondition(_condition){
	var _query_cyclerecord_sql = "select * from cyclerecord where " + 
		_condition + ";";
	var _data = localdb.querySync(_query_cyclerecord_sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data;
}

// 根据条件查询重复记录数据
module.exports.query_cycleRecordByCondition = function(_condition){
	return query_cycleRecordByCondition(_condition);
};

/**
 * 周期记录表（cyclerecord）中获取所有数据
 * @returns
 */
function query_cycleRecordData(){
	var _query_cyclerecord_sql = "select * from cyclerecord;";
	var _data = localdb.querySync(_query_cyclerecord_sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data;
}

//周期记录表（cyclerecord）中获取所有数据
module.exports.query_cycleRecordAllData = function(){
	return query_cycleRecordData();
};

/**
 * 周期记录表（cyclerecord）中获取count数据
 * @returns
 */
function query_cycleRecordCount(){
	var _query_cyclerecord_sql = "select count from cyclerecord where id="
		+ tools.formatSqlValue(_id) + ";";
	var _data = localdb.querySync(_query_cyclerecord_sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data[0];
}

//周期记录表（cyclerecord）中获取count数据
module.exports.query_countById = function(_id){
	return query_cycleRecordCount(_id);
};