/**
 * 联系人属性表
 * 
 * @Author : Cydow
 * @Timestamp : 2017-12-04
 */
//引入组件库
var deviceone = require("deviceone");
var do_storage = deviceone.sm("do_Storage");

var tools = require("tools");            //工具
var constants = require("constants");    //常量

//数据库
var localdb = deviceone.mm("M0026_sqlite", "localdb", "app");

/**
 * 判断关联数据是否存在
 * @param _changedData
 */
function isExistcydowIdRelationData(_changedData){
	var _data = query_cydowIdDataByPhone(_changedData.phone);
	if(null != _data){
		if(_data.cydowid != _changedData.cydowid){
			// 更新慧影号信息
			update_cydowIdRelationData(_changedData);
		}
		return true;
	}
	return false;
}

/**
 * 获取插入慧影号和手机号关联数据的sql语句
 * @param _changedData
 * @returns
 */
function getInsertCydowIdRelDataSql(_changedData){
	var _sql_insert = "";
	if(null != _changedData && 
			_changedData.length > 0){
		var _sql_data = "";
		var _cydowIdRelData = {};
		for(var i=0; i<_changedData.length; i++){
			var _data = _changedData[i];
			if(null == _data) continue;
			if(_data.phone in _cydowIdRelData){
				continue;
			}
			// 若慧影号和手机号关联数据存在，则检查下一条信息
			if(isExistcydowIdRelationData(_data)) {
				continue;
			}
			if(_sql_data.length > 0){
				_sql_data += ",";
			}
			_sql_data += "("
			_sql_data +=  tools.formatSqlValue(_data.cydowid) + ",";
			_sql_data +=  tools.formatSqlValue(_data.phone) + ")";
			
			_cydowIdRelData[_data.phone] = _data.cydowid;
		}
		if(_sql_data.length > 0){
			_sql_insert = "insert into contacts_cydowid_relation(cydowid, phone) values";
			_sql_insert += _sql_data;
			_sql_insert += ";";
		}
	}
	return _sql_insert;
}

/**
 * 插入慧影号和手机号关联数据
 * @param _changedData
 */
function insertCydowIdRelData(_changedData){
	var _isSuc = true;
	var _sql = getInsertCydowIdRelDataSql(_changedData);
	if("" != _sql){
		_isSuc = localdb.executeSync(_sql);
	}
	return _isSuc;
}

// 插入慧影号和手机号关联数据
module.exports.insertCydowIdRelData = function(_changedData){
	return insertCydowIdRelData(_changedData);	
};

/**
 * 更新联系人属性表（contacts_cydowid_relation）表中的数据
 * @param _changedData
 */
function update_cydowIdRelationData(_changedData){
	var _sql_update = "update contacts_cydowid_relation set cydowid="
		+ tools.formatSqlValue(_changedData.cydowid) + " where phone="
		+ tools.formatSqlValue(_changedData.phone) + ";";
	return localdb.executeSync(_sql_update);
}

//更新慧影号关联表（contacts_cydowid_relation）数据
module.exports.update_cydowIdRelationData = function(_changedData){
	return update_cydowIdRelationData(_changedData);	
};

/**
 * 删除慧影号关联表（contacts_cydowid_relation）中指定id的数据
 * @param _phone
 */
function delete_cydowIdRelationData(_phone){
	var _sql_del ="delete from contacts_cydowid_relation where phone=" 
		+ tools.formatSqlValue(_phone) + ";";
	var isSuccess = localdb.executeSync(_sql_del);
	return isSuccess;
}

//删除慧影号关联表（contacts_cydowid_relation）中指定phone的数据
module.exports.delete_cydowIdRelationData = function(_phone){
	return delete_cydowIdRelationData(_phone);	
};

/**
 * 条件删除慧影号关联表（contacts_cydowid_relation）中的数据
 * @param _phone
 */
function deleteRelByCond(cond){
	var _sql_del ="delete from contacts_cydowid_relation where " + cond +";"; 
	var isSuccess = localdb.executeSync(_sql_del);
	return isSuccess;
}

// 条件删除慧影号关联表（contacts_cydowid_relation）中的数据
module.exports.deleteRelByCond = function(cond){
	return deleteRelByCond(cond);	
};

/**
 * 查询指定条件（_phone）的慧影号
 * @param _phone
 * @returns
 */
function query_cydowIdDataByPhone(_phone){
	var sql ="select cydowid from contacts_cydowid_relation where phone="
		+ tools.formatSqlValue(_phone) + ";";
	var _data = localdb.querySync(sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data[0];
}

//查询指定条件（_phone）的慧影号
module.exports.query_cydowIdDataByPhone = function(_phone){
	return query_cydowIdDataByPhone(_phone);
};

/**
 * 查询指定条件（_cydowId）的电话号码
 * @param _cydowId
 * @returns
 */
function query_phoneDataByCydowId(_cydowId){
	var sql ="select phone from contacts_cydowid_relation where cydowid="
		+ tools.formatSqlValue(_cydowId) + ";";
	var _data = localdb.querySync(sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data[0];
}

//查询指定条件（_cydowId）的电话号码
module.exports.query_phoneDataByCydowId = function(_cydowId){
	return query_phoneDataByCydowId(_cydowId);
};
