/**
 * 本地消息表
 * 
 * @Author : Cydow
 * @Timestamp : 2019-08-29
 */
//工具
var tools = require("tools");
// 常量
var constants = require("constants");
//生成objectId
var objectId = require("objectId");

// 数据库操作-消息中心
var dbmessagecenter = require("dbmessagecenter.ui");

//数据库
var localdb = mm("M0026_sqlite", "localdb", "app");

/**
 * 插入一条本地消息
 * @param _changedData
 */
function insertLocalMessage(_changedData){
	var _nowTime = new Date();
	if(null == _changedData.id){
		_changedData.id = objectId.getObjectId();
	}
	if(null == _changedData.flag){
		_changedData.flag = constants.getConstants("localMsg_flag_unRead");
	}
	var _sql_insert = "insert into local_message(id, type, remindtime, againtime, notifyid, flag, createtime, updatetime) values("
		+ tools.formatSqlValue(_changedData.id) + ","
		+ _changedData.type + ", "
		+ _changedData.remindtime + ", "
		+ _changedData.againtime + ", "
		+ _changedData.notifyid + ", "
		+ _changedData.flag + ", "
		+ _nowTime.getTime() + ", "
		+ _nowTime.getTime() + ");";
	var isSuccess = localdb.executeSync(_sql_insert);
	return isSuccess;
}

//插入一条本地消息
module.exports.insertLocalMessage = function(_changedData){
	return insertLocalMessage(_changedData);	
};

/**
 * 更新一条本地消息
 * @param _changedData
 */
function updateLocalMessage(_changedData){
	var _setList = "";
	if (_changedData.remindtime != null) {
		if (_setList.length > 0)
			_setList += ",";
		_setList += "remindtime=" + _changedData.remindtime;
	}
	if (_changedData.againtime != null) {
		if (_setList.length > 0)
			_setList += ",";
		_setList += "againtime=" + _changedData.againtime;
	}
	if (_changedData.flag != null) {
		if (_setList.length > 0)
			_setList += ",";
		_setList += "flag=" + _changedData.flag;
	}
	var _nowTime = new Date();
	if(_setList.length > 0)
		_setList += ",";
	_setList += "updatetime=" + _nowTime.getTime();
	
	var _sql_update = "update local_message set "
		+ _setList + " where id="
		+ tools.formatSqlValue(_changedData.id) + ";";
	var isSuccess = localdb.executeSync(_sql_update);
	return isSuccess;
}

// 更新一条本地消息
module.exports.updateLocalMessage = function(_changedData){
	return updateLocalMessage(_changedData);	
};

/**
 * 设置本地消息
 * @param _type
 * @param _title
 * @param _content
 * @param _notifyId
 * @param _isDel
 * @returns
 */
function setLocalMsg(_type, _title, _content, _notifyId, _isDel){
	var _localMsg = queryLocalMsgByNotifyId(_notifyId);
	var _promptTime = (new Date()).getTime();
	print("_localMsg " + JSON.stringify(_localMsg));
	print("_promptTime " + _promptTime);
	var _id = objectId.getObjectId();
	// 若指定本地消息为空，则插入一条本地消息；
	// 否则，更新指定的本地消息
	if(null == _localMsg){
		if(!_isDel){
			// 插入一条本地消息
			insertLocalMessage({
				id: _id,
				type: _type,
				remindtime: _promptTime,
				againtime: _promptTime,
				notifyid: _notifyId
			});
			// 插入一条消息中心数据
			dbmessagecenter.insertMsgCenterData({
				msgid: _id,
				source: constants.getConstants("msgCenterSource_local"),
				msgtype: _type,
				title: _title,
				content: _content,
				msgcount: 1,
				remindtime: _promptTime
			});
		}
	} else {
		_id = _localMsg.id;
		if(_isDel){
			delLocalMessage(_id);
			// 删除消息中心的数据
			dbmessagecenter.delMsgCenterDataByMsgId(_id);
		} else {
			// 更新指的一条本地消息
			updateLocalMessage({
				id: _id,
				remindtime: _promptTime,
				againtime: _promptTime + 1,
				flag: constants.getConstants("localMsg_flag_unRead")
			});
			// 更新一条消息中心数据
			dbmessagecenter.updateMsgCenterByMsgId({
				msgid: _id,
				remindtime: _promptTime,
				status: constants.getConstants("msgCenterMsgStatus_unread")
			});
		}
	}
}

/**
 * 设置同步失败本地消息（时间流同步、联系人同步）
 * @param _type
 * @param _syncFailNum
 * @param _isDel
 * @returns
 */
function setSyncFailLocalMsg(_type, _syncFailNum, _isDel){
	var _notifyId = 1100000000;
	var _title = "时间流同步";
	var _content = "同步完成。" + _syncFailNum + "条信息同步失败，点击查看。";
	// 本地消息类型，联系人同步失败
	if(_type == constants.getConstants("localMsgType_syncContactFail")){
		_notifyId = 1100000001;
		_title = "联系人同步";
	} else if(_type == constants.getConstants("localMsgType_syncExpFail")){
		_notifyId = 1100000004;
		_title = "信息库同步";
	}
	// 设置本地消息
	setLocalMsg(_type, _title, _content, _notifyId, _isDel);
}

// 设置同步失败本地消息（时间流同步、联系人同步、信息库同步）
module.exports.setSyncFailLocalMsg = function(_type, _syncFailNum, _isDel){
	setSyncFailLocalMsg(_type, _syncFailNum, _isDel);
};

/**
 * 设置同步异常中断本地消息
 * @param _type
 * @param _isDel
 * @returns
 */
function setSyncBreakLocalMsg(_type, _isDel){
	var _notifyId = 1100000002;
	var _title = "时间流同步";
	var _content = "同步未完成。同步过程中出现未知异常，请重新同步。";
	// 本地消息类型，联系人同步异常中断
	if(_type == constants.getConstants("localMsgType_syncContactBreak")){
		_notifyId = 1100000003;
		_title = "联系人同步";
	} else if(_type == constants.getConstants("localMsgType_syncExpBreak")){
		_notifyId = 1100000005;
		_title = "信息库同步";
	} else if(_type == constants.getConstants("localMsgType_syncAllBreak")){
		_notifyId = 1100000006;
		_title = "整体同步";
	}
	// 设置本地消息
	setLocalMsg(_type, _title, _content, _notifyId, _isDel);
}

// 设置同步异常中断本地消息
module.exports.setSyncBreakLocalMsg = function(_type, _isDel){
	setSyncBreakLocalMsg(_type, _isDel);
};

/**
 * 删除一条本地消息（通过id）
 * @param _id
 */
function delLocalMessage(_id){
	var _sql_del = "delete from local_message where id="
		+ tools.formatSqlValue(_id) + ";";
	return localdb.executeSync(_sql_del);
}

// 删除一条本地消息（通过id）
module.exports.delLocalMessage = function(_id){
	return delLocalMessage(_id);	
};

/**
 * 删除一条本地消息（通过type和notifyid）
 * @param _type
 * @param _notifyId
 * @returns
 */
function delLocalMsgByTypeAndNotifyId(_type, _notifyId){
	var _sql_del = "delete from local_message where notifyid="
		+ _notifyId + " and type="
		+ _type + ";";
	return localdb.executeSync(_sql_del);
}

// 删除一条本地消息（通过type和notifyid）
module.exports.delLocalMsgByTypeAndNotifyId = function(_id){
	return delLocalMsgByTypeAndNotifyId(_id);	
};

/**
 * 查询一条本地消息（通过id）
 * @param _id
 * @returns
 */
function queryLocalMessageById(_id){
	var sql ="select * from local_message where id=" + tools.formatSqlValue(_id) +  ";";
	var _data=localdb.querySync(sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data[0];
}

// 查询一条本地消息（通过id）
module.exports.queryLocalMessageById = function(_id){
	return queryLocalMessageById(_id);	
};

/**
 * 查询一条本地消息（通过notifyId）
 * @param _notifyId
 * @returns
 */
function queryLocalMsgByNotifyId(_notifyId){
	var sql ="select * from local_message where notifyid=" + _notifyId +  ";";
	var _data=localdb.querySync(sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data[0];
}

// 查询一条本地消息（通过notifyId）
module.exports.queryLocalMsgByNotifyId = function(_notifyId){
	return queryLocalMsgByNotifyId(_notifyId);	
};

/**
 * 查询本地消息（通过提醒时间）
 * @param _remindTime
 * @returns
 */
function queryLocalMsgByRemidTime(_remindTime){
	var _sql ="select * from local_message where remindtime<"
		+ _remindTime + " order by remindtime DESC;";
	var _data = localdb.querySync(_sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data;
}

// 查询本地消息（通过提醒时间）
module.exports.queryLocalMsgByRemidTime = function(_remindTime){
	return queryLocalMsgByRemidTime(_remindTime);	
};

/**
 * 查询本地消息（通过提醒时间和标记）
 * @param _remindTime
 * @param _flag
 * @returns
 */
function queryLocalMsgByRemidTimeAndFlag(_remindTime, _flag){
	var _sql ="select * from local_message where remindtime<"
		+ _remindTime + " and flag=" + _flag + ";";
	var _data = localdb.querySync(_sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data;
}

// 查询本地消息（通过提醒时间和标记）
module.exports.queryLocalMsgByRemidTimeAndFlag = function(_remindTime, _flag){
	return queryLocalMsgByRemidTimeAndFlag(_remindTime, _flag);	
};

/**
 * 查询本地未读消息数量
 * @returns
 */
function queryLocalUnReadMsgNum(){
	var _nowTime = new Date();
	var _sql ="select count(*) as num from (select * from local_message where remindtime<"
		+ _nowTime.getTime() + " and flag=0 group by type);";
	var _data = localdb.querySync(_sql);
	print("queryLocalUnReadMsgNum + _data " + JSON.stringify(_data));
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data[0];
}

// 查询本地未读消息数量
module.exports.queryLocalUnReadMsgNum = function(){
	return queryLocalUnReadMsgNum();	
};
