/**
 * 数据同步
 * 
 * @Author : Cydow
 * @Timestamp : 2017-02-22
 */
var deviceone = require("deviceone");
var tools = require("tools");

//数据库
var localdb = deviceone.mm("M0026_sqlite", "localdb", "app");

/**
 * 获取更新记录的sql语句
 * @param _updRecord
 * @returns
 */
function getUpdRecordDataSql(_updRecord){
	var _sql = "";
	//计数值
	var _count = _updRecord.count;
	//记录更新（新增）
	var recordData = query_updrecordDataById(_updRecord.id);
	if(null == recordData) {
		_sql = getInsertUpdRecordDataSql({
			id : _updRecord.id,
			moduleid: _updRecord.moduleid,
			updtime : _updRecord.updtime,
			recordtype : _updRecord.recordtype,
			count : _count, 
			datatype : _updRecord.datatype
		});
	} else {
		_sql = getUpdUpdRecordDataSql({
			id : _updRecord.id,
			updtime : _updRecord.updtime, 
			recordtype : _updRecord.recordtype, 
			count : _count
		});
	}
	return _sql;
}

// 获取更新记录的sql语句
module.exports.getUpdRecordDataSql = function(_updRecord){
	return getUpdRecordDataSql(_updRecord);	
};

/**
 * 获取修改更新记录的sql语句
 * @param _id
 * @returns
 */
function getModifyUpdRecordDataSql(_updRecord){
	var _sqlList = [];
	var _sql = "";
	//计数值
	var _count = 1;
	//获取更新计数器的值
	var _updCount = query_updCountData(_updRecord.moduleid);
	if(null == _updCount){
		_sql = getInsertUpdCountDataSql({
			moduleid: _updRecord.moduleid,
			updatetime: _updRecord.updtime,
			updatecount: _count
		});
	} else {
		_count = parseInt(_updCount.updatecount) + 1;
		_sql = getUpdUpdCountDataSql({
			updtime: _updRecord.updtime,
			updcount: _count,
			moduleid: _updRecord.moduleid
		});
	}
	_sqlList.push(_sql);
	
	//记录更新（新增）
	var recordData = query_updrecordDataById(_updRecord.id);
	if(null == recordData) {
		_sql = getInsertUpdRecordDataSql({
			id : _updRecord.id,
			moduleid: _updRecord.moduleid,
			updtime : _updRecord.updtime,
			recordtype : _updRecord.recordtype,
			count : _count, 
			datatype : _updRecord.datatype
		});
	} else {
		_sql = getUpdUpdRecordDataSql({
			id : _updRecord.id,
			updtime : _updRecord.updtime, 
			recordtype : _updRecord.recordtype, 
			count : _count
		});
	}
	_sqlList.push(_sql);
	return _sqlList;
}

//获取修改更新记录的sql语句
module.exports.getModifyUpdRecordDataSql = function(_updRecord){
	return getModifyUpdRecordDataSql(_updRecord);	
};

/**
 * 设置更新记录相关信息
 * @param _updRecord
 */
function set_updRecord(_updRecord){
	//计数值
	var _count = 1;
	//获取更新计数器的值
	var _updCount = query_updCountData(_updRecord.moduleid);
	if(null == _updCount){
		insert_updCountData({
			moduleid: _updRecord.moduleid,
			updatetime: _updRecord.updtime,
			updatecount: _count
		});
	} else {
		_count = parseInt(_updCount.updatecount) + 1;
		update_updCountData(_updRecord.updtime, _count, _updRecord.moduleid);
	}
	//记录更新（新增）
	var recordData = query_updrecordDataById(_updRecord.id);
	if(null == recordData) {
		insert_updRecordData({
			id : _updRecord.id,
			moduleid: _updRecord.moduleid,
			updtime : _updRecord.updtime,
			recordtype : _updRecord.recordtype,
			count : _count, 
			datatype : _updRecord.datatype
		});
	} else {
		update_updRecordData({
			id : _updRecord.id,
			updtime : _updRecord.updtime, 
			recordtype : _updRecord.recordtype, 
			count : _count
		});
	}
}

//保存本地使用记录数据
module.exports.setUpdRecord = function(_updRecord){
	set_updRecord(_updRecord);
};

/**
 * 获取插入更新计数器的sql语句
 * @param _updData
 * @returns
 */
function getInsertUpdCountDataSql(_updData){
	var _sql_insert = "insert into datasync_updcount(moduleid, updatetime, " +
		"updatecount) values("
		+ tools.formatSqlValue(_updData.moduleid) + ","
		+ _updData.updatetime + ","
		+ _updData.updatecount + ");";
	return _sql_insert;
}

/**
 * 向更新计数器表（datasync_updcount）中插入数据
 * @param _updData
 */
function insert_updCountData(_updData){
	var _sql_insert = getInsertUpdCountDataSql(_updData);
	localdb.executeSync(_sql_insert);
}

/**
 * 获取更新更新计数器的sql语句
 * @param _updTime
 * @param _updCount
 * @param _moduleId
 * @returns
 */
function getUpdUpdCountDataSql(_updData){
	var _sql_update = "update datasync_updcount set " 
		+ "updatetime=" + _updData.updtime 
		+ ", updatecount=" + _updData.updcount 
		+ " where moduleid=" + tools.formatSqlValue(_updData.moduleid) + ";";
	return _sql_update;
}

// 获取更新更新计数器的sql语句
module.exports.getUpdUpdCountDataSql = function(_updData){
	return getUpdUpdCountDataSql(_updData);
}

/**
 * 更新计数器表（datasync_updcount）中的数据
 * @param _updTime
 * @param _updCount
 * @param _moduleId
 */
function update_updCountData(_updTime, _updCount, _moduleId){
	var _sql_updcount_update = getUpdUpdCountDataSql({
		updtime: _updTime,
		updcount: _updCount,
		moduleid: _moduleId
	});
	localdb.executeSync(_sql_updcount_update);
}

/**
 * 查询更新计数器表（datasync_updcount）中的数据
 * @returns
 */
function query_updCountData(_moduleId){
	var _sql_updcount_query = "select * from datasync_updcount where moduleid="
		+ tools.formatSqlValue(_moduleId) + ";";
	var _data_updcount = localdb.querySync(_sql_updcount_query);
	if(null == _data_updcount || _data_updcount.length <= 0){
		return null;
	}
	return _data_updcount[0];
}

//查询更新计数器
module.exports.query_updCount = function(_moduleId){
	return query_updCountData(_moduleId);
}

/**
 * 获取插入更新记录的sql语句
 * @param _updRecord
 * @returns
 */
function getInsertUpdRecordDataSql(_updRecord){
	var _sql_insert = "insert into datasync_updrecord(id, moduleid, recordtime, " +
		" recordtype, updatecount, datatype) values(" 
		+ tools.formatSqlValue(_updRecord.id) + ","
		+ tools.formatSqlValue(_updRecord.moduleid) + ","
		+ _updRecord.updtime + ","
		+ _updRecord.recordtype + ","
		+ _updRecord.count + ","
		+ tools.formatSqlValue(_updRecord.datatype) + ");";
	return _sql_insert;
}

/**
 * 向更新记录表（datasync_updrecord）中插入数据
 * @param _updRecord
 */
function insert_updRecordData(_updRecord){
	var _sql_updrecord_insert = getInsertUpdRecordDataSql(_updRecord);
	localdb.executeSync(_sql_updrecord_insert);
}

/**
 * 获取更新更新记录的sql语句
 * @param _updRecord
 * @returns
 */
function getUpdUpdRecordDataSql(_updRecord){
	var _sql_update = "update datasync_updrecord set " 
		+ "recordtime=" + _updRecord.updtime 
		+ ", recordtype=" + _updRecord.recordtype 
		+ ", updatecount=" + _updRecord.count
		+ " where id=" + tools.formatSqlValue(_updRecord.id) + ";";
	return _sql_update;
}

/**
 * 更新记录表（datasync_updrecord）中的数据
 * @param _updRecord
 */
function update_updRecordData(_updRecord){
	var _sql_updrecord_update = getUpdUpdRecordDataSql(_updRecord);
	localdb.executeSync(_sql_updrecord_update);
}

/**
 * 通过id和moduleid删除更新记录
 * @param _id
 * @param _moduleId
 * @returns
 */
function delUpdrecordDataByIdAndModuleId(_id, _moduleId){
	var _sql_updrecord_query = "delete from datasync_updrecord where id=" 
		+ tools.formatSqlValue(_id)
		+ " and moduleid=" + tools.formatSqlValue(_moduleId)
		+ ";";
	var _data_updrecord = localdb.querySync(_sql_updrecord_query);
	return _data_updrecord;
}

//通过id和moduleid删除更新记录
module.exports.delUpdrecordDataByIdAndModuleId = function(_id, _moduleId){
	return delUpdrecordDataByIdAndModuleId(_id, _moduleId);
}

/**
 * 通过id查询更新记录表（datasync_updrecord）中的数据
 * @param _id
 * @returns
 */
function query_updrecordDataById(_id){
	var _sql_updrecord_query = "select * from datasync_updrecord where id=" 
		+ tools.formatSqlValue(_id) + ";";
	var _data_updrecord = localdb.querySync(_sql_updrecord_query);
	if(null == _data_updrecord || _data_updrecord.length <= 0){
		return null;
	}
	
	return _data_updrecord[0];
}

//通过id查询更新记录
module.exports.query_updrecordById = function(_id){
	return query_updrecordDataById(_id);
}

/**
 * 通过synccount查询更新记录表（datasync_updrecord）中的数据
 * @param _id
 * @returns
 */
function query_updrecordDataBySyncCount(_syncCount, _moduleId){
	var _sql_updrecord_query = "select * from datasync_updrecord where updatecount>" 
		+ _syncCount + " and moduleid=" + tools.formatSqlValue(_moduleId)
		+ " order by updatecount ASC;";
	var _data_updrecord = localdb.querySync(_sql_updrecord_query);
	return _data_updrecord;
}

//通过更新计数查询更新记录
module.exports.query_updrecordBySyncCount = function(_syncCount, _moduleId){
	return query_updrecordDataBySyncCount(_syncCount, _moduleId);
}

/**
 * 向同步计数器表（datasync_synccount）中插入数据
 * @param _syncCountData
 */
function insert_synccountData(_syncCountData){
	var _sql_synccount_insert = "insert into datasync_synccount(moduleid, updatetime, " +
			"synccount, syncid) values("
		+ tools.formatSqlValue(_syncCountData.moduleid) + ","
		+ _syncCountData.updatetime + ","
		+ _syncCountData.synccount + ","
		+ tools.formatSqlValue(_syncCountData.syncid) + ");";
	localdb.executeSync(_sql_synccount_insert);
}

/**
 * 更新同步计数器表（datasync_synccount）中的数据
 * @param _syncCountData
 */
function update_synccountData(_syncCountData){
	var _sql_synccount_update = "update datasync_synccount set " 
		+ "updatetime=" + _syncCountData.updatetime 
		+ ", synccount=" + _syncCountData.synccount 
		+ " where syncid=" + tools.formatSqlValue(_syncCountData.syncid) + " and moduleid="
		+ tools.formatSqlValue(_syncCountData.moduleid) + ";";
	localdb.executeSync(_sql_synccount_update);
}

/**
 * 查询同步计数器表（datasync_synccount）中的数据
 * @param _syncId
 * @param _moduleId
 * @returns
 */
function query_synccountData(_syncId, _moduleId){
	var _sql_synccount_query = "select * from datasync_synccount where syncid=" 
		+ tools.formatSqlValue(_syncId) + " and moduleid="
		+ tools.formatSqlValue(_moduleId) + ";";
	var _data_synccount = localdb.querySync(_sql_synccount_query);
	return _data_synccount[0];
}

//通过同步方id查询同步计数器
module.exports.query_synccountOne = function(_syncid, _moduleId){
	return query_synccountData(_syncid, _moduleId);
};

/**
 * 更新同步计数器表(datasync_synccount) -- 插入或更新
 * @param _syncData
 */
function sync_synccountData(_syncCountData){
	var _syncCount = query_synccountData(_syncCountData.syncid, _syncCountData.moduleid);
	if(_syncCount == null){
		insert_synccountData(_syncCountData);
		return ;
	}
	update_synccountData(_syncCountData);
}

//更新同步计数器
module.exports.sync_synccount = function(_syncData){
	return sync_synccountData(_syncData);
};

/**
 * 向同步记录表（datasync_syncrecord）中插入数据
 * @param _syncData
 */
function insert_syncrecordData(_syncData){
	var _sql_syncrecord_insert = "insert into datasync_syncrecord(dataid, moduleid, syncid, " +
			" synctime, syncstate) values(" 
		+ tools.formatSqlValue(_syncData.id) + ","
		+ tools.formatSqlValue(_syncData.moduleid) + ","
		+ tools.formatSqlValue(_syncData.syncid) + ","
		+ _syncData.synctime + ","
		+ _syncData.syncstate + ");";
	localdb.executeSync(_sql_syncrecord_insert);
}

/**
 * 更新同步记录表（datasync_syncrecord）中的数据
 * @param _syncData
 */
function update_syncrecordData(_syncData){
	var _sql_syncrecord_update = "update datasync_syncrecord set " 
		+ "synctime=" + _syncData.synctime 
		+ ", syncstate=" + _syncData.syncstate 
		+ " where dataid=" 
		+ tools.formatSqlValue(_syncData.id)
		+ " and syncid="
		+ tools.formatSqlValue(_syncData.syncid)
		+ " and moduleid="
		+ tools.formatSqlValue(_syncData.moduleid)
		+ ";";
	localdb.executeSync(_sql_syncrecord_update);
}

/**
 * 查询同步记录表（datasync_syncrecord）中的数据
 * @param _id
 * @param _syncId
 * @param _moduleId
 * @returns
 */
function query_syncrecordData(_id, _syncId, _moduleId){
	var _sql_syncrecord_query = "select * from datasync_syncrecord where dataid=" 
		+ tools.formatSqlValue(_id) 
		+ " and syncid=" 
		+ tools.formatSqlValue(_syncId)
		+ " and moduleid="
		+ tools.formatSqlValue(_moduleId)
		+ ";";
	var _data_syncrecord = localdb.querySync(_sql_syncrecord_query);
	
	return _data_syncrecord[0];
}

/**
 * 查询同步记录表（datasync_syncrecord）中的数据（通过syncstate）
 * @param _id
 * @param _state
 * @returns
 */
function query_syncrecordDataByState(_state, _syncId, _moduleId){
	var _sql_syncrecord_query = "select * from datasync_syncrecord where syncstate=" 
		+ _state + " and syncid=" 
		+ tools.formatSqlValue(_syncId)
		+ " and moduleid="
		+ tools.formatSqlValue(_moduleId)
		+ ";";
	var _data_syncrecord = localdb.querySync(_sql_syncrecord_query);
	
	return _data_syncrecord;
}

//根据同步状态查询同步记录
module.exports.query_syncrecordByState = function(_syncId, _moduleId){
	return query_syncrecordDataByState(2, _syncId, _moduleId);
}

/**
 * 查询同步记录表（datasync_syncrecord）中的数据（通过id、syncstate及moduleid）
 * @param _id
 * @param _moduleId
 * @param _state
 * @returns
 */
function querySyncRecordDataByIdAndState(_id, _moduleId, _state){
	var _sql_syncrecord_query = "select * from datasync_syncrecord where syncstate=" 
		+ _state + " and id=" 
		+ tools.formatSqlValue(_id) + " and moduleid="
		+ tools.formatSqlValue(_moduleId) + ";";
	var _data_syncrecord = localdb.querySync(_sql_syncrecord_query);
	if(null == _data_syncrecord || _data_syncrecord.length <= 0){
		return null;
	}
	return _data_syncrecord;
}

//根据id和moduleid查询失败的同步记录
module.exports.querySyncRecordDataByIdAndState = function(_id, _moduleId){
	return querySyncRecordDataByIdAndState(_id, _moduleId, 2);
}

/**
 * 更新同步记录表(datasync_syncrecord) -- 插入或更新
 * @param _syncData
 */
function sync_syncrecordData(_syncData){
	var _syncRecord = query_syncrecordData(_syncData.id, _syncData.syncid, _syncData.moduleid);
	if(_syncRecord == null){
		insert_syncrecordData(_syncData);
		return ;
	}
	update_syncrecordData(_syncData);
}

//更新同步记录
module.exports.sync_syncrecord = function(_syncData){
	return sync_syncrecordData(_syncData);
};
