/**
 * 系统日历
 * 
 * @Author : Cydow
 * @Timestamp : 2018-01-10
 */
//引入组件库
var deviceone = require("deviceone");

//工具
var tools = require("tools");
// 常量
var constants = require("constants");
//数据库
var localdb = deviceone.mm("M0026_sqlite", "localdb", "app");

/**
 * 向syscalendar表中插入一条数据
 * @param _changedData
 */
function insertSysCalendarData(_changedData){
	var _nowTime = new Date();
	if(null == _changedData.source){
		_changedData.source = constants.getConstants("calendarSource_sys");
	}
	var _sql_insert = "insert into syscalendar(id, taskid, source, createtime) values("
		+ tools.formatSqlValue(_changedData.id) + ","
		+ tools.formatSqlValue(_changedData.taskid) + ", "
		+ tools.formatSqlValue(_changedData.source) + ", "
		+ _nowTime.getTime() + ");";
	var isSuccess = localdb.executeSync(_sql_insert);
	return isSuccess;
}

//向syscalendar表中插入一条数据
module.exports.insertSysCalendarData = function(_changedData){
	return insertSysCalendarData(_changedData);	
};

/**
 * 更新系统日历提醒记录（通过taskid和source）
 * @param _changedData
 */
function updSysCalRecordByTaskIdAndSource(_changedData){
	var _sql_upd = "update syscalendar set id="
		+ tools.formatSqlValue(_changedData.id) + " where taskid="
		+ tools.formatSqlValue(_changedData.taskid) + " and source="
		+ tools.formatSqlValue(_changedData.source) + ";";
	var isSuccess = localdb.executeSync(_sql_upd);
	return isSuccess;
}

// 更新系统日历提醒记录（通过taskid和source）
module.exports.updSysCalRecordByTaskIdAndSource = function(_changedData){
	return updSysCalRecordByTaskIdAndSource(_changedData);	
};

/**
 * 获取id的字符串
 * @param _ids
 * @returns
 */
function getIdStr(_ids){
	var _idSqlStr = "";
	if(null != _ids && _ids.length > 0){
		for(var i=0; i<_ids.length; i++){
			if(null == _ids[i] ||
					"" == _ids[i]){
				continue;
			}
			_idSqlStr += tools.formatSqlValue(_ids[i]);
			if(i != _ids.length - 1){
				_idSqlStr += ",";
			}
		}
	}
	return _idSqlStr;
}

/**
 * 删除系统日历记录（通过id）
 * @param _ids
 */
function delSysCalendarRecordById(_ids){
	var _idSqlStr = getIdStr(_ids);
	if("" != _idSqlStr){
		var _sql_del = "delete from syscalendar where id in (" + _idSqlStr + ");";
		localdb.executeSync(_sql_del);
	}
}

// 删除系统日历记录（通过id）
module.exports.delSysCalendarRecordById = function(_ids){
	delSysCalendarRecordById(_ids);	
};

/**
 * 删除系统日历记录（通过taskid）
 * @param _ids
 */
function delSysCalendarRecordByTaskId(_ids){
	var _idSqlStr = getIdStr(_ids);
	if("" != _idSqlStr){
		var _sql_del = "delete from syscalendar where taskid in (" + _idSqlStr + ");";
		localdb.executeSync(_sql_del);
	}
}

// 删除系统日历记录（通过taskid）
module.exports.delSysCalendarRecordByTaskId = function(_ids){
	delSysCalendarRecordByTaskId(_ids);	
};

/**
 * 删除系统日历记录（通过id和source）
 * @param _id
 * @param _source
 */
function delSysCalRecordByIdAndSource(_id, _source){
	var _sql_del = "delete from syscalendar where id="
		+ tools.formatSqlValue(_id) + " and source="
		+ tools.formatSqlValue(_source) + ";";
	localdb.executeSync(_sql_del);
}

// 删除系统日历记录（通过id和source）
module.exports.delSysCalRecordByIdAndSource = function(_id, _source){
	delSysCalRecordByIdAndSource(_id, _source);	
};

/**
 * 通过id从syscalendar表中获取指定的一条数据
 * @param _id
 * @returns
 */
function querySysCalendarOneData(_id){
	var sql ="select * from syscalendar where id=" + tools.formatSqlValue(_id) +  ";";
	var _data=localdb.querySync(sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data[0];
}

//通过id获取一条待办数据
module.exports.querySysCalendarOneData = function(_id){
	return querySysCalendarOneData(_id);	
};

/**
 * 查询系统日历记录数据（通过taskid和source）
 * @param _taskId
 * @param _source
 * @returns
 */
function querySysCalByTaskIdAndSource(_taskId, _source){
	var sql ="select * from syscalendar where taskid="
		+ tools.formatSqlValue(_taskId) + " and source="
		+ tools.formatSqlValue(_source) + ";";
	var _data=localdb.querySync(sql);
	if(_data == null || _data.length <= 0){
		return null;
	}
	return _data[0];
}

// 查询系统日历记录数据（通过taskid和source）
module.exports.querySysCalByTaskIdAndSource = function(_taskId, _source){
	return querySysCalByTaskIdAndSource(_taskId, _source);	
};

/**
 * 异步查询系统日历记录数据（通过taskid和source）
 * @param _taskId
 * @param _source
 * @param _callFunc
 * @returns
 */
function querySysCalByTaskIdAsync(_taskId, _source, _callFunc){
	var _sql ="select * from syscalendar where" +
			" source=? and taskid=?;";
	localdb.query(_sql, [_source, _taskId], function(_data){
		if(_data == null || _data.length <= 0){
			_callFunc.call(this, null);
		} else {
			_callFunc.call(this, _data[0]);
		}
	});
}

//异步查询系统日历记录数据（通过taskid和source）
module.exports.querySysCalByTaskIdAsync = function(_taskId, _source, _callFunc){
	querySysCalByTaskIdAsync(_taskId, _source, _callFunc);	
};
