/**
 * 操作使用记录（使用本地记录表和库外共享记录表）
 * 
 * @Author : Cydow
 * @Timestamp : 2017-02-21
 */
var deviceone = require("deviceone");
var constants = require("constants"); // 常量
var dbclsyncdata = require("dbclsyncdata"); // 数据同步
var tools = require("tools"); // 工具
var objectId = require("objectId"); // 生成objectId
var dbcycle = require("dbcycle");

// 数据库
var localdb = deviceone.mm("M0026_sqlite", "localdb", "app");

// -----------------------------------------
// 本地使用记录
/**
 * 获取插入使用记录的sql语句
 * @param _changedData
 * @returns
 */
function getInsertUseRecordSql(_changedData){
	if (_changedData.id == null) {
		_changedData.id = objectId.getObjectId();
	}
	if (_changedData.taskid == null) {
		_changedData.taskid = "";
	}
	if (_changedData.isid == null) {
		_changedData.isid = "";
	}
	if (_changedData.personname == null) {
		_changedData.personname = "";
	}
	if (_changedData.applyevent == null) {
		_changedData.applyevent = "";
	}
	if (_changedData.applyproc == null) {
		_changedData.applyproc = "";
	}
	if (_changedData.destorganization == null) {
		_changedData.destorganization = "";
	}
	if (_changedData.sharechannel == null) {
		_changedData.sharechannel = "";
	}
	var _nowTime = new Date();
	if (_changedData.labeltime == null) {
		_changedData.labeltime = _nowTime.getTime();
	}
	if (_changedData.syscall == null) {
		_changedData.syscall = constants.getConstants("syscall_system_2");
	}
	if (_changedData.processmark == null) {
		_changedData.processmark = constants.getConstants("processMark_false");
	}
	var _sql_insert = "insert into userecord(id, taskid, isid, personname, applyevent, applyproc, destorganization, sharechannel, useformat, syscall, labeltime, processmark) values("
			+ tools.formatSqlValue(_changedData.id) + ","
			+ tools.formatSqlValue(_changedData.taskid) + ","
			+ tools.formatSqlValue(_changedData.isid) + ","
			+ tools.formatSqlValue(_changedData.personname) + ","
			+ tools.formatSqlValue(_changedData.applyevent) + ","
			+ tools.formatSqlValue(_changedData.applyproc) + ","
			+ tools.formatSqlValue(_changedData.destorganization) + ","
			+ tools.formatSqlValue(_changedData.sharechannel) + ","
			+ _changedData.useformat + ","
			+ tools.formatSqlValue(_changedData.syscall) + ","
			+ _changedData.labeltime + ","
			+ tools.formatSqlValue(_changedData.processmark) + ");";
	return _sql_insert;
}

/**
 * 获取插入使用记录的所有sql语句
 * @param _changedData
 * @param _sqlList
 * @param _isRecord
 * @returns
 */
function getInsertUseRecordAllSqls(_changedData, _sqlList, _isRecord){
	// 获取插入使用记录的sql语句
	var _sql = getInsertUseRecordSql(_changedData);
	_sqlList.push(_sql);
	
	// 更新记录
	var _updRecord = {
			id: _changedData.id,
			moduleid: _changedData.moduleid,
			recordtype: constants.getConstants("recordType_create"), 
			datatype: constants.getConstants("datatype_userecord")
		};
	// 设置更新记录
	if(_isRecord){
		_sql = dbclsyncdata.getUpdRecordUnHandleSql(_updRecord);
	} else {
		_sql = dbclsyncdata.getUpdRecordSql(_updRecord);
	}
	_sqlList.push(_sql);
}

/**
 * 插入使用记录数据
 * @param _changedData
 * @param _isRecord
 * @returns
 */
function insertUseRecordData(_changedData, _isRecord) {
	if(null != _changedData.id){
		var _data = queryUseRecordOneData(_changedData.id);
		if(null != _data){
			return true;
		}
	}
	var _sqlList = [];
	getInsertUseRecordAllSqls(_changedData, _sqlList, _isRecord);
	return localdb.executeSync1({sqls: _sqlList});
}

// 插入本地使用记录数据
module.exports.insertUseRecordData = function(_changedData) {
	return insertUseRecordData(_changedData, true);
};

// 插入本地使用记录数据-数据同步
module.exports.insertUseRecordSyncData = function(_changedData) {
	print("insertUseRecordSyncData " + JSON.stringify(_changedData));
	return insertUseRecordData(_changedData, false);
};

/**
 * 获取删除使用记录的sql语句
 * @param _changedData
 * @param _sqlList
 * @param _isRecord
 * @returns
 */
function getDelUseRecordDataSql(_changedData, _sqlList, _isRecord){
	var _id = _changedData.id;
	// 删除使用记录
	var _sql = "delete from userecord where id="
		+ tools.formatSqlValue(_id) + ";";
	_sqlList.push(_sql);
	
	// 更新记录
	var _updRecord = {
			id: _id,
			moduleid: _changedData.moduleid,
			recordtype: constants.getConstants("recordType_delete"), 
			datatype: constants.getConstants("datatype_userecord")
		};
	// 设置更新记录
	if(_isRecord){
		_sql = dbclsyncdata.getUpdRecordUnHandleSql(_updRecord);
	} else {
		_sql = dbclsyncdata.getUpdRecordSql(_updRecord);
	}
	_sqlList.push(_sql);
}

// 获取删除使用记录的sql语句
module.exports.getDelUseRecordDataSql = function(_changedData, _sqlList, _isRecord) {
	return getDelUseRecordDataSql(_changedData, _sqlList, _isRecord);
};

/**
 * 删除使用记录
 * @param _id
 * @param _moduleId
 * @param _isRecord
 * @returns
 */
function delUseRecordData(_id, _moduleId, _isRecord) {
	var _sqlList = [];
	getDelUseRecordDataSql({
		id: _id,
		moduleid: _moduleId
	}, _sqlList, _isRecord);
	var _isSuc = localdb.executeSync1({
		sqls: _sqlList
	});
	return _isSuc;
}

// 删除本地使用记录
module.exports.delUseRecordData = function(_id, _moduleId) {
	return delUseRecordData(_id, _moduleId, true);
};

//删除本地使用记录-数据同步
module.exports.delUseRecordSyncData = function(_id, _moduleId) {
	return delUseRecordData(_id, _moduleId, false);
};

/**
 * 获取删除使用记录的所有sql语句
 * @param _changedData
 * @param _sqlList
 * @param _isRecord
 * @returns
 */
function getDelUseRecordAllSqls(_changedData, _sqlList, _isRecord){
	var _useRecordList = queryUseRecordAllDataByTaskId(_changedData.id);
	if (null != _useRecordList &&
			_useRecordList.length > 0){
		var _len = _useRecordList.length;
		for (var i = 0; i < _useRecordList.length; i++) {
			if (_useRecordList[i] == null)
				continue;
			var _sql = getDelUseRecordDataSql({
				id: _useRecordList[i].id,
				moduleid: _changedData.moduleid
			}, _sqlList, _isRecord);
		}
	}
}

//获取删除使用记录的所有sql语句
module.exports.getDelUseRecordAllSqls = function(_changedData, _sqlList, _isRecord){
	getDelUseRecordAllSqls(_changedData, _sqlList, _isRecord);	
};

/**
 * 删除指定数据（id）的所有使用记录数据
 * @param _id
 * @param _moduleId
 * @param _isRecord
 * @returns
 */
function delUseRecordAllDataByTaskId(_id, _moduleId, _isRecord) {
	var _sqlList = [];
	getDelUseRecordAllSqls({
		id: _id,
		moduleid: _moduleId
	}, _sqlList, _isRecord);
	var _isSuc = localdb.executeSync1({
		sqls: _sqlList
	});
	return _isSuc;
}

//删除指定数据（id）的所有使用记录数据
module.exports.delUseRecordAllData = function(_id, _moduleId) {
	return delUseRecordAllDataByTaskId(_id, _moduleId, true);
};

/**
 * 通过id从userecord表中获取指定的一条数据
 * 
 * @param _id
 * @returns
 */
function queryUseRecordOneData(_id) {
	var sql = "select * from userecord where id=" 
		+ tools.formatSqlValue(_id) + ";";
	var _data = localdb.querySync(sql);
	if (_data == null || _data.length <= 0) {
		return null;
	}
	return _data[0];
}

// 查询使用记录数据
module.exports.getUseRecordOneData = function(_id) {
	return queryUseRecordOneData(_id);
};

/**
 * 通过taskid从userecord表中获取所有使用记录数据
 * 
 * @param _taskId
 * @returns
 */
function queryUseRecordAllDataByTaskId(_taskId) {
	var sql = "select * from userecord where taskid=" 
		+ tools.formatSqlValue(_taskId) + " order by labeltime DESC;";
	var _data = localdb.querySync(sql);
	if (_data == null || _data.length <= 0) {
		return null;
	}
	return _data;
}

//通过taskid从userecord表中获取所有使用记录数据
module.exports.getUseRecordAllDataByTaskId = function(_taskId) {
	return queryUseRecordAllDataByTaskId(_taskId);
};

// -----------------------------------------
// 库外共享记录

/**
 * 获取插入共享记录的sql语句
 * @param _changedData
 * @returns
 */
function getInsertShareRecordSql(_changedData){
	if (_changedData.id == null) {
		_changedData.id = objectId.getObjectId();
	}
	if (_changedData.personname == null) {
		_changedData.personname = "";
	}
	if (_changedData.applyevent == null) {
		_changedData.applyevent = "";
	}
	if (_changedData.applyproc == null) {
		_changedData.applyproc = "";
	}
	if (_changedData.destorganization == null) {
		_changedData.destorganization = "";
	}
	if (_changedData.sharechannel == null) {
		_changedData.sharechannel = "";
	}
	var _nowTime = new Date();
	if (_changedData.labeltime == null) {
		_changedData.labeltime = _nowTime.getTime();
	}
	var _sql_insert = "insert into sharerecord(id, taskid, personname, applyevent, applyproc, destorganization, sharechannel, shareformat, labeltime) values("
			+ tools.formatSqlValue(_changedData.id) + ","
			+ tools.formatSqlValue(_changedData.taskid) + ","
			+ tools.formatSqlValue(_changedData.personname) + ","
			+ tools.formatSqlValue(_changedData.applyevent) + ","
			+ tools.formatSqlValue(_changedData.applyproc) + ","
			+ tools.formatSqlValue(_changedData.destorganization) + ","
			+ tools.formatSqlValue(_changedData.sharechannel) + ","
			+ _changedData.shareformat  + "," 
			+ _changedData.labeltime + ");";
	return _sql_insert;
}

/**
 * 获取插入共享记录的所有sql语句
 * @param _changedData
 * @param _sqlList
 * @param _isRecord
 * @returns
 */
function getInsertShareRecordAllSqls(_changedData, _sqlList, _isRecord){
	// 获取插入共享记录的sql语句
	var _sql = getInsertShareRecordSql(_changedData);
	_sqlList.push(_sql);
	
	// 更新记录
	var _updRecord = {
			id: _changedData.id,
			moduleid: _changedData.moduleid,
			recordtype: constants.getConstants("recordType_create"), 
			datatype: constants.getConstants("datatype_sharerecord")
		};
	// 设置更新记录
	if(_isRecord){
		_sql = dbclsyncdata.getUpdRecordUnHandleSql(_updRecord);
	} else {
		_sql = dbclsyncdata.getUpdRecordSql(_updRecord);
	}
	_sqlList.push(_sql);
}

/**
 * 插入共享记录数据
 * @param _changedData
 * @param _isRecord
 * @returns
 */
function insertShareRecordData(_changedData, _isRecord) {
	var _sqlList = [];
	// 获取插入共享记录的所有sql语句
	getInsertShareRecordAllSqls(_changedData, _sqlList, _isRecord)
	var _isSuc = localdb.executeSync1({sqls: _sqlList});
	return _isSuc;
}

// 插入共享记录数据
module.exports.insertShareRecordData = function(_changedData) {
	return insertShareRecordData(_changedData, true);
};

/**
 * 插入共享记录数据
 * @param _changedData
 * @param _isRecord
 * @returns
 */
function insertShareRecordSyncData(_changedData, _isRecord) {
	var _isSuc = true;
	if(null != _changedData.id){
		var _data = queryShareRecordOneData(_changedData.id);
		if(null != _data){
			return _isSuc;
		}
	}
	var _sqlList = [];
	// 获取插入共享记录的所有sql语句
	getInsertShareRecordAllSqls(_changedData, _sqlList, _isRecord)
	_isSuc = localdb.executeSync1({sqls: _sqlList});
	return _isSuc;
}

// 插入共享记录数据-数据同步
module.exports.insertShareRecordSyncData = function(_changedData) {
	return insertShareRecordSyncData(_changedData, false);
};

/**
 * 获取删除共享记录的sql语句
 * @param _changedData
 * @param _sqlList
 * @param _isRecord
 * @returns
 */
function getDelShareRecordDataSql(_changedData, _sqlList, _isRecord){
	var _id = _changedData.id;
	// 删除共享记录
	var _sql = "delete from sharerecord where id="
		+ tools.formatSqlValue(_id) + ";";
	_sqlList.push(_sql);
	
	// 更新记录
	var _updRecord = {
			id: _id,
			moduleid: _changedData.moduleid,
			recordtype: constants.getConstants("recordType_delete"), 
			datatype: constants.getConstants("datatype_sharerecord")
		};
	// 设置更新记录
	if(_isRecord){
		_sql = dbclsyncdata.getUpdRecordUnHandleSql(_updRecord);
	} else {
		_sql = dbclsyncdata.getUpdRecordSql(_updRecord);
	}
	_sqlList.push(_sql);
}

//获取删除共享记录的sql语句
module.exports.getDelShareRecordDataSql = function(_changedData, _sqlList, _isRecord){
	return getDelShareRecordDataSql(_changedData, _sqlList, _isRecord);	
};

/**
 * 删除共享记录
 * @param _id
 * @param _moduleId
 * @param _isRecord
 * @returns
 */
function delShareRecordData(_id, _moduleId, _isRecord) {
	var _sqlList = [];
	var _sql_sharerecord_del = getDelShareRecordDataSql({
		id: _id,
		moduleid: _moduleId
	}, _sqlList, _isRecord);
	var _isSuc = localdb.executeSync1({sqls: _sqlList});
	return _isSuc;
}

// 删除共享记录
module.exports.delShareRecordData = function(_id, _moduleId) {
	return delShareRecordData(_id, _moduleId, true);
};

// 删除共享记录-数据同步
module.exports.delShareRecordSyncData = function(_id, _moduleId) {
	return delShareRecordData(_id, _moduleId, false);
};

/**
 * 获取删除共享记录的所有sql语句
 * @param _changedData
 * @param _sqlList
 * @param _isRecord
 * @returns
 */
function getDelShareRecordAllSqls(_changedData, _sqlList, _isRecord){
	var _shareRecordList = queryShareRecordAllDataByTaskId(_changedData.id);
	if (null != _shareRecordList && 
			_shareRecordList.length > 0){
		for (var i = 0; i < _shareRecordList.length; i++) {
			if (_shareRecordList[i] == null)
				continue;
			getDelShareRecordDataSql({
				id: _shareRecordList[i].id,
				moduleid: _changedData.moduleid
			}, _sqlList, _isRecord);
		}
	}
}

// 获取删除共享记录的所有sql语句
module.exports.getDelShareRecordAllSqls = function(_changedData, _sqlList, _isRecord){
	getDelShareRecordAllSqls(_changedData, _sqlList, _isRecord);	
};

/**
 * 删除指定数据（id）的所有共享记录数据
 * @param _id
 * @param _moduleId
 * @param _isRecord
 * @returns
 */
function delShareRecordAllDataByTaskId(_id, _moduleId, _isRecord) {
	var _sqlList = [];
	getDelShareRecordAllSqls({
		id: _id,
		moduleid: _moduleId
	}, _sqlList, _isRecord);
	var _isSuc = localdb.executeSync1({sqls: _sqlList});
	return _isSuc;
}

//删除指定数据（id）的所有共享记录数据
module.exports.delShareRecordAllData = function(_id, _moduleId) {
	return delShareRecordAllDataByTaskId(_id, _moduleId, true);
};

/**
 * 通过id从共享记录表（sharerecord）中获取指定的一条数据
 * 
 * @param _id
 * @returns
 */
function queryShareRecordOneData(_id) {
	var sql = "select * from sharerecord where id=" + tools.formatSqlValue(_id) + ";";
	var _data = localdb.querySync(sql);
	if (_data == null || _data.length <= 0) {
		return null;
	}
	return _data[0];
}

// 查询使用记录数据
module.exports.getShareRecordOneData = function(_id) {
	return queryShareRecordOneData(_id);
};

/**
 * 通过taskid从sharerecord表中获取所有共享记录数据
 * 
 * @param _id
 * @returns
 */
function queryShareRecordAllDataByTaskId(_taskId) {
	var sql = "select * from sharerecord where taskid=" 
		+ tools.formatSqlValue(_taskId) + " order by labeltime DESC;";
	var _data = localdb.querySync(sql);
	if (_data == null || _data.length <= 0) {
		return null;
	}
	return _data;
}

//通过taskid从sharerecord表中获取所有共享记录数据
module.exports.getShareRecordAllDataByTaskId = function(_taskId) {
	return queryShareRecordAllDataByTaskId(_taskId);
};
