/**
 * http通信
 * 
 * @Author : Cydow
 * @Timestamp : 2017-08-21
 */
var deviceone = require("deviceone");
var do_Storage = deviceone.sm("do_Storage");
var do_Global = deviceone.sm("do_Global");
var do_Algorithm = deviceone.sm("do_Algorithm");
var do_InitData = deviceone.sm("do_InitData");

//js文件路径管理
var jsPathManager = require("jsPathManager.ui");
//全局常量
var globalConst = require(jsPathManager.getGlobalConstPath());
//url常量
var urlConst = require(jsPathManager.getUrlConstPath());
//json工具
var jsonTools = require(jsPathManager.getJsonToolPath());
//sha256算法
var sha256 = require(jsPathManager.getSha256Path());

//当前是否开发调试（测试）模式
//true：调试模式，包括开发调试和开发版测试
//false：生产模式，包括生产和生产测试
var isDebug = false;

// 错误码
module.exports.errorCode = {
		nouser: "800101",
		erruser: "800102",
		existuser: "800201",
		errsign: "800991",
		errvcode: "900801",
		errticket: "900999",
		
};

/**
 * 使用本地测试数据
 * @param _options
 * @returns
 */
function useMockData(_options){
	var _status = _options.status;
	if(0 == _status){
		_options.complete.call(this, _options.mockdata);
	} else {
		_options.error.call(this, _options.mockdata);
	}
}

/**
 * 
 * @param _paramData
 * @returns
 */
function getParams(_paramData){
	var _param = "";
	var _encodeData = "";
	if(null != _paramData && _paramData.length > 0){
		for(var i=0; i<_paramData.length; i++){
			var _oneData = _paramData[i];
			if(null == _oneData) continue;
			var _paramTmp = _oneData.key + "=" + _oneData.value;
			_param += _paramTmp;
			_paramTmp = encodeURIComponent(_oneData.key) +
				"=" + encodeURIComponent(_oneData.value);
			_encodeData += _paramTmp;
			if(i < _paramData.length - 1){
				_param += "&";
				_encodeData += "&";
			}
		}
	}
	return {
		param: _param,
		encodeData: _encodeData
	};
}

/**
 * 获取url
 * @param _options
 * @returns
 */
function getUrl(_options){
	//默认为生产服务url
	var _url = urlConst.constants["release"];
	//是否debug模式
	//若是，则url是测试服务地址；
	//否则， 需要检查默认生产服务和当前生产服务url是否一致，
	//不一致则取当前生成服务url
	if(isDebug){
		_url = urlConst.constants["debug"];
	} else {
		var _cloudUrl = do_Global.getMemory("cloudHost");
		if(null != _cloudUrl && "" != _cloudUrl && _url != _cloudUrl){
			_url = _cloudUrl;
		}
	}
	_url += urlConst.constants[_options.url];
	return _url;
}

/**
 * http请求
 * @param _options
 * @returns
 */
function httpRequest(_options){
	if(_options.usemockdata){
		useMockData(_options);
		return ;
	}
	var do_http = deviceone.mm("do_Http");
	var _type = _options.type;
	var _timeout = _options.timeout;
	//获取url
	var _url = getUrl(_options);
	var _dataBody = "";
	if("GET" == _type){
		var _paramData = getParams(_options.dataBody);
		_dataBody = _paramData.encodeData;
		do_http.url = _url + "?" + _paramData.param;
	} else if("POST" == _type){
		do_http.url = _url;
		do_http.body = JSON.stringify(_options.dataBody);
	} else {
		_type = "POST";
		var _paramData = getParams(_options.dataBody);
		_dataBody = _paramData.encodeData;
		do_http.url = _url + "?" + _paramData.param;
	}
	var _nowTime = new Date();
	var _sign = sha256.hex_sha256(_dataBody +
			urlConst.constants["apikey"] +
			urlConst.constants["secret"] + _nowTime.getTime());
	do_http.method = _type;  // GET | POST
	do_http.timeout = _timeout; // 超时时间 : 单位 毫秒
	do_http.contentType = "application/json;charset=UTF-8"; // Content-Type
	do_http.setRequestHeader("apikey", urlConst.constants["apikey"]);
	do_http.setRequestHeader("sign", _sign);
	do_http.setRequestHeader("timestamp", _nowTime.getTime() + "");
	do_http.setRequestHeader("apiversion", 2 + "");
	if(_options.hastooken){
		var _accessTicket = do_Global.getMemory("accessTicket");
		if("" == _accessTicket){
			// token已失效，不需要再访问云端接口
			_options.error.call(this, "");
			return;
		}
		do_http.setRequestHeader("access-ticket", _accessTicket);
	}
	do_http.on("success", function(_data) {
		
    });
	do_http.on("fail", function(_data) {
		
    });
	do_http.on("result", function(_data) {
		if(200 == _data.status){
			_options.complete.call(this, _data.data);
		} else if(400 == _data.status){
			var _msg = jsonTools.strToJsonObj(_data.data);
			if(null != _msg && "900999" == _msg.error){
				do_Global.fire("AgainLogin");
				_options.error.call(this, "");
			} else {
				_options.error.call(this, _data.data);
			}
		} else if(500 == _data.status){
			_options.error.call(this, "");
		} else {
			_options.error.call(this, "");
		}
    });
	do_http.request();	
}


//---------------------------------------------------------------
/**
 * ajax访问http服务，默认配置在httpSetting.js中配置
 * @param url 访问的url路径，可以是相对于rootUrl的路径
 * @param options 传入的选项内容
		//上级选项名称（在defaultSetting/httpSetting.js中配置,可继承选项内容）
		parent:null,
		//服务请求的url（如果不是以http://或https://开始，则会自动加上httpRootUrl变量值的前缀）
		url:null,
		//请求的类型（GET,POST, PUT, DELETE）。
		type:"GET",
		//发送数据到服务器时所使用的内容类型。可以是："application/x-www-form-urlencoded", "application/json"
		contentType:"application/x-www-form-urlencoded; charset=UTF-8",
		//要发送到服务器的数据，发送前可以根据需要自动转换为请求字符串格式（例如：{a:"1", b:"2"}，对于GET请求可自动转换为&a=1&b=2）
		data:null,
		//返回内容的解码方式（utf-8或GBK）
		responseEncoding:"utf-8",
		//设置本地的请求超时时间（以毫秒计）
		timeout:30000,
		//发送请求前运行的函数:beforeSend(options, do_Http)
		beforeSend:null,
		//返回数据后的进行预处理的函数:dataFilter(data)
		dataFilter:null,
		//当请求成功时运行的函数success(data, status)
		success:null,
		//如果请求失败要运行的函数error(data, status)
		error:null,
		//请求完成时运行的函数（在请求成功或失败之后均调用，即在 success 和 error 函数之后）complete(data, status)
		complete:null,
		//是否缓存上次结果 （为true的时候，在返回服务结果之前会先返回上次结果，一般用于改善数据查询的交互体验）
		cacheLastResult:false,
		//缓存失效时长（单位为毫秒，-1表示永远使用缓存，0表示每次都先用缓存后刷新变化）
		cacheExpires:0,
		//是否使用调试数据
		useMockData:false,
		//调试数据
		mockData:null
 */
module.exports.ajax = function(options){
	httpRequest(options);
};

/**
 * 访问http服务，配置参数
 * @param url 访问的url路径，可以是相对于rootUrl的路径
 * @param options 传入的选项内容
		//服务请求的url（服务接口）
		url: null,
		//请求的类型（GET,POST）。
		type: "GET",
		//要发送到服务器的数据，发送前可以根据需要自动转换为请求字符串格式（例如：{a:"1", b:"2"}，对于GET请求可自动转换为&a=1&b=2）
		dataBody: null,
		//下载文件后本地存储路径，主要httptype值为download会用到
		path: "",
		//设置本地的请求超时时间（以毫秒计）
		timeout: 30000,
		//是否鉴权
		hastooken: false,
		//http类型（request,form,download）
		httptype: 'request',
		//如果请求失败要运行的函数error(data, status)
		error: null,
		//请求完成时运行的函数（在请求成功或失败之后均调用，即在 success 和 error 函数之后）complete(data, status)
		complete: null,
		//是否使用调试数据
		useMockData: false,
		//调试数据
		mockData: null
 * @returns
 **/
function syncRequest(_options){
	// 是否使用调试数据
	if(_options.usemockdata){
		useMockData(_options);
		return ;
	}
	var do_http = mm("do_Http");
	var _type = _options.type; // 类型
	var _httpType = _options.httptype; // 请求类型
	var _timeout = _options.timeout; // 超时时间
	//获取url
	var _url = getUrl(_options);
	// 设置 url 和 body 信息
	var _dataBody = "";
	if("GET" == _type){
		var _paramData = getParams(_options.dataBody);
		_dataBody = _paramData.encodeData;
		do_http.url = _url + "?" + _paramData.param;
	} else if("POST" == _type){
		do_http.url = _url;
		if("form" != _httpType){
			do_http.body = JSON.stringify(_options.dataBody);
		}
	} else {
		_type = "POST";
		var _paramData = getParams(_options.dataBody);
		_dataBody = _paramData.encodeData;
		do_http.url = _url + "?" + _paramData.param;
	}
	// 时间戳
	var _timeStamp = (new Date()).getTime();
	// 签名
	var _sign = sha256.hex_sha256(_dataBody +
			urlConst.constants["apikey"] +
			urlConst.constants["secret"] + _timeStamp);
	
	// 根据请求类型设置，设置Content-Type
	if("form" == _httpType){
		do_http.contentType = "multipart/form-data";
	} else {
		do_http.contentType = "application/json;charset=UTF-8";
	}
	// 设置method
	do_http.method = _type;  // GET | POST
	do_http.timeout = _timeout; // 超时时间 : 单位 毫秒
	// 设置header
	do_http.setRequestHeader("apikey", urlConst.constants["apikey"]);
	do_http.setRequestHeader("sign", _sign);
	do_http.setRequestHeader("timestamp", _timeStamp + "");
	do_http.setRequestHeader("apiversion", 2 + "");
	// 是否鉴权，设置鉴权信息
	if(_options.hastooken){
		var _accessTicket = do_Global.getMemory("accessTicket");
		if("" == _accessTicket){
			// token已失效，不需要再访问云端接口
			_options.error.call(this, "");
			return;
		}
		do_http.setRequestHeader("access-ticket", _accessTicket);
	}
	do_http.on("success", function(_data) {
		
    });
	do_http.on("fail", function(_data) {
		
    });
	do_http.on("result", function(_data) {
		if(undefined != _data && null != _data){
			if(200 == _data.status){
				_options.complete.call(this, _data.data);
			} else if(400 == _data.status){
				var _msg = jsonTools.strToJsonObj(_data.data);
				if(null != _msg && "900999" == _msg.error){
					do_Global.fire("AgainLogin");
				} else {
					_options.error.call(this, _data.data);
				}
			} else if(500 == _data.status){
				_options.error.call(this, "");
			} else {
				_options.error.call(this, "");
			}
		}
    });
	// 根据请求类型，调用不同的方法
	if("form" == _httpType){
		do_http.form(_options.dataBody);
	} else if("download" == _httpType){
		do_http.download(_options.path);
	} else {
		do_http.request();
	}
}

// 同步数据请求
module.exports.syncRequest = function(options){
	syncRequest(options);
};

/**
 * 获取通用（一般）url
 * @param _options
 * @returns
 */
function getCommonUrl(_options){
	//默认为生产服务url
	var _url = urlConst.constants["common"];
	//是否debug模式
	//若是，则url是测试服务url
	if(isDebug){
		_url = urlConst.constants["commondev"];
	}
	_url += _options.url;
	return _url;
}

/**
 * 一般请求
 * @param _options
 * @returns
 */
function commonRequest(_options){
	var do_http = mm("do_Http");
	//获取通用（一般）url
	var _url = getCommonUrl(_options);
	var _type = _options.type;
	var _contentType = _options.contentType;
	var _timeout = _options.timeout;
		
	// 设置 url 和 body 信息
	if("GET" == _type){
		do_http.url = _url + "?" + _options.dataBody;
	} else if("POST" == _type){
		do_http.url = _url;
		do_http.body = JSON.stringify(_options.dataBody);
	}
		
	do_http.method = _type;  // GET | POST
	do_http.timeout = _timeout; // 超时时间 : 单位 毫秒
	do_http.contentType = _contentType; // Content-Type
	do_http.url = _url;
	do_http.on("result", function(_data) {
		if(undefined != _data && null != _data){
			if(200 == _data.status){
				_options.complete.call(this, _data.data);
			} else {
				_options.error.call(this, _data.data);
			}
		}
    });
	do_http.request();
}

// 一般请求
module.exports.commonRequest = function(_options){
	commonRequest(_options);
};

/**
 * 一般下载文件
 * @param _options
 * @returns
 */
function commonDownload(_options){
	var do_http = mm("do_Http");
	//获取通用（一般）url
	var _url = getCommonUrl(_options);
	do_http.method = "GET";  // GET | POST
	do_http.timeout = 60000; // 超时时间 : 单位 毫秒
	do_http.contentType = "application/octet-stream"; // Content-Type
	do_http.url = _url;
	do_http.on("result", function(_data) {
		if(undefined != _data && null != _data){
			if(200 == _data.status){
				_options.complete.call(this, _data.data);
			} else {
				_options.error.call(this, _data.data);
			}
		}
    });
	do_http.download(_options.filePath);
}

//一般下载文件
module.exports.commonDownload = function(options){
	commonDownload(options);
};

/**
 * 获取url（8080端口）
 * @param _options
 * @returns
 */
function getUrl8080(_options){
	//默认为生产服务url（8080端口）
	var _url = urlConst.constants["release8080"];
	//是否debug模式
	//若是，则url是测试服务url（8080端口）
	if(isDebug){
		_url = urlConst.constants["dev8080"];
	}
	_url += _options.url;
	return _url;
}

/**
 * 发送意见反馈
 * @param _options
 * @returns
 */
function sendFeedBack(_options){
	var do_http = mm("do_Http");
	//获取通用url（8080端口）
	var _url = getUrl8080(_options);
	do_http.method = "POST";  // GET | POST
	do_http.timeout = 15 * 1000; // 超时时间 : 单位 毫秒
	do_http.contentType = "application/json"; // Content-Type
	do_http.url = _url;
	do_http.body = JSON.stringify(_options.dataBody);
	do_http.on("result", function(_data) {
		print("sendFeedBack result: " + JSON.stringify(_data));
		if(undefined != _data && null != _data){
			if(200 == _data.status){
				_options.complete.call(this, _data.data);
			} else {
				_options.error.call(this, _data.data);
			}
		}
    });
	do_http.request();
}

//发送意见反馈
module.exports.sendFeedBack = function(_options){
	sendFeedBack(_options);
};

/**
 * 上传日志
 * @param _options
 * @returns
 */
function uploadLog(_options){
	var do_http = mm("do_Http");
	//获取通用url（8080端口）
	var _url = getUrl8080(_options);
	print("uploadLog url " + _url);
	do_http.method = "POST";  // GET | POST
	do_http.timeout = 60000; // 超时时间 : 单位 毫秒
	do_http.contentType = "multipart/form-data"; // Content-Type
	do_http.url = _url;
	do_http.on("result", function(_data) {
		print("uploadLog result " + JSON.stringify(_data));
		if(undefined != _data && null != _data){
			if(200 == _data.status){
				_options.complete.call(this, _data.data);
			} else {
				_options.error.call(this, _data.data);
			}
		}
    });
	do_http.upload(_options.filePath);
}

//上传日志
module.exports.uploadLog = function(options){
	uploadLog(options);
};
