/**
 * 时间工具
 * 备注：
 * 1、使用系统日历提醒
 * 待办、纪念日、生日提醒信息会写入系统日历进行提醒
 * 2、时间流内部提醒
 * （1）提醒的信息
 * 待办、纪念日、生日提醒
 * 固定时间提醒
 * （2）提醒标识的类型及取值（也就是notifyid的取值）
 * 一般提醒（待办、纪念日、生日提醒），从1开始并小于1000000000
 * 固定时间提醒通知（每周待办统计、定期同步备份） 1000000000~1000000009
 * 同步相关提醒通知 1100000000~1100000006
 * 待办倒计时提醒通知 1200000001
 * 其他 1001000000
 * 3、提醒（或通知）音频文件
 * （1）待办、纪念日、生日提醒，文件路径：data://audio/notifications/prompt.m4a
 * （2）固定时间提醒通知，文件路径：data://audio/notifications/notification.m4a
 * （3）倒计时提醒通知，文件路径：data://audio/notifications/notification.m4a
 * 
 * @Author : Cydow
 * @Timestamp : 2017-07-11
 */
//引入组件库
var deviceone = require("deviceone");
var do_storage = deviceone.sm("do_Storage");
var do_initData = deviceone.sm("do_InitData");
var do_dataCache = sm("do_DataCache");
var do_global = sm("do_Global");
var do_sysCalendar = sm("do_SysCalendar");

var objectId = require("objectId"); // 唯一标识
var calendar = require("calendar");      //日期
var constants = require("constants");     //常量
var dbsyscalendar = require("dbsyscalendar"); // 系统日历记录
var dblocalmessage = require("dblocalmessage"); // 本地消息
var dbmessagecenter = require("dbmessagecenter.ui"); // 消息中心
var urlConstants = require("urlconstants.ui"); // url常量

//js文件路径管理
var jsPathManager = require("jsPathManager.ui");
//工具
var tools = require(jsPathManager.getToolPath());
//数据同步-全局常量
var syncGlobalConst = require(jsPathManager.getSyncConstPath());
//信息库同步
var dbexpsyncdata = require(jsPathManager.getExpSyncDBPath());
//权限请求工具
var permReqTools = require(jsPathManager.getPermReqToolsPath());

var commonUrl = urlConstants.constants["common"] + "/app/start/index.html";
var appName = constants.getConstants("appName");

var description = "查看更多详情，请打开“" + appName + "”APP，" + commonUrl + " 。";

//设备信息
var deviceInfo = do_global.getMemory("deviceInfo");
// 默认为ios系统
var isIOS = deviceInfo.isIOS;

//保存提醒音频文件
tools.savePromptAudioFile(function(_result){
	print("savePromptAudioFile " + _result);
});

var promptPath = "";
var notificationPath = "";
//有电话和存储权限才获取提醒文件路径
if(permReqTools.checkPhonePerm() &&
		permReqTools.checkStorePerm()){
	//获取提醒音频文件路径
	promptPath = tools.getPromptAudioFilePath(0);
	//获取通知音频文件路径
	notificationPath = tools.getPromptAudioFilePath(1);
}

/**
 * 检测访问系统日历权限
 * @returns
 */
function checkAndroidPerm(){
	//引入获取安卓权限组件
	var _androidPermission = mm("do_AndroidPermission");
	// 引入权限常量
	var _permConstants = require("source://view/mainView/androidPerm/permConstants.ui");
	var _permName = _permConstants.permNames["writeSycCalendar"];
	var _result = _androidPermission.check(_permName);
	if(undefined == _result || 0 == _result){
		return true;
	}
	return false;
}

/**
 * 检测是否可以日历提醒
 * @returns
 */
function checkIsSycCalNotify(){
	if(!isIOS && checkAndroidPerm()){
		return true;
	}
	return false;
}

//检测是否可以日历提醒
module.exports.checkIsSycCalNotify = function(){
	return checkIsSycCalNotify();
};

/**
 * 更新系统日历提醒
 * @param _sysCalendarRecord
 * @param _params
 * @returns
 */
function updSysCalendarNotify(_sysCalendarRecord, _params){
	print("updSysCalendarNotify");
	// 删除已存在的系统日历提醒,并添加新的系统日历提醒
	do_sysCalendar.delete({id: _sysCalendarRecord.id}, function(_isDel){
		print("calendar del " + _isDel);
		// 添加系统日历提醒
		do_sysCalendar.add(_params, function(_calendarId){
			print("calendar add " + JSON.stringify(_calendarId));
			if(null != _calendarId){
				// 更新系统日历提醒记录
				dbsyscalendar.updSysCalRecordByTaskIdAndSource({
					id: _calendarId,
					taskid: _sysCalendarRecord.taskid,
					source: _sysCalendarRecord.source
				})
			}
		});
	});
}

/**
 * 删除系统日历提醒
 * @param _sysCalendarRecord
 * @param _callFunc
 * @returns
 */
function delSysCalendarNotify(_sysCalendarRecord, _callFunc){
	print("delSysCalendarNotify");
	// 若不允许访问系统日历，则直接放回；
	// 否则，删除系统日历提醒
	if(!checkAndroidPerm()){
		if(!tools.isNull(_callFunc)){
			_callFunc.call(this);
		}
		return;
	}
	// 删除系统日历提醒
	do_sysCalendar.delete({id: _sysCalendarRecord.id}, function(_isDel){
		print("calendar del " + _isDel);
		dbsyscalendar.delSysCalRecordByIdAndSource(_sysCalendarRecord.id,
				_sysCalendarRecord.source);
		if(!tools.isNull(_callFunc)){
			_callFunc.call(this);
		}
	});
}

/**
 * 添加系统日历提醒
 * @param _taskId
 * @param _params
 * @param _callFunc
 * @returns
 */
function addSysCalendarNotify(_taskId, _params, _callFunc){
	print("addSysCalendarNotify " + JSON.stringify(_params));
	// 添加系统日历提醒
	do_sysCalendar.add(_params, function(_calendarId){
		print("calendar add " + JSON.stringify(_calendarId));
		if(null != _calendarId){
			// 添加系统日历提醒记录
			dbsyscalendar.insertSysCalendarData({
				id: _calendarId,
				taskid: _taskId,
				source: constants.getConstants("calendarSource_tf")
			});
		}
		// 回调函数
		if(!tools.isNull(_callFunc)){
			_callFunc.call(this);
		}
	});
}

/**
 * 设置系统日历提醒-数据同步
 * @param _notifyData
 * @param _callFunc
 * @returns
 */
function setSycCalNotifySync(_notifyData, _callFunc){
	// 若不允许访问系统日历，则直接放回；
	// 否则，设置系统日历提醒
	if(!checkAndroidPerm()){
		_callFunc.call(this);
		return;
	}
	print("setSycCalNotifySync _notifyData " + JSON.stringify(_notifyData));
	var _title = "[" + appName +"]" + _notifyData.content;
	var _remindTime = _notifyData.remindtime;
	var _params = {
			title: _title,
			description: description,
			startTime: _notifyData.start,
			endTime: _notifyData.end,
			reminderTime: _remindTime
	};
	// 添加系统日历提醒
	addSysCalendarNotify(_notifyData.id, _params, _callFunc);
}

// 设置系统日历提醒-数据同步
module.exports.setSycCalNotifySync = function(_notifyData, _callFunc){
	setSycCalNotifySync(_notifyData, _callFunc);
};

/**
 * 设置系统日历提醒
 * @param _notifyData
 * @param _isUpd
 * @param _operType
 * @param _callFunc
 * @returns
 */
function setSysCalendarNotify(_notifyData, _isUpd, _operType, _callFunc){
	// 若不允许访问系统日历，则直接放回；
	// 否则，设置系统日历提醒
	if(!checkAndroidPerm()) return;
	// 查询系统日历提醒的记录
	var _sysCalendarRecord = dbsyscalendar.querySysCalByTaskIdAndSource(_notifyData.id,
			constants.getConstants("calendarSource_tf"));
	print("_sysCalendarRecord " + JSON.stringify(_sysCalendarRecord));
	print("setSysCalendarNotify _notifyData " + JSON.stringify(_notifyData));
	if(_notifyData.remindtime != -1){
		//提醒时间
		var _promptTime = calendar.getPromptTime(_notifyData.start, _notifyData.remindtime);
		//判断提醒时间是否已经过去（暂时过去的时间不设置提醒）
		var _nowTime = new Date();
		if(_nowTime.getTime() < _promptTime.getTime()){
			var _title = "[" + appName +"]" + _notifyData.content;
			var _remindTime = _notifyData.remindtime;
			var _params = {
					title: _title,
					description: description,
					startTime: _notifyData.start,
					endTime: _notifyData.end,
					reminderTime: _remindTime
			};
			if(null == _sysCalendarRecord){
				// 查询读取系统日历的记录
				_sysCalendarRecord = dbsyscalendar.querySysCalByTaskIdAndSource(_notifyData.id,
						constants.getConstants("calendarSource_sys"));
				// 若读取系统日历的记录不存在或者存在且操作类型为修改，
				// 则，添加系统日历提醒；否则不做处理
				if(null == _sysCalendarRecord ||
						(null != _sysCalendarRecord && 
								"update" == _operType)){
					// 添加系统日历提醒
					addSysCalendarNotify(_notifyData.id, _params, _callFunc);
				}
			} else {
				// 若需要更新系统日历提醒，则才会更新系统日历提醒；
				// 否则不做处理
				if(_isUpd){
					// 更新系统日历提醒
					updSysCalendarNotify(_sysCalendarRecord, _params);
				}
			}
		} else {
			// 若需要删除系统日历提醒，则才会删除系统日历提醒；
			// 否则不做处理
			if(_isUpd){
				// 若系统日历提醒记录不为空，则删除系统日历提醒
				if(null != _sysCalendarRecord){
					// 删除系统日历提醒
					delSysCalendarNotify(_sysCalendarRecord);
				}
			}
		}
	} else {
		// 若系统日历提醒记录不为空，则删除系统日历提醒
		if(null != _sysCalendarRecord){
			// 删除系统日历提醒
			delSysCalendarNotify(_sysCalendarRecord);
		}
	}
}

/**
 * 删除时间流内部本地提醒
 * @param _notifyIds
 * @returns
 */
function delTimeFlowNotify(_notifyIds){
	if(!isIOS){
		var M0026_myNotify = deviceone.sm("M0026_myNotify");
		M0026_myNotify.removeNotify({notifyIds: _notifyIds});
	} else {
		var do_LocalNotification = deviceone.sm("do_LocalNotification");
		do_LocalNotification.removeNotify({notifyIds: _notifyIds});
	}
}

//删除时间流提醒
module.exports.delTimeFlowNotify = function(_notifyIds){
	delTimeFlowNotify(_notifyIds);
};

/**
 * 添加时间流内部本地提醒
 * @param _notifyData
 * @param _promptTime
 * @param _ringing
 * @returns
 */
function addTimeFlowNotify(_notifyData, _promptTime, _ringing){
	print("addTimeFlowNotify _notifyData " + JSON.stringify(_notifyData));
	//格式化提醒时间
	var _promptTimeText = calendar.getStandardTimeText(_promptTime);
	//提醒内容及相关参数值
	var _params = {
			notifyTime: _promptTimeText,
			notifyId: _notifyData.notifyId,
			contentText: _notifyData.content,
			contentTitle: appName,
			extra:{isShow: _notifyData.isShow, 
				dataid: _notifyData.id,
				dataType: _notifyData.dataType},
			repeatMode: "None",
			ringing: _ringing
	};
	print("addTimeFlowNotify _params " + JSON.stringify(_params));
	//不同操作系统（ios or android）设置不同提醒方式
	//ios通知栏提醒、android通知栏提醒加全屏提醒
	if(!isIOS){
		var M0026_myNotify = deviceone.sm("M0026_myNotify");
		M0026_myNotify.addNotify(_params);
	} else {
		var do_LocalNotification = deviceone.sm("do_LocalNotification");
		do_LocalNotification.addNotify(_params);
	}
}

/**
 * 设置时间流内部本地提醒
 * @param _notifyData
 * @returns
 */
function setTimeFlowNotify(_notifyData){
	if(_notifyData.remindtime != -1){
		//提醒时间
		var _promptTime = calendar.getPromptTime(_notifyData.start, 
				_notifyData.remindtime);
		//判断提醒时间是否已经过去（暂时过去的时间不设置提醒）
		var _nowTime = new Date();
		if(_nowTime.getTime() < _promptTime.getTime()){
			// 添加时间流内部本地提醒
			addTimeFlowNotify(_notifyData, _promptTime, promptPath);
			return ;
		}
	}
	// 删除时间流内部本地提醒
	delTimeFlowNotify([_notifyData.notifyId]);
}

/**
 * 设置本地提醒
 * @param _notifyData
 * @param _isUpd
 * @param _operType
 * @param _callFunc
 */
function setLocalNotification(_notifyData, _isUpd, _operType, _callFunc){
	if (_notifyData.remindtime != null){
		var _nowTime = new Date();
		_nowTime.setTime(_notifyData.start);
		_nowTime.setSeconds(0, 0);
		_notifyData.start = _nowTime.getTime();
		var _isSysCalendarNotify = do_global.getMemory("allowSysCalendarNotify");
		print("_isSysCalendarNotify " + _isSysCalendarNotify);
		// 根据需要设置为时间流内部本地提醒或系统日历提醒（解决不提醒问题）
		if(0 == _isSysCalendarNotify){
			// 设置时间流内部本地提醒
			setTimeFlowNotify(_notifyData);
		} else {
			// 设置系统日历提醒
			setSysCalendarNotify(_notifyData, _isUpd, _operType, _callFunc);
		}
	} 
}

//设置提醒
module.exports.setLocalNotification = function(_changedData, _isUpd, _operType){
	setLocalNotification(_changedData, _isUpd, _operType);
};

/**
 * 删除系统日历提醒-异步
 * @param _notifyList
 * @param _index
 * @param _callFunc
 * @returns
 */
function delSysCalendarNotifyAsync(_notifyList, _index, _callFunc){
	print("delSysCalendarNotifyAsync " + _index);
	if(_index < _notifyList.length){
		var _dataId = _notifyList[_index];
		if(null != _dataId && _dataId.length > 0){
			//查询系统日历提醒的记录
			dbsyscalendar.querySysCalByTaskIdAsync(_dataId,
					constants.getConstants("calendarSource_tf"), 
					function(_sysCalendarRecord){
				if(null != _sysCalendarRecord){
					// 删除系统日历提醒
					do_sysCalendar.delete({id: _sysCalendarRecord.id}, 
							function(_isDel){
						print("calendar del " + _isDel);
						dbsyscalendar.delSysCalRecordByIdAndSource(_sysCalendarRecord.id,
								_sysCalendarRecord.source);
						
						_index++;
						delSysCalendarNotifyAsync(_notifyList, 
								_index, _callFunc);
					});
					return;
				}
				_index++;
				delSysCalendarNotifyAsync(_notifyList, 
						_index, _callFunc);
			});
		} else {
			_index++;
			delSysCalendarNotifyAsync(_notifyList, 
					_index, _callFunc);
		}
	} else {
		_callFunc.call(this);
	}
}

/**
 * 删除本地提醒-异步
 * @param _notifyData
 * @param _callFunc
 */
function delLocalNotificationAsync(_notifyData, _callFunc){
	if(null != _notifyData){
		var _isSysCalendarNotify = do_global.getMemory("allowSysCalendarNotify");
		print("_isSysCalendarNotify " + _isSysCalendarNotify);
		//若设置的是系统日历提醒，且允许访问系统日历权限，
		//则删除系统日历提醒；否则，删除时间流自身本地提醒
		if(1 == _isSysCalendarNotify &&
				checkAndroidPerm()){
			if(null != _notifyData.dataIds &&
					_notifyData.dataIds.length > 0){
				//删除系统日历提醒
				delSysCalendarNotifyAsync(_notifyData.dataIds, 
						0, _callFunc);
				return;
			}
		} else {
			if(null != _notifyData.notifyIds &&
					_notifyData.notifyIds.length > 0){
				//删除时间流自身本地提醒
				delTimeFlowNotify(_notifyData.notifyIds);
			}
		}
	}
	_callFunc.call(this);
}

//删除本地提醒-异步
module.exports.delLocalNotificationAsync = function(_notifyList, _callFunc){
	delLocalNotificationAsync(_notifyList, _callFunc);
};

/**
 * 设置系统日历提醒-异步
 * @param _notifyList
 * @param _index
 * @param _endTime
 * @param _callFunc
 * @returns
 */
function setSysCalendarNotifyAsync(_notifyList, _index, _endTime, _callFunc){
	print("setSysCalendarNotifyAsync " + _index);
	if(_index < _notifyList.length){
		var _notifyData = _notifyList[_index];
		if(null != _notifyData){
			if(_notifyData.remindtime != -1){
				var _date = new Date();
				var _nowTime = _date.getTime();
				_date.setTime(_notifyData.start);
				_date.setSeconds(0, 0);
				_notifyData.start = _date.getTime();
				//提醒时间
				var _promptTime = calendar.getPromptTime(_notifyData.start, 
						_notifyData.remindtime);
				//判断提醒时间是否已经过去（暂时过去的时间不设置提醒）
				if(_nowTime < _promptTime.getTime()){
					//设置未来一段时间内提醒（15天）
					//若提醒时间在这个时间范围内，则设置提醒；否则，不设置提醒
					if(_promptTime.getTime() <= _endTime){
						//查询系统日历提醒的记录
						dbsyscalendar.querySysCalByTaskIdAsync(_notifyData.id,
								constants.getConstants("calendarSource_tf"), 
								function(_sysCalendarRecord){
							print("_sysCalendarRecord " + JSON.stringify(_sysCalendarRecord));
							print("setSysCalendarNotify _notifyData " + JSON.stringify(_notifyData));
							var _title = "[" + appName +"]" + _notifyData.content;
							var _remindTime = _notifyData.remindtime;
							var _params = {
									title: _title,
									description: description,
									startTime: _notifyData.start,
									endTime: _notifyData.end,
									reminderTime: _remindTime
							};
							if(null == _sysCalendarRecord){
								//查询读取系统日历的记录
								dbsyscalendar.querySysCalByTaskIdAsync(_notifyData.id,
										constants.getConstants("calendarSource_sys"), 
										function(_sysCalendarRecord){
									// 若读取系统日历的记录不存在或者存在且操作类型为修改，
									// 则，添加系统日历提醒；否则不做处理
									if(null == _sysCalendarRecord ||
											(null != _sysCalendarRecord && 
													_notifyData.isUpd)){
										// 添加系统日历提醒
										do_sysCalendar.add(_params, function(_calendarId){
											print("calendar add " + JSON.stringify(_calendarId));
											if(null != _calendarId && 
													"日程结束时间不能小于开始时间" != _calendarId){
												// 添加系统日历提醒记录
												dbsyscalendar.insertSysCalendarData({
													id: _calendarId,
													taskid: _notifyData.id,
													source: constants.getConstants("calendarSource_tf")
												});
											}
											_index++;
											setSysCalendarNotifyAsync(_notifyList, 
													_index, _endTime, _callFunc);
										});
									} else {
										_index++;
										setSysCalendarNotifyAsync(_notifyList, 
												_index, _endTime, _callFunc);
									}
								});
							} else {
								// 删除已存在的系统日历提醒,并添加新的系统日历提醒
								do_sysCalendar.delete({id: _sysCalendarRecord.id}, function(_isDel){
									print("calendar del " + _isDel);
									// 添加系统日历提醒
									do_sysCalendar.add(_params, function(_calendarId){
										print("calendar add " + JSON.stringify(_calendarId));
										if(null != _calendarId){
											// 更新系统日历提醒记录
											dbsyscalendar.updSysCalRecordByTaskIdAndSource({
												id: _calendarId,
												taskid: _sysCalendarRecord.taskid,
												source: _sysCalendarRecord.source
											});
										}
										_index++;
										setSysCalendarNotifyAsync(_notifyList, 
												_index, _endTime, _callFunc);
									});
								});
							}
						});
					} else {
						_index++;
						setSysCalendarNotifyAsync(_notifyList, 
								_index, _endTime, _callFunc);
					}
					return;
				}
			}
			//更新的时候改为不提醒或提醒时间已过去，才删除日历提醒
			if(_notifyData.isUpd){
				//删除系统日历提醒
				delSysCalendarNotifyAsync([_notifyData.id], 
						0, function(){
					_index++;
					setSysCalendarNotifyAsync(_notifyList, 
							_index, _endTime, _callFunc);
				});
				return;
			}
		}
		_index++;
		setSysCalendarNotifyAsync(_notifyList, 
				_index, _endTime, _callFunc);
	} else {
		_callFunc.call(this);
	}
}

/**
 * 设置时间流内部本地提醒-异步
 * @param _notifyData
 * @param _endTime
 * @returns
 */
function setTimeFlowNotifyAsync(_notifyData, _endTime){
	if(_notifyData.remindtime != -1){
		var _date = new Date();
		var _nowTime = _date.getTime();
		_date.setTime(_notifyData.start);
		_date.setSeconds(0, 0);
		_notifyData.start = _date.getTime();
		//提醒时间
		var _promptTime = calendar.getPromptTime(_notifyData.start, 
				_notifyData.remindtime);
		//判断提醒时间是否已经过去（暂时过去的时间不设置提醒）
		if(_nowTime < _promptTime.getTime()){
			//设置未来一段时间内提醒（15天）
			//若提醒时间在这个时间范围内，则设置提醒；否则，不设置提醒
			if(_promptTime.getTime() <= _endTime){
				//添加时间流内部本地提醒
				addTimeFlowNotify(_notifyData, _promptTime, promptPath);
			}
			return;
		}
	}
	//更新的时候改为不提醒或提醒时间已过去，才移除本地提醒
	if(_notifyData.isUpd){
		// 删除时间流内部本地提醒
		delTimeFlowNotify([_notifyData.notifyId]);
	}
}

/**
 * 设置本地提醒-异步
 * @param _notifyList
 * @param _callFunc
 */
function setLocalNotificationAsync(_notifyList, _callFunc){
	if(null != _notifyList && _notifyList.length > 0){
		var _isSysCalendarNotify = do_global.getMemory("allowSysCalendarNotify");
		print("_isSysCalendarNotify " + _isSysCalendarNotify);
		//设置未来15天内的提醒
		var _date = new Date();
		var _endTime = _date.getTime() + 15 * 24 * 3600 * 1000;
		//若设置的是系统日历提醒，且允许访问系统日历权限，
		//则写入系统日历提醒；否则，设置为时间流自身提醒
		if(1 == _isSysCalendarNotify &&
				checkAndroidPerm()){
			//设置系统日历提醒-异步
			setSysCalendarNotifyAsync(_notifyList, 0, _endTime, _callFunc);
			return;
		} else {
			for(var i=0; i<_notifyList.length; i++){
				var _notifyData = _notifyList[i];
				if(null == _notifyData ||
						null == _notifyData.remindtime) 
					continue;
				// 设置时间流内部本地提醒
				setTimeFlowNotifyAsync(_notifyData, _endTime);
			}
		}
	}
	_callFunc.call(this);
}

//设置本地提醒-异步
module.exports.setLocalNotificationAsync = function(_notifyList, _callFunc){
	setLocalNotificationAsync(_notifyList, _callFunc);
};

// 设置提醒-待办开始时间
module.exports.setTaskNotification = function(_changedData, _isUpd, _operType, _callFunc){
	// 提醒数据
	var _notifyData = {
			id: _changedData.id,
			content: _changedData.title,
			start: parseInt(_changedData.start),
			end: parseInt(_changedData.end),
			remindtime: _changedData.remindtime,
			notifyId: parseInt(_changedData.notifyid),
			isShow: true,
			dataType: constants.getConstants("notifyDataType_task")
	};
	// 设置本地提醒
	setLocalNotification(_notifyData, _isUpd, _operType, _callFunc);
};

//设置提醒-待办结束时间
module.exports.setTaskEndTimeNotif = function(_changedData){
	var _isSysCalendarNotify = do_global.getMemory("allowSysCalendarNotify");
	print("setTaskEndTimeNotif isSysCalendarNotify " + _isSysCalendarNotify);
	// 当使用时间流内部提醒，则才设置待办结束时间提醒
	if(0 == _isSysCalendarNotify){
		var _notifyId = parseInt(_changedData.endnotifyid);
		if(null != _changedData.endremindtime &&
				-1 != _changedData.endremindtime){
			var _endTime = new Date();
			_endTime.setTime(_changedData.end);
			_endTime.setSeconds(0, 0);
			//提醒时间
			var _promptTime = calendar.getPromptTime(_endTime.getTime(), 
					_changedData.endremindtime);
			//当前时间
			var _nowTime = (new Date()).getTime();
			//结束提醒时间大于当前时间，并且结束提醒时间大于等于开始时间才设置结束时间提醒
			if(_promptTime.getTime() > _nowTime &&
					_promptTime >= parseInt(_changedData.start)){
				// 提醒数据
				var _notifyData = {
						id: _changedData.id,
						content: _changedData.title,
						notifyId: _notifyId,
						isShow: true,
						dataType: constants.getConstants("notifyDataType_taskEndTime")
				};
				// 添加时间流内部本地提醒
				addTimeFlowNotify(_notifyData, _promptTime, promptPath);
				return;
			}
		}
		// 删除结束时间提醒
		delTimeFlowNotify([_notifyId]);
	}
};

//设置提醒-纪念日
module.exports.setCommNotification = function(_changedData, _isUpd, _operType){
	// 提醒数据
	var _notifyData = {
			id: _changedData.id,
			content: _changedData.commname,
			start: parseInt(_changedData.start),
			end: parseInt(_changedData.start) + 60 * 60 * 1000,
			remindtime: _changedData.remindtime,
			notifyId: parseInt(_changedData.notifyid),
			isShow: true,
			dataType: constants.getConstants("notifyDataType_comm")
	};
	// 设置本地提醒
	setLocalNotification(_notifyData, _isUpd, _operType);
};

//设置提醒-生日
module.exports.setBirthdayNotification = function(_changedData, _isUpd, _operType){
	// 生日提醒为当天早上8点整
	var _birthTime = new Date();
	_birthTime.setTime(parseInt(_changedData.start));
	_birthTime.setHours(8, 0, 0, 0);
	// 提醒数据
	var _notifyData = {
			id: _changedData.id,
			content: _changedData.commname + "的生日",
			start: _birthTime.getTime(),
			end: _birthTime.getTime() + 60 * 60 * 1000,
			remindtime: _changedData.remindtime,
			notifyId: parseInt(_changedData.notifyid),
			isShow: true,
			dataType: constants.getConstants("notifyDataType_birthday")
	};
	// 设置本地提醒
	setLocalNotification(_notifyData, _isUpd, _operType);
};

/**
 * 设置课程的本地提醒
 * @param _notifyData
 * @param _callFunc
 * @returns
 */
function setLessonNotification(_notifyData, _callFunc){
	if(null != _notifyData){
		//设置本地提醒
		setLocalNotificationAsync(_notifyData.notifyList, function(){
			//删除本地提醒
			delLocalNotificationAsync(_notifyData.delNotifyData, function(){
				_callFunc.call(this);
			});
		});
	} else {
		_callFunc.call(this);
	}
}

//设置提醒-课程
module.exports.setLessonNotification = function(_notifyData, _callFunc){
	print("setLessonNotification  " + JSON.stringify(_notifyData));
	//设置课程的本地提醒
	setLessonNotification(_notifyData, _callFunc)
};

/**
 * 移除本地提醒或系统日历提醒
 * @param _notifyId
 * @param _taskIds
 */
function removeLocalNotification(_notifyIds, _taskIds){
	var _isSysCalendarNotify = do_global.getMemory("allowSysCalendarNotify");
	// 根据需要设置为时间流内部本地提醒或系统日历提醒（解决不提醒问题）
	if(0 == _isSysCalendarNotify){
		// 删除时间流内部本地提醒
		delTimeFlowNotify(_notifyIds);
	} else {
		// 若不允许访问系统日历，则直接放回；
		// 否则，删除系统日历提醒
		if(!checkAndroidPerm()) return;
		if(null != _taskIds && 
				_taskIds.length > 0){
			for(var i=0; i<_taskIds.length; i++){
				if(null == _taskIds[i] ||
						"" == _taskIds[i]){
					continue;
				}
				// 查询系统日历提醒的记录
				var _sysCalendarRecord = dbsyscalendar.querySysCalByTaskIdAndSource(_taskIds[i],
						constants.getConstants("calendarSource_tf"));
				// 若系统日历提醒记录不为空，则删除系统日历提醒
				if(null != _sysCalendarRecord){
					// 删除系统日历提醒
					delSysCalendarNotify(_sysCalendarRecord);
				}
			}
		}
	}
}

//移除本地提醒或系统日历提醒
module.exports.removeLocalNotification = function(_notifyIds, _taskIds){
	removeLocalNotification(_notifyIds, _taskIds);
};

/**
 * 移除所有本地提醒
 */
function removeAllLocalNotification(){
	// 若为安卓系统，则移除所有全屏提醒通知;
	// 否则，移除应用所有本地提醒通知
	if(!isIOS){
		var M0026_myNotify = deviceone.sm("M0026_myNotify");
		M0026_myNotify.removeNotify({notifyIds: []});
	} else {
		var do_LocalNotification = deviceone.sm("do_LocalNotification");
		do_LocalNotification.removeNotify({notifyIds: []});
	}
}

// 移除所有本地提醒（使用场景为用户主动退出登录或设置为系统日历提醒）
module.exports.removeAllLocalNotification = function(){
	removeAllLocalNotification();
};

/**
 * 获取按周重复提醒时间
 * @param _nowTime
 * @param _weekDay
 * @returns
 */
function getWeekCyclePromptTime(_nowTime, _weekDay){
	var _week = _nowTime.getDay();
	var _lengthTime = 0;
	if(_week < _weekDay){
		_lengthTime = (_weekDay - _week) * 24 * 3600 * 1000;
	} else if(_week == _weekDay){
		if(_nowTime.getHours() > 20){
			_lengthTime = (_weekDay - _week + 7) * 24 * 3600 * 1000;
		} else if(_nowTime.getHours() == 20){
			if(_nowTime.getMinutes() > 0 ||
					_nowTime.getSeconds() > 0 ||
					_nowTime.getMilliseconds() > 0){
				_lengthTime = (_weekDay - _week + 7) * 24 * 3600 * 1000;
			}
		}
	} else {
		_lengthTime = (_weekDay - _week + 7) * 24 * 3600 * 1000;
	}
	var _promptTime = new Date();
	_promptTime.setTime(_nowTime.getTime() + _lengthTime);
	_promptTime.setHours(20, 0, 0, 0);
	return _promptTime;
}

/**
 * 设置定时任务的本地消息
 * @param _type
 * @param _promptTime
 * @param _notifyId
 * @param _title
 * @param _content
 * @returns
 */
function setFixedTimeLocalMsg(_type, _promptTime, _notifyId, 
		_title, _content){
	var _localMsg = dblocalmessage.queryLocalMsgByNotifyId(_notifyId);
	print("_localMsg " + JSON.stringify(_localMsg));
	// 若指定本地消息为空，则插入一条本地消息；
	// 否则，更新指定的本地消息
	if(null == _localMsg){
		var _id = objectId.getObjectId();
		// 插入一条本地消息
		dblocalmessage.insertLocalMessage({
			id: _id,
			type: _type,
			remindtime: _promptTime,
			againtime: _promptTime,
			notifyid: _notifyId
		});
	} else {
		var _nowTime = new Date();
		var _againTime = parseInt(_localMsg.againtime);
		if(_type == constants.getConstants("localMsgType_taskStatisYear")){
			_nowTime.setFullYear(2021, 0, 4);
			_nowTime.setHours(10, 0, 0, 0);
			if(_againTime < _nowTime.getTime()){
				_againTime = _nowTime.getTime();
				// 更新指定的一条本地消息
				dblocalmessage.updateLocalMessage({
					id: _localMsg.id,
					againtime: _againTime
				});
			} else {
				if(parseInt(_localMsg.remindtime) < _nowTime.getTime()){
					// 更新指定的一条本地消息
					dblocalmessage.updateLocalMessage({
						id: _localMsg.id,
						remindtime: _nowTime.getTime(),
						flag: constants.getConstants("localMsg_flag_read")
					});
					// 设置消息中心数据
					dbmessagecenter.setMsgCenterData({
						msgid: _localMsg.id,
						source: constants.getConstants("msgCenterSource_local"),
						msgtype: _type,
						title: _title,
						content: _content,
						msgcount: 1,
						remindtime: _nowTime.getTime(),
						status: constants.getConstants("msgCenterMsgStatus_read")
					});
				}
			}
		}
		//若再次提醒时间不等于提醒时间，则说明提醒时间有变化，需要更新再次提醒时间，
		//如果更新对应消息的提醒时间和标记；否则，进一步判断
		if(_againTime != _promptTime){
			if(_againTime <= _nowTime.getTime()){
				// 更新指定的一条本地消息
				dblocalmessage.updateLocalMessage({
					id: _localMsg.id,
					remindtime: _againTime,
					againtime: _promptTime,
					flag: constants.getConstants("localMsg_flag_unRead")
				});
				// 设置消息中心数据
				dbmessagecenter.setMsgCenterData({
					msgid: _localMsg.id,
					source: constants.getConstants("msgCenterSource_local"),
					msgtype: _type,
					title: _title,
					content: _content,
					msgcount: 1,
					remindtime: _againTime,
					status: constants.getConstants("msgCenterMsgStatus_unread")
				});
			} else {
				// 更新指定的一条本地消息
				dblocalmessage.updateLocalMessage({
					id: _localMsg.id,
					againtime: _promptTime,
					flag: constants.getConstants("localMsg_flag_read")
				});
				// 设置消息中心数据
				dbmessagecenter.setMsgCenterData({
					msgid: _localMsg.id,
					source: constants.getConstants("msgCenterSource_local"),
					msgtype: _type,
					title: _title,
					content: _content,
					msgcount: 1,
					remindtime: _promptTime,
					status: constants.getConstants("msgCenterMsgStatus_read")
				});
			}
		}
	}
}

/**
 * 设置定时任务的本地提醒（待办统计、数据同步即清除待办等）
 * @param _notifyId
 */
function setFixedTimeLocalNotification(_changedData){
	var _promptTime = new Date();
	var _notifyId = 1000000000; // 注：1001000000 已经被用了，不能在其他地方使用了
	var _title = appName;
	var _content = "想知道您本周做了多少事吗？快去看看吧！";
	var _dataType = _changedData.dataType;
	var _localMsgType = constants.getConstants("localMsgType_taskStatis");
	var _nowTime = new Date();
	var _nowYear = _nowTime.getFullYear();
	if(_dataType == constants.getConstants("notifyDataType_taskStatis")){
		//设置提醒信息（提醒ID：1000000000，提醒类型：待办统计，提醒时间：每周五 晚上8点）
		// 获取按周重复提醒时间（每周五）
		_promptTime = getWeekCyclePromptTime(_nowTime, 5);
	} else if(_dataType == constants.getConstants("notifyDataType_syncData")){
		//设置提醒信息（提醒ID：1000000001，提醒类型：数据同步，提醒时间：每月1日 晚上8点）
		_notifyId = 1000000001;
		_localMsgType = constants.getConstants("localMsgType_backUpToPC");
		_content = "请您及时把数据备份到慧影PC版，避免信息丢失！";
		_promptTime.setDate(1);
		_promptTime.setHours(20, 0, 0, 0);
		//若提醒时间（本月1号）已经过去，则设置下一个提醒时间（下个月1号）
		if(_nowTime.getTime() > _promptTime.getTime()){
			//若本月为12月，则设置为下一年的1月；否则设置为本年的下个月
			if(_nowTime.getMonth() == 11){
				_promptTime.setFullYear(_nowTime.getFullYear() + 1);
				_promptTime.setMonth(0);
			} else {
				_promptTime.setMonth(_nowTime.getMonth() + 1);
			}
		}
	} else if(_dataType == constants.getConstants("notifyDataType_clearCache1")){
		//设置提醒信息（提醒ID：1000000002，提醒类型：数据清理-1，提醒时间：每年6月23日 晚上8点）
		_notifyId = 1000000002;
		_localMsgType = constants.getConstants("localMsgType_clearCache");
		_content = (_nowTime.getFullYear() - 1) + "年的待办数据将于6月30日清理，请及时将数据同步到慧影PC版进行备份！";
		_promptTime.setMonth(5);
		_promptTime.setDate(23);
		_promptTime.setHours(20, 0, 0, 0);
	} else if(_dataType == constants.getConstants("notifyDataType_clearCacheDone")){
		//设置提醒信息（提醒ID：1000000003，提醒类型：数据清理完成，提醒时间：当前时间向后推2秒）
		_notifyId = 1000000003;
		_content = (_nowTime.getFullYear() - 1) + "年的待办数据已清理完成，请将未清理的数据及时备份到慧影PC版！";
		_promptTime.setTime(_nowTime.getTime() + 2 * 1000);
	} else if(_dataType == constants.getConstants("notifyDataType_sysCalendar")){
		//设置提醒信息（提醒ID：1000000004，提醒类型：访问系统日历，提醒时间：每周一 晚上8点）
		_notifyId = 1000000004;
		_localMsgType = constants.getConstants("localMsgType_sysCalendar");
		_content = "查不到系统日历的待办？快去开启日历权限吧！";
		// 获取按周重复提醒时间（每周一）
		_promptTime = getWeekCyclePromptTime(_nowTime, 1);
	} else if(_dataType == constants.getConstants("notifyDataType_taskStatisYear")){
		//设置提醒信息（提醒ID：1000000005，提醒类型：年度待办统计，提醒时间：2021年1月4日 上午10点）
		_notifyId = 1000000005;
		_localMsgType = constants.getConstants("localMsgType_taskStatisYear");
		_promptTime.setHours(10, 0, 0, 0);
		_nowYear = 2021;
		_content = "您" + (_nowYear - 1) + "年的时间帐单已经生成，快来看看吧！";
		// 若时间在2021年之前的，则设置提醒时间为2021年；
		// 否则，进行其他判断
		if(_nowYear < 2021){
		} else {
			_promptTime.setFullYear(_nowYear, 0, 4);
			// 若当前时间大于当前年的提醒时间，则设置为下一年的提醒时间；
			// 否则，默认提醒时间为当前年的提醒时间
			if(_promptTime.getTime() < _nowTime.getTime()){
				_nowYear += 1;
			}
		}
		_promptTime.setFullYear(_nowYear, 0, 4);
	} else if(_dataType == constants.getConstants("notifyDataType_clearCache2")){
		//设置提醒信息（提醒ID：1000000006，提醒类型：数据清理-2，提醒时间：每年6月29日 晚上8点）
		_notifyId = 1000000006;
		_localMsgType = constants.getConstants("localMsgType_clearCache");
		_content = (_nowTime.getFullYear() - 1) + "年的待办数据将于6月30日清理，请及时将数据同步到慧影PC版进行备份！";
		_promptTime.setMonth(5);
		_promptTime.setDate(29);
		_promptTime.setHours(20, 0, 0, 0);
	} else if(_dataType == constants.getConstants("notifyDataType_clearCache3")){
		//设置提醒信息（提醒ID：1000000007，提醒类型：数据清理-3，提醒时间：每年6月30日 上午10点）
		_notifyId = 1000000007;
		_localMsgType = constants.getConstants("localMsgType_clearCache");
		_content = (_nowTime.getFullYear() - 1) + "年的待办数据将于6月30日清理，请及时将数据同步到慧影PC版进行备份！";
		_promptTime.setMonth(5);
		_promptTime.setDate(30);
		_promptTime.setHours(10, 0, 0, 0);
	} else if(_dataType == constants.getConstants("notifyDataType_sysCalNotify")){
		//设置提醒信息（提醒ID：1000000008，提醒类型：使用系统日历提醒，提醒时间：每周日 晚上8点）
		_notifyId = 1000000008;
		_localMsgType = constants.getConstants("localMsgType_sysCalNotify");
		_title = "一个开关搞定时间流不准时提醒";
		_content = "时间流不准时提醒？试试使用系统日历提醒吧！";
		// 获取按周重复提醒时间（每周日）
		_promptTime = getWeekCyclePromptTime(_nowTime, 0);
	} else if(_dataType == constants.getConstants("notifyDataType_autoSyncData")){
		//设置提醒信息（提醒ID：1000000009，提醒类型：自动同步提醒，提醒时间：每周3 晚上8点）
		_notifyId = 1000000009;
		_localMsgType = constants.getConstants("localMsgType_autoSyncData");
		_content = "总是忘记手动同步？快来开启自动同步吧！";
		// 获取按周重复提醒时间（每周日）
		_promptTime = getWeekCyclePromptTime(_nowTime, 3);
	}
	_nowTime = new Date();
	//判断提醒时间是否已经过去，若未过去，则设置提醒；否则不设置提醒
	if(_promptTime.getTime() > _nowTime.getTime()){
		//设置本地通知
		var _promptTimeText = calendar.getStandardTimeText(_promptTime);
		var _notifyData = {
				notifyTime: _promptTimeText,
				notifyId: _notifyId,
				contentText: _content,
				contentTitle: _title,
				extra: {isShow: false, dataid: "", dataType: _dataType},
				repeatMode: "None",
				ringing: notificationPath
			};
		print("_notifyData " + JSON.stringify(_notifyData));
		// 若根据系统类型，调用不同的组件设置提醒
		if(!isIOS){
			var M0026_myNotify = deviceone.sm("M0026_myNotify");
			M0026_myNotify.addNotify(_notifyData);
		} else {
			var do_LocalNotification = deviceone.sm("do_LocalNotification");
			do_LocalNotification.addNotify(_notifyData);
		}
		// 设置定时任务的本地消息
		setFixedTimeLocalMsg(_localMsgType, _promptTime.getTime(), 
				_notifyId, _title, _content);
	}
}

//设置定时任务的本地提醒（待办统计、数据同步及清除待办等）
module.exports.setFixedTimeLocalNotification = function(_changedData){
	setFixedTimeLocalNotification(_changedData);
};

/**
 * 移除数据清理相关的本地提醒
 * @returns
 */
function removeClearDataLocalNotif(){
	var do_LocalNotification = sm("do_LocalNotification");
	// 移除定时提醒通知
	do_LocalNotification.removeNotify([1000000002, 1000000006, 1000000007]);
}

/**
 * 设置访问系统日历本地提醒
 * @returns
 */
function setSysCalFixedTimeNotif(){
	var _isAllow = false;
	//允许读取系统日历信息
	if(do_dataCache.hasData("allowCalendar")){
		var _allowCalendar = do_dataCache.loadData("allowCalendar");
		//用户允许的情况下才读取用户信息
		if(_allowCalendar == "true"){
			_isAllow = true;
		}
	}else{
		_isAllow = true;
	}
	if(!_isAllow){
		// 设置访问系统日历本地提醒
		setFixedTimeLocalNotification({
			dataType: constants.getConstants("notifyDataType_sysCalendar")
		});
	} else {
		// 移除访问系统日历的本地提醒
		delTimeFlowNotify([1000000004]);
	}
}

// 设置打开自动同步开关的本地提醒
module.exports.setSysCalFixedTimeNotif = function(){
	setSysCalFixedTimeNotif();
};

/**
 * 设置打开自动同步开关的本地提醒
 * @returns
 */
function setAutoSyncFixedTimeNotif(){
	var _isAutoSync = false;
	// 时间流自动同步
	if(do_dataCache.hasData(constants.getConstants("autoSync_tf"))){
		_isAutoSync = true;
	}
	// 联系人自动同步
	if(do_dataCache.hasData(constants.getConstants("autoSync_contact"))){
		_isAutoSync = true;
	}
	// 获取信息库同步信息
	var _expSyncInfo = dbexpsyncdata.querySyncInfo();
	// 信息库自动同步
	if(syncGlobalConst.setting.auto_sync_yes == _expSyncInfo.autosync){
		_isAutoSync = true;
	}
	if(!_isAutoSync){
		//设置自动同步本地提醒
		setFixedTimeLocalNotification({
			dataType: constants.getConstants("notifyDataType_autoSyncData")
		});
	} else {
		// 移除打开自动同步开关的本地提醒
		delTimeFlowNotify([1000000009]);
	}
}

// 设置打开自动同步开关的本地提醒
module.exports.setAutoSyncFixedTimeNotif = function(){
	setAutoSyncFixedTimeNotif();
};

/**
 * 设置定时任务的本地提醒（待办统计、数据同步及清除待办等）
 * @returns
 */
function setAllFixedTimeLocalNotif(){
	//设置待办统计本地提醒
	setFixedTimeLocalNotification({
		dataType: constants.getConstants("notifyDataType_taskStatis")
	});
	
	//设置数据同步本地提醒
	setFixedTimeLocalNotification({
		dataType: constants.getConstants("notifyDataType_syncData")
	});
	
	/**
	 * 暂时移除数据清理本地推送通知
	 */
	removeClearDataLocalNotif();
//	//设置清除待办本地提醒-1
//	setFixedTimeLocalNotification({
//		dataType: constants.getConstants("notifyDataType_clearCache1")
//	});
	
//	//设置清除待办本地提醒-2
//	setFixedTimeLocalNotification({
//		dataType: constants.getConstants("notifyDataType_clearCache2")
//	});
	
//	//设置清除待办本地提醒-3
//	setFixedTimeLocalNotification({
//		dataType: constants.getConstants("notifyDataType_clearCache3")
//	});
	
	//设置访问系统日历本地提醒
	setSysCalFixedTimeNotif();
	
//	//设置年度待办统计本地提醒
//	setFixedTimeLocalNotification({
//		dataType: constants.getConstants("notifyDataType_taskStatisYear")
//	});
	
	// 只有安卓系统才有这个提醒
	if(!isIOS){
//		// 设置使用系统日历提醒的本地提醒
//		setFixedTimeLocalNotification({
//			dataType: constants.getConstants("notifyDataType_sysCalNotify")
//		});
		// 移除使用系统日历提醒的本地提醒
		delTimeFlowNotify([1000000008]);
	}
	
	//设置自动同步本地提醒
	setAutoSyncFixedTimeNotif();
}

//设置所有定时任务的本地提醒（待办统计、数据同步及清除待办等）
module.exports.setAllFixedTimeLocalNotif = function(){
	setAllFixedTimeLocalNotif();
};

/**
 * 设置倒计时提醒
 * @param _notifyTime
 * @returns
 */
function setCountDownLocalNotif(_notifyTime){
	var _promptTime = new Date();
	_promptTime.setTime(_notifyTime);
	//提醒参数
	var _notifyData = {
			notifyId: "1200000001",
			content: "倒计时已结束！",
			isShow: false,
			id: "",
			dataType: constants.getConstants("notifyDataType_countDown")
		};
	print("setCountDownLocalNotif _notifyData " + JSON.stringify(_notifyData));
	//添加时间流提醒
	addTimeFlowNotify(_notifyData, _promptTime, notificationPath);
}

//设置倒计时提醒
module.exports.setCountDownLocalNotif = function(_notifyTime){
	setCountDownLocalNotif(_notifyTime);
};

/**
 * 删除倒计时提醒
 * @returns
 */
function delCountDownLocalNotif(){
	var _notifyId = "1200000001";
	print("delCountDownLocalNotif notifyId " + _notifyId);
	//移除时间流提醒
	delTimeFlowNotify([_notifyId]);
}

//删除倒计时提醒
module.exports.delCountDownLocalNotif = function(){
	delCountDownLocalNotif();
};
