/**
 * shareTools.js
 * 分享工具
 * 
 * @Author : xiufu
 * @Timestamp : 2016-09-05
 */
//引入组件库
var deviceone = require("deviceone");
var do_nf = deviceone.sm("do_Notification");
var do_initData = deviceone.sm("do_InitData");
var do_global = deviceone.sm("do_Global");
var do_storage = deviceone.sm("do_Storage");
//引入第三方组件
var do_qq = deviceone.sm("do_TencentQQ");
var do_wx = deviceone.sm("do_TencentWX");

//js文件路径管理
var jsPathManager = require("jsPathManager.ui");
//全局常量
var globalConst = require(jsPathManager.getGlobalConstPath());
//url常量
var urlConst = require(jsPathManager.getUrlConstPath());
//http通信
var http = require(jsPathManager.getHttpPath());
//权限请求工具
var permReqTools = require(jsPathManager.getPermReqToolsPath());

//logo图片地址
var logoImgPath = "data://logo1.jpg";
//有电话和存储权限才拷贝logo图片到本地文件目录
if(permReqTools.checkPhonePerm() &&
		permReqTools.checkStorePerm()){
	//拷贝应用logo图片
	if(!do_storage.fileExist(logoImgPath)){
		do_initData.copyFile("initdata://logo1.png", 
				logoImgPath, function(_data) {
			
		});
	}
}

//选项配置
var options = {
	    qq: "1105292009",
	    wx: "wx0c3685e52cfaf329",
	    wxLink: "/app/weixin/",
	    qqLink: "/qq_conn/1105292009",
	    title: "慧影时间流，时间、人脉、个人信息全方位智能管理",
	    image: logoImgPath,
	    commonUrl: urlConst.constants.common,
	    url: urlConst.constants.commonappdownload + "?type=5",
	    summary: "待办倒计时纪念日倒数日课程表清单笔记日记备忘合同保单账户资料文件总结通知规划",
	    success: "分享成功！",
	    fail: "分享失败！",
	    file: "文件分享",
	    appName: "慧影时间流"
	};

/**
 * 获取universal链接
 * @param _callFunc
 * @returns
 */
function getUniversalLink(_callFunc){
	// 请求参数
	var _options = {
		url: urlConst.constants.commonuniversallink,
		type: "GET",
		contentType: "text/plain",
		timeout: 30000,
		complete: function(_data){
			print("getUniversalLink " + JSON.stringify(_data));
			if(null != _data && "" != _data){
				var _universalLinkKey = globalConst.globalMemoryKey.niversalLinkKey;
				do_global.setMemory(_universalLinkKey, _data);
				_callFunc.call(this, _data);
			}
		},
		error: function(){
			print("getUniversalLink fail ");
			_callFunc.call(this, null);
		}
	};
	http.commonRequest(_options);
}

/**
 * 检测universal链接
 * @param _isWx
 * @param _callFunc
 * @returns
 */
function checkUniveralLink(_isWx, _callFunc){
	var _universalLinkKey = globalConst.globalMemoryKey.niversalLinkKey;
	var _universalLink = do_global.getMemory(_universalLinkKey);
	var _link = "";
	if(null != _universalLink && "" != _universalLink){
		if(_isWx){
			_link = _universalLink + options.wxLink;
		} else {
			_link = _universalLink + options.qqLink;
		}
		_callFunc.call(this, _link);
	} else {
		getUniversalLink(function(_data){
			if(null != _data && "" != _data){
				do_global.setMemory(_universalLinkKey, _data);
				if(_isWx){
					_link = _data + options.wxLink;
				} else {
					_link = _data + options.qqLink;
				}
			} else {
				if(_isWx){
					_link = options.commonUrl + options.wxLink;
				} else {
					_link = options.commonUrl + options.qqLink;
				}
			}
			_callFunc.call(this, _link);
		});
	}
}

/**
 * 分享给QQ好友
 * @returns
 */
function shareToQQ(){
	checkUniveralLink(false, function(_qqLink){
		var _shareUrl = options.url + "&chnl=1";
		do_qq.shareToQQ({
			appId: options.qq,
			qqLink: _qqLink,
			type: 0,
			title: options.title,
			url: _shareUrl,
			image: options.image,
			summary: options.summary,
			audio: "",
			appName: options.appName
		}, function(_data) {
			deviceone.print("share to qq: " + _data);
			if(_data){
				do_nf.toast(options.success);
			}
		});
	});
}

//分享给QQ好友
module.exports.shareToQQ = function() {
	shareToQQ();
}

/**
 * 分享到微信
 * 分享给微信好友 wxScene = 0，分享到微信朋友圈：wxScene = 1
 * @param _wxScene
 * @returns
 */
function shareToWX(_wxScene){
	checkUniveralLink(true, function(_wxLink){
		var _shareUrl = options.url + "&chnl=2";
		if(1 == _wxScene){
			_shareUrl = options.url + "&chnl=3";
		}
		do_wx.share({
			appId : options.wx,
			wxLink: _wxLink,
			scene : _wxScene,
			type : 0,
			title : options.title,
			content : options.summary,
			url : _shareUrl, 
			image : options.image
		}, function(_data) {
			if(_data){		
				do_nf.toast(options.success);
			} 
		});
	});
}

//分享到微信
module.exports.shareToWX = function(_wxScene) {
	shareToWX(_wxScene);
}

/**
 * 卡片信息分享到QQ
 * 分享的内容：图片的路径（_path）
 * @param _path
 * @returns
 */
function shareCardToQQ(_path){
	checkUniveralLink(false, function(_qqLink){
		do_qq.shareToQQ({
			appId: options.qq,
			qqLink: _qqLink,
			type: 1,
			image: _path,
			appName: options.appName
		}, function(_data) {
			if(_data){
				do_global.fire("ShareSuccessNotification");
			}
		});
	});
}

//卡片信息分享到QQ，分享的内容：图片的路径（_path）
module.exports.shareCardToQQ = function(_path) {
	shareCardToQQ(_path);
}

/**
 * 卡片信息分享到微信
 * 分享给微信好友：wxScene = 0，分享到微信朋友圈：wxScene = 1
 * 分享的内容：图片的路径（_path）
 * 图片缩略图路径（thumbpath）
 * @param _wxScene
 * @param _path
 * @param _thumbPath
 * @returns
 */
function shareCardToWX(_wxScene, _path, _thumbPath){
	checkUniveralLink(true, function(_wxLink){
		do_wx.share({
			appId: options.wx,
			wxLink: _wxLink,
			scene: _wxScene, 
			type: 1,
			image: _path,
			trumb: _thumbPath
		}, function(_data) {
			print("shareCardToWX " + _data);
			if(_data){
				do_global.fire("ShareSuccessNotification");
			}
		});
	});
}

//卡片信息分享到微信
module.exports.shareCardToWX = function(_wxScene, _path, _thumbPath) {
	shareCardToWX(_wxScene, _path, _thumbPath);
}

/**
 * 文件分享到微信
 * 分享给微信好友：wxScene = 0
 * @param _filePath
 * @param _fileName
 * @param _callFunc
 * @returns
 */
function shareFileToWX(_filePath, _fileName, _callFunc){
	checkUniveralLink(true, function(_wxLink){
		var _wxScene = 0;
		do_wx.share({
			appId: options.wx,
			wxLink: _wxLink,
			scene: _wxScene, 
			type: 3,
			title: _fileName,
			content: options.file,
			filePath: _filePath
		}, function(_data, e) {
			if(_data){		
				do_nf.toast(options.success);
			}
			_callFunc.call(this, _data);
		});
	});
}

//文件分享到微信
module.exports.shareFileToWX = function(_filePath, _fileName, _callFunc) {
	shareFileToWX(_filePath, _fileName, _callFunc);
}
