/**
 * 工具
 * 
 * @Author : Cydow
 * @Timestamp : 2017-02-22
 */
var deviceone = require("deviceone");
var do_dataCache = deviceone.sm("do_DataCache");
var do_initData = deviceone.sm("do_InitData");
var do_storage = deviceone.sm("do_Storage");
var do_device = deviceone.sm("do_Device");
var do_global = deviceone.sm("do_Global");

var tmpColorIndex=0;
var colorData=["899dc0ff", "83af9bff", "cda49eff", "adc3c0ff", "979797ff"];

/**
 * 格式化文件大小, 输出成带单位的字符串
 * @param {Number} size 文件大小
 * @param {Number} [pointLength=2] 精确到的小数点数。
 * @param {Array} [units=[ 'B', 'K', 'M', 'G', 'TB' ]] 单位数组。从字节，到千字节，一直往上指定。
 *    如果单位数组里面只指定了到了K(千字节)，同时文件大小大于M, 此方法的输出将还是显示成多少K.
 */
function formatSize(size, pointLength, units) {
  var unit;
  units = units || [ 'B', 'KB', 'MB', 'GB', 'TB' ];
  while ( (unit = units.shift()) && size > 1024 ) {
      size = size / 1024;
  }
  return (unit === 'B' ? size : size.toFixed( pointLength === undefined ? 2 : pointLength)) + "" + unit;
}

// 格式化显示文件大小
module.exports.formatSize = function(size){
	return formatSize(size, 2, null);
};

module.exports.getRadomColor = function(){
	tmpColorIndex ++;
	if (tmpColorIndex >=5) tmpColorIndex = 0;
	return colorData[tmpColorIndex];
};

module.exports.getTimeDesc = function(_time){
	var _t=new Date(_time);
	return _t.getFullYear() + "-" +(_t.getMonth() + 1) + "-" + _t.getDate()
			+ " " + _t.getHours() + ":" + _t.getMinutes() + ":" + _t.getSeconds();
};

/**
 * 提醒时长转换（远端到本地）
 * @param _remindTime
 * @returns
 */
function remindTimeRemoteToLocal(_remindTime){
	var _remindTimeIndex = 17;
	switch(_remindTime + ""){
	case "10": //10分钟
		_remindTimeIndex = 0;
		break; 
	case "20": //20分钟
		_remindTimeIndex = 1;
		break;
	case "30": //30分钟
		_remindTimeIndex = 2;
		break;
	case "60": //1小时
		_remindTimeIndex = 3;
		break;	
	case "120": //2小时
		_remindTimeIndex = 4;
		break;
	case "240": //4小时
		_remindTimeIndex = 5;
		break;
	case "1440": //1天
		_remindTimeIndex = 6;
		break;
	case "2880": //2天
		_remindTimeIndex = 7;
		break;
	case "10080": //1周
		_remindTimeIndex = 8;
		break;
	case "-1": //不提醒
		_remindTimeIndex = 9;
		break;
	case "5": //5分钟
		_remindTimeIndex = 10;
		break;
	case "15": //15分钟
		_remindTimeIndex = 11;
		break;
	case "360": //6小时
		_remindTimeIndex = 12;
		break;
	case "720": //12小时
		_remindTimeIndex = 13;
		break;
	case "4320": //3天
		_remindTimeIndex = 14;
		break;
	case "21600": //半个月
		_remindTimeIndex = 15;
		break;
	case "43200": //一个月
		_remindTimeIndex = 16;
		break;
	case "0": //正点提醒
		_remindTimeIndex = 17;
		break;
	}
	
	return _remindTimeIndex;
}

/**
 * 提醒时长转换（本地到远端）
 * @param _remindTime
 * @returns
 */
function remindTimeLocalToRemote(_remindTimeIndex){
	var _remindTime = 0;
	switch (_remindTimeIndex + ""){
	case "0": //10分钟
		_remindTime = 10;
		break;
	case "1": //20分钟
		_remindTime = 20;
		break;
	case "2": //30分钟
		_remindTime = 30;
		break;
	case "3": //1小时
		_remindTime = 60;
		break;
	case "4": //2小时
		_remindTime = 120;
		break;
	case "5": //4小时
		_remindTime = 240;
		break;
	case "6": //1天
		_remindTime = 60*24;
		break;
	case "7": //2天
		_remindTime = 60*24*2;
		break;
	case "8": //1周
		_remindTime = 60*24*7;
		break;
	case "9": //不提醒
		_remindTime = -1;
		break;
	case "10": //5分钟
		_remindTime = 5;
		break;
	case "11": //15分钟
		_remindTime = 15;
		break;
	case "12": //6小时
		_remindTime = 360;
		break;
	case "13": //12小时
		_remindTime = 720;
		break;
	case "14": //3天
		_remindTime = 60*24*3;
		break;
	case "15": //半个月
		_remindTime = 60*24*15;
		break;
	case "16": //一个月
		_remindTime = 60*24*30;
		break;
	case "17": //正点提醒
		_remindTime = 0;
		break;
	}
	return _remindTime;
}

//获取提醒时长
module.exports.getRemindTimeValue = function(_remindTime){
	var _remindTimeValue = remindTimeLocalToRemote(_remindTime) * 1000 * 60;
	return  _remindTimeValue;
}

//获取提前提醒时长（以分钟为单位）
module.exports.getRemindTime = function(_remindTime){
	return remindTimeLocalToRemote(_remindTime);
}

//获取转化为远端类型的提醒时长
module.exports.getRemoteRemindTime = function(_remindTime){
	return remindTimeLocalToRemote(_remindTime);
}

// 获取信息库信息的提前提醒时长
module.exports.getExpRemindTime = function(_remindTime){
	return remindTimeLocalToRemote(_remindTime);
}

//获取转化为本地类型的提醒时长
module.exports.getLocalRemindTime = function(_remindTime){
	return remindTimeRemoteToLocal(_remindTime);
}

/**
 * 删除字符串中的换行符,将换行用空格代替
 * @param str
 * @returns
 */
function delNewLine(_str){ 
	_str = _str.replace(/[\n]/g, " ");
	return _str.replace(/\ +/g, " ");
}

//删除字符串中的换行符
module.exports.delNewLine = function(_str){
	return delNewLine(_str);
}

/**
 * 删除左右两端的空格
 * 
 * @param str
 * @returns
 */
function trimStr(_str){ 
    return _str.replace(/(^\s*)|(\s*$)/g, "");
}

//删除字符串左右两端的空格
module.exports.trim = function(_str){
	return trimStr(_str);
}

/**
 * 删除字符串左边的空格
 * 
 * @param str
 * @returns
 */
function ltrimStr(_str){
	return _str.replace(/(^\s*)/g,"");
}

//删除字符串左边的空格
module.exports.ltrim = function(_str){
    return ltrimStr(_str);
}

/**
 * 删除字符串右边的空格
 * 
 * @param str
 * @returns
 */
function rtrimStr(_str){
	return _str.replace(/(\s*$)/g,"");
}

//删除字符串右边的空格
module.exports.rtrim = function(_str){
    return rtrimStr(_str);
}

/**
 * 删除字符串的空格
 * 
 * @param str
 * @returns
 */
function removeStrSpace(_str){
	if(undefined == _str || null == _str)
		return "";
	return _str.replace(/\s/g,'');
}

//删除字符串的空格
module.exports.removeStrSpace = function(_str){
    return removeStrSpace(_str);
}

/**
 * 删除字符串的横线
 * 
 * @param str
 * @returns
 */
function removeStrLine(_str){
	return _str.replace(/-/g,'');
}

//删除字符串的横线
module.exports.removeStrLine = function(_str){
    return removeStrLine(_str);
}

/**
 * 删除字符串首尾换行（\n）
 * @param str
 * @returns
 */
function removeStrBr(_str){
	return _str.replace(/-/g,'');
}

// 删除字符串收尾换行（\n）
module.exports.removeStrBr = function(_str){
    return removeStrBr(_str);
}

/**
 * 删除字符串的空格和横线（'-'）
 * 
 * @param str
 * @returns
 */
function removeStrSpaceAndLine(_str){
	return _str.replace(/\s|-/g,'');
}

//删除字符串的空格和横线（line）
module.exports.removeStrSpaceAndLine = function(_str){
    return removeStrSpaceAndLine(_str);
}

/**
 * 格式化地址字段
 * 
 * @param str
 * @returns
 */
function formatLoc(_data){
	var _name = trimStr(_data.name);
	var _address = trimStr(_data.address);
	var _location = _name;
	if("" != _address){
		if("" != _name){
			_location += "，" + _address;
		} else {
			_location = _address;
		}
	}
	return _location;
}

//格式化地址字段
module.exports.formatLoc = function(_data){
	return formatLoc(_data);
}

/**
 * 格式化sql语句（字符串格式化）
 * @param _value
 * @returns {String}
 */
function buildSqlValue(_value){
	if (_value==null){
		_value="";
	}
	return "'" + (_value + "").replace(/'/g, "''") + "'";
}

//格式化sql语句（字符串格式化）
module.exports.formatSqlValue = function(_value){
	return buildSqlValue(_value);
}

/**
 * 格式化sql语句（字符串格式化）
 * @param _value
 * @returns {String}
 */
function buildSqlValueNoQuote(_value){
	if (_value==null){
		_value="";
	}
	return (_value + "").replace(/'/g, "''");
}

//格式化sql语句（字符串格式化）
module.exports.formatSqlValueNoQuote = function(_value){
	return buildSqlValueNoQuote(_value);
}

/**
 * 字符串转数值（主要是用于转换id）
 * @param _str
 * @returns {Number}
 */
function str2Number(_str){		
	if (_str==null) return 0;
	var _num=0;
	for(var i=0; i<_str.length;++i){
		_num+= _str.charCodeAt(i);
		if (_num >= 9999999)_num=_num%9999999;
	}
	return _num;
}

//字符串转数值（主要是用于转换id）
module.exports.str2Number = function(_str){
	return str2Number(_str);
}

/**
 * 获取更新计数
 * 
 * @returns
 */
var SyncCountkeyName="LocalSyncCount";
function _getSyncSequence(){
	if (!do_dataCache.hasData(SyncCountkeyName)) {
		do_dataCache.saveData(SyncCountkeyName, "1");
		return 1;
	}
	var _strData = do_dataCache.loadData(SyncCountkeyName);
	var _intData = parseInt(_strData) + 1;
	do_dataCache.saveData(SyncCountkeyName, _intData);
	return _intData;
}

//获取更新计数
module.exports.getSyncSequence = function(){
	return _getSyncSequence();
};

/**
 * 设置提醒id
 * 
 * @returns
 */
var notifyIdKey = "notifyIdKey";
function setNotifyId(_notifyId){
	if(!isNullOrEmpty(_notifyId)){
		do_dataCache.saveData(notifyIdKey, parseInt(_notifyId));
	}
}

//设置提醒id
module.exports.setNotifyId = function(_notifyId){
	setNotifyId(_notifyId);
};

/**
 * 获取提醒id
 * 
 * @returns
 */
function getNotifyId(){
	//待办提醒 id小于1000000000，之后的值用于其他特殊提醒
	var _notifyIdMax = 1000000000;
	var _intData = 10;
	if(do_dataCache.hasData(notifyIdKey)){
		var _notifyId = do_dataCache.loadData(notifyIdKey);
		if(!isNullOrEmpty(_notifyId) && 
				"null" != _notifyId){
			_intData = parseInt(_notifyId) + 1;
			if(_intData >= _notifyIdMax){
				_intData = 10;
			}
		}
	}
	setNotifyId(_intData);
	return _intData;
}

//获取提醒id
module.exports.getNotifyId = function(){
	return getNotifyId();
};

/**
 * 创建数据的id
 * 
 * @param _id
 * @param _index
 * @returns
 */
function createTaskId(_id, _index){
	var _taskId = _id + "000000";
	var _indexTmp = _index + "";
	_taskId = _taskId.substr(0, _taskId.length - _indexTmp.length) + _indexTmp;
	return _taskId;
}

//创建数据的id
module.exports.createTaskId = function(_id, _index){
	return createTaskId(_id, _index);
};

///**
// * [isEqualIPAddress 判断两个IP地址是否在同一个网段]
// * @param  addr1 [地址一]
// * @param  addr2 [地址二]
// * @return {Boolean} [true or false]
// */
//function isEqualIPAddress (addr1, addr2){
//    if(null == addr1 || null == addr2){
//        return false;
//    }
//    var res1 = [], res2 = [];
//    res1 = addr1.split(".", 3); //暂时只分割ip地址前面三段
//    res2 = addr2.split(".", 3);
//    if(res1.join(".") == res2.join(".")){
//        return true;
//    }else{
//        return false;
//    }
//}
//
//module.exports.isEqualIPAddress = function(addr1, addr2){
//	return isEqualIPAddress (addr1, addr2);
//}

/**
 * [isEqualIPAddress 判断两个IP地址是否在同一个网段]
 * @param  {[String]}  addr1 [地址一]
 * @param  {[String]}  addr2 [地址二]
 * @param  {[String]}  mask  [子网掩码]
 * @return {Boolean}         [true or false]
 */
function isEqualIPAddress (addr1, addr2, mask){
    if(!addr1 || !addr2 || !mask){
        return false;
    }
    var res1 = [], res2 = [];
    addr1 = addr1.split(".");
    addr2 = addr2.split(".");
    mask  = mask.split(".");
    for(var i = 0,ilen = addr1.length; i < ilen; i += 1){
        res1.push(parseInt(addr1[i]) & parseInt(mask[i]));
        res2.push(parseInt(addr2[i]) & parseInt(mask[i]));
    }
    if(res1.join(".") == res2.join(".")){
        return true;
    } else {
        return false;
    }
}

//判断两个IP地址是否在同一个网段
module.exports.isEqualIPAddress = function(_localIp, _remoteIp, _mask){
	return isEqualIPAddress(_localIp, _remoteIp, _mask);
};

/**
 * ip和端口号
 * 
 * @param _url
 * @returns
 */
function isIpAndPort(_url){
	// IP形式的URL- 192.168.1.184
	// port形式：220
	var _strRegex = /^([0-9]{1,3}\.){3}[0-9]{1,3}(:[0-9]{1,4})$/;
	return _strRegex.test(_url);
}

//ip和端口号
module.exports.isIpAndPort = function(_url){
	return isIpAndPort(_url);
}

/**
 * 校验手机号
 * 
 * @param str
 * @returns
 */
function isPhone(_str){
	if(null == _str || "" == _str){
		return false;
	}
	//删除号码中间的横线
	_str = removeStrLine(_str);
	//删除号码中间的空格
	_str = removeStrSpace(_str);
	if(_str.length != 11){
		return false;
	}
	//校验是否为手机号码
	var _strRegex = "^[1][3,4,5,6,7,8,9][0-9]{9}$";
    var _pattern = new RegExp(_strRegex);
	return _pattern.test(_str);
}

//校验手机号
module.exports.isPhone = function(_str){
	return isPhone(_str);
}

/**
 * 校验电子邮件地址
 * 
 * @param str
 * @returns
 */
function isEmail(_str){
	var _strRegex = "^\\w+([\\.|+|-]|[A-Za-z0-9])*[A-Za-z0-9+]\\@[A-Za-z0-9]+((\\.|-)[A-Za-z0-9]+)*\\.[A-Za-z]+";
    var _pattern = new RegExp(_strRegex);
	return _pattern.test(_str);
}

//校验电子邮件地址
module.exports.isEmail = function(_str){
	return isEmail(_str);
}

/**
 * 检测字符串是否全为字母
 * @param _str
 * @returns
 */
function isChar(_str){
	var str = /^[A-Za-z]*$/;
	if(str.test(_str)){
		return true;	
	}
	return false;
}

//检测字符串是否全为字母
module.exports.isChar = function(_str){
	return isChar(_str);
}

/**
 * 检测是否全为数字字符串
 * @param _str
 * @returns
 */
function isFullDigit(_str){
	var _pattern = /^\d+$/;
	if(_pattern.test(_str)){
		return true;	
	}
	return false;
}

//检测是否全为数字字符串
module.exports.isFullDigit = function(_str){
	return isFullDigit(_str);
}

/**
 * 截取指定长度的子串并填充字符（...）
 * @param _str
 * @param _length
 * @returns
 */
function subAndFillStr(_str, _length){
	if(isNullOrEmpty(_str)) return "";
	if(_str.length > _length){
		_str = _str.substr(0, _length);
		_str += "...";
	}
	return _str;
}

// 截取指定长度的子串并填充字符
module.exports.subAndFillStr = function(_str, _length){
	return subAndFillStr(_str, _length);
}

var emojiStr = /[\uD83C|\uD83D|\uD83E][\uDC00-\uDFFF][\u200D|\uFE0F]|[\uD83C|\uD83D|\uD83E][\uDC00-\uDFFF]|[0-9|*|#]\uFE0F\u20E3|[0-9|#]\u20E3|[\u203C-\u3299]\uFE0F\u200D|[\u203C-\u3299]\uFE0F|[\u2122-\u2B55]|\u303D|[\A9|\AE]\u3030|\uA9|\uAE|\u3030/;
/**
 * 截取指定长度的表情子串并填充字符（...）
 * @param _str
 * @param _length
 * @returns
 */
function subEmojiStr(_str, _length){
	var _result = "";
	if(null != _str && _str.length > 0){
		if (_str.length > _length){
			if(emojiStr.test(_str.substring(_length - 1, _length + 1))){
				_length += 1;
			}
			_result = _str.substr(0, _length) + "...";
		} else {
			_result = _str;
		}
		
	}
    return _result;
}

// 截取指定长度的子串并填充字符
module.exports.subEmojiStr = function(_str, _length){
	return subEmojiStr(_str, _length);
}

/**
 * 是否为空（undefine || null）
 * @param _data
 * @returns
 */
function isNull(_data){
	if(undefined == _data || 
			null == _data ||
			"null" == _data){
		return true;	
	}
	return false;
}

//是否为空（undefine || null）
module.exports.isNull = function(_data){
	return isNull(_data);
}

/**
 * 是否为空字符串（undefine || null || length == 0）
 * @param _data
 * @returns
 */
function isNullOrEmpty(_data){
	if(isNull(_data) || _data.length == 0){
		return true;	
	}
	return false;
}

//是否为空字符串（undefine || null || length == 0）
module.exports.isNullOrEmpty = function(_data){
	return isNullOrEmpty(_data);
}

/**
 * 获取设备系统信息
 * @returns
 */
function getDeviceInfo(){
	// 获取当前系统类型（android或iOS）
	var _curOS = do_device.getOS();
	// 是否iOS系统
	var _isIOS = true;
	if("android" == _curOS){
		_isIOS = false;
	}
	//获取设备系统信息
	var _osInfo = do_device.getInfo();
	var _deviceModel = _osInfo.phoneType;
	var _deviceName = _osInfo.deviceName;
	var _deviceOs = "ios";
	var _barStateS = "show";
	var _barStateT = "transparent";
	var _barBgColor = "000000FF";
	var _barFgColor = "white";
	if(!_isIOS){
		_deviceOs = "android";
		_deviceModel = _osInfo.deviceName;
		_deviceName = _osInfo.deviceName;
	} else {
		_barBgColor = "FFFFFFFF";
		_barFgColor = "black";
		if(undefined == _deviceModel || 
				null == _deviceModel || 
				"" == _deviceModel){
			_deviceModel = "unknown";
			_barStateS = "show_XFullScreen";
			_barStateT = "transparent_XFullScreen";
		} else if(_deviceModel.indexOf("iPhone X") > -1){
			_barStateS = "show_XFullScreen";
			_barStateT = "transparent_XFullScreen";
		}
	}
	//设备标识
	if(null == _osInfo.deviceId || "" == _osInfo.deviceId){
		_osInfo.deviceId = do_dataCache.loadData("machineId");
	}
	//设备标识码(取自当前时间的毫秒数)
	var _sdCode = (new Date()).getTime() + "";
	//设备系统信息
	var _deviceInfo = {
		devicetype: "mobile",
		deviceos: _deviceOs,
		deviceid: _osInfo.deviceId,
		devicemodel: _deviceModel,
		devicename: _deviceName,
		osver: _osInfo.OSVersion,
		sdkver: _osInfo.sdkVersion,
		sdcode: _sdCode,
		barStateS: _barStateS,
		barStateT: _barStateT,
		barBgColor: _barBgColor,
		barFgColor: _barFgColor,
		isIOS: _isIOS
	};
	return _deviceInfo;
}

//获取设备系统信息
module.exports.getDeviceInfo = function(){
	return getDeviceInfo();
}

/**
 * 设置登录票据
 * @returns
 */
var accessTicketKeyName = "accessTicket";
function setAccessTicket(_accessTicket){
	do_dataCache.saveData(accessTicketKeyName, _accessTicket)
}

//设置登录票据
module.exports.setAccessTicket = function(_accessTicket){
	return setAccessTicket(_accessTicket);
}

/**
 * 获取登录票据
 * @returns
 */
function getAccessTicket(){
	var _accessTicket = "";
	if(do_dataCache.hasData(accessTicketKeyName)){
		_accessTicket = do_dataCache.loadData(accessTicketKeyName);
	}
	return _accessTicket;
}

//获取登录票据
module.exports.getAccessTicket = function(){
	return getAccessTicket();
}

/**
 * 获取背景图片文件的路径
 * @returns
 */
function getBgImagePath(){
	var _imageFilePath = do_global.getMemory("bgImageFilePath");
	if(isNullOrEmpty(_imageFilePath)){
		if(do_storage.fileExist("data://bgimg.txt")){
			var _url = do_storage.readFileSync("data://bgimg.txt");
			if (!isNullOrEmpty(_url)){
				if(_url.indexOf("source://image/") > -1){
					_imageFilePath = _url;
				} else {
					if(do_storage.fileExist(_url)){
						_imageFilePath = _url;	
					}
				}
			}
		}
	}
	// 判断背景图片文件路径是否为空，若不空，则将该文件路径存到全局缓存中
	if(isNullOrEmpty(_imageFilePath)){
		_imageFilePath = "";
	} else {
		// 将背景图片缓存到全局缓存中
		do_global.setMemory("bgImageFilePath", _imageFilePath);
	}
	return _imageFilePath;
}

// 获取背景图片文件的路径
module.exports.getBgImagePath = function(){
	return getBgImagePath();
}

/**
 * 设置背景图片文件的路径
 * @param _imageFilePath
 * @returns
 */
function setBgImagePath(_imageFilePath){
	// 将背景图片缓存到全局缓存中
	do_global.setMemory("bgImageFilePath", _imageFilePath);
	
}

// 设置背景图片文件的路径
module.exports.setBgImagePath = function(_imageFilePath){
	setBgImagePath(_imageFilePath);
}

/**
 * 获取所有默认背景图片文件的路径
 * @returns
 */
function getAllDefaultBgImgPath(){
	var _imgPathList = [
		"source://image/default_bg001.jpg",
		"source://image/default_bg002.jpg",
		"source://image/default_bg003.jpg",
		"source://image/default_bg02.jpg",
		"source://image/default_bg03.jpg",
		"source://image/default_bg04.jpg",
		"source://image/default_bg07.jpg",
		"source://image/default_bg08.jpg",
		"source://image/default_bg09.jpg",
		"source://image/default_bg10.jpg",
		"source://image/default_bg11.jpg",
		"source://image/default_bg12.jpg",
		"source://image/default_bg13.jpg",
		"source://image/default_bg14.jpg"
	];
	return _imgPathList;
}

// 设置背景图片文件的路径
module.exports.getAllDefaultBgImgPath = function(){
	return getAllDefaultBgImgPath();
}

/**
 * 随机获取一张默认背景图片文件的路径
 * @returns
 */
function getRandomDefBgImgPath(){
	var _imgPathList = getAllDefaultBgImgPath();
	var _imgNum = _imgPathList.length;
	var _random = Math.floor(Math.random() * _imgNum);
	var _imgPath = _imgPathList[0];
	if(0 <= _random && _random < _imgNum){
		_imgPath = _imgPathList[_random];
	}
	return _imgPath;
}

// 随机获取一张默认背景图片文件的路径
module.exports.getRandomDefBgImgPath = function(){
	return getRandomDefBgImgPath();
}

/**
 * 随机获取一张默认背景图片文件的路径
 * @returns
 */
function getPromptAudioFilePath(_type){
	var _audioFilePath = "data://";
	if(0 == _type){
		return _audioFilePath + "prompt.m4a";
	} else if(1 == _type){
		return _audioFilePath + "notification.m4a";
	} else {
		return _audioFilePath + "notification.m4a";
	}
}

//保存提醒音频文件
module.exports.getPromptAudioFilePath = function(_type){
	return getPromptAudioFilePath(_type);
}

/**
 * 保存提醒音频文件
 * @param _callFunc
 * @returns
 */
function savePromptAudioFile(_callFunc){
	//初始化音频列表
	var _initAudioList = [
		"initdata://bell/prompt.m4a",
		"initdata://bell/notification.m4a"
	];
	//音频文件本地存储目录
	var _audioFilePath = "data://";
	var _promptPath = _audioFilePath + "notification.m4a";
	if(!do_storage.fileExist(_promptPath)){
		//拷贝通知声音文件
		do_initData.copy(_initAudioList, _audioFilePath, 
				function(_data) {
			_callFunc.call(this, _data);
		});
		return;
	}
	_callFunc.call(this, true);
}

//保存提醒音频文件
module.exports.savePromptAudioFile = function(_callFunc){
	savePromptAudioFile(_callFunc);
}

/**
 * 过滤表情符号
 * @param _txt
 * @returns
 */
function filterEmoji(_txt){
	var ranges = [
	  '\ud83c[\udf00-\udfff]',
	  '\ud83d[\udc00-\ude4f]',
	  '\ud83d[\ude80-\udeff]'
	];
	_txt = _txt.replace(new RegExp(ranges.join('|'), 'g'), '');
	return _txt;
}

//过滤表情符号
module.exports.filterEmoji = function(_txt){
	return filterEmoji(_txt);
}

/**
 * 版本字符串转数字
 * @param _txt
 * @returns
 */
function verStrToNumberStr(_str){
    var _verStr = _str.toString();
    var _verArray = _verStr.split('.');
    var _numPlace = ["0000", "000", "00", "0", ""];
    for (var i=0; i<3; i++){
    	if(i < _verArray.length){
    		var _len = _verArray[i].length;       
            _verArray[i] = _numPlace[_len] + _verArray[i]; 
    	} else {
    		_verArray.push(_numPlace[0]);
    	}
    } 
    var _result = _verArray.join('');
    return _result; 
}

/**
 * 比较版本号
 * @param _oldVer
 * @param _newVer
 * @returns
 */
function compareVersion(_oldVer, _newVer){
    var _oldVerNum = verStrToNumberStr(_oldVer);
    var _newVerNum = verStrToNumberStr(_newVer); 
    if(_oldVerNum == _newVerNum){
    	return 0;    
    } else if(_oldVerNum > _newVerNum){
    	return -1;
    } else if(_oldVerNum < _newVerNum){
    	return 1;
    }
}

// 比较版本
module.exports.compareVersion = function(_oldVer, _newVer){
	return compareVersion(_oldVer, _newVer);
};

/**
 * 比较操作系统版本
 * @param _osVer1
 * @param _osVer2
 * @returns
 */
function compareOSVer(_osVer1, _osVer2){
    var _osVerNum1 = verStrToNumberStr(_osVer1);
    var _osVerNum2 = verStrToNumberStr(_osVer2); 
    if(_osVerNum1 == _osVerNum2){
    	return 0;    
    } else if(_osVerNum1 > _osVerNum2){
    	return -1;
    } else if(_osVerNum1 < _osVerNum2){
    	return 1;
    }
}

// 比较操作系统版本
module.exports.compareOSVer = function(_osVer1, _osVer2){
	return compareOSVer(_osVer1, _osVer2);
};

/**
 * 比较姓名拼音结果
 * @param _name1
 * @param _name2
 * @returns
 */
function compareName(_name1, _name2){
	if(_name1 < _name2){
		return -1;
	} else if(_name1 > _name2){
		return 1;
	} else {
		return 0;
	}
}

/**
 * 比较函数（用于联系人数组排序-姓名首字母为中文或字母）
 * @param _objOne
 * @param _objTwo
 * @returns
 */
function compare(_objOne, _objTwo) {
    var _name1 = _objOne.name;
    var _val1 = _objOne.pinyin;
    if(null == _name1 || "" == _name1){
    	_name1 = _objOne.goodsname;
	}
    var _name2 = _objTwo.name;
    var _val2 = _objTwo.pinyin;
    if(null == _name1 || "" == _name1){
    	_name2 = _objTwo.goodsname;
	}
    if(isChar(_name1[0]) || isChar(_name2[0])){
    	return compareName(_name1, _name2);
    } else {
    	var _val3 = _val1.split(' ')[0];
    	var _val4 = _val2.split(' ')[0];
    	if(_val1 < _val2){
    		if(_val3 == _val4){
    			if(_name1[0] != _name2[0] && _name1 > _name2){
    				return 1;
    			}
    		}
    		return -1;
    	} else if(_val1 > _val2){
    		if(_val3 == _val4){
    			if(_name1[0] != _name2[0] && _name1 < _name2){
    				return -1;
    			}
    		}
    		return 1;
    	} else {
    		return compareName(_name1, _name2);
    	}
    }
}

//比较函数（用于联系人数组排序-姓名首字母为中文或字母）
module.exports.compare = function(_objOne, _objTwo){
	return compare(_objOne, _objTwo);
};

/**
 * 比较函数（用于联系人数组排序-姓名首字母为其他）
 * @param _objOne
 * @param _objTwo
 * @returns
 */
function compareContact(_objOne, _objTwo) {
    var _val = _objOne.name;
    if(null == _val || "" == _val){
    	_val = _objOne.goodsname;
	}
    var _value = _objTwo.name;
    if(null == _value || "" == _value){
    	_value = _objTwo.goodsname;
	}
    if (_val < _value) {
        return -1;
    } else if (_val > _value) {
        return 1;
    } else {
        return 0;
    }            
}

//比较函数（用于联系人数组排序-姓名首字母不是字母）
module.exports.compareContact = function(_objOne, _objTwo){
	return compareContact(_objOne, _objTwo);
};

function findNum(_num, _arrayNum){
	var ret = false;
	for (var i = 0; i < _arrayNum.length; i++) {
		if(_arrayNum[i] == _num){
			ret = true;
			break;
		}
	}
	return ret;
}

function getSerialNum(_index, _arrayNum, _maxnum){
	var _serialIndex = _index;
	if(_index == _maxnum){
		return _index;
	}
	for (var i = _index+1; i <= _maxnum; i++) {
		if(findNum(i, _arrayNum)){
			_serialIndex++;
		}else{
			break;
		}
	}
	return _serialIndex;
}

/**
 * 获取一串数字的简单显示
 * 如周数、课节，1,2,3可显示为1-3
 * @param _numstr
 * @param _maxnum
 * @returns
 */
function getNumSimpleShow(_numstr, _maxnum){
	if(isNullOrEmpty(_numstr) || isNull(_maxnum)){
		return "";
	}
	var _ret = [];
	var array = _numstr.split(",");
	if(array.length == 1){
		return _numstr;
	}
	var _serialnum = 0;
	for (var i = 0; i < array.length; i++) {
		var _num = parseInt(array[i]);
		if(_num <= _serialnum){
			// 数字已记录
			continue;
		}
		_serialnum = getSerialNum(_num, array, _maxnum);
		if(_num == _serialnum){
			_ret.push(_num);
		}else{
			_ret.push(_num + "-" + _serialnum);
		}
	}
	return _ret.join(',');
}

// 获取一串数字的简单显示（用于课程表里多个周数、课节的显示）
module.exports.getNumSimpleShow = function(_numstr, _maxnum){
	return getNumSimpleShow(_numstr, _maxnum);
};

/**
 * 获取一些周数合适的显示字段
 * @param strWeeks 选择的周数
 * @param maxweeknum 最大周数
 * @returns
 */
function getWeeksShowText(strWeeks, maxweeknum){
	var _ret = "";
	if(isNullOrEmpty(strWeeks) || isNull(maxweeknum)){
		return _ret;
	}
	var array = strWeeks.split(",");
	if(array.length == maxweeknum){
		_ret = "第1-"+maxweeknum+"周（每周）";
	}else{
		// 是否全部为单周
		var _oddAll = true;
		// 是否全部为双周
		var _evenAll = true;
		var _weekType = maxweeknum % 2;
		if(_weekType == 0){
			// 偶数周
			var _oddAndEvenNum = parseInt(maxweeknum / 2);
			if(array.length != _oddAndEvenNum){
				_oddAll = false;
				_evenAll = false;
			}
		}else{
			// 单数周
			var _oddAndEvenNum = parseInt(maxweeknum / 2);
			if(array.length == _oddAndEvenNum+1){
				_evenAll = false;
			}else if(array.length == _oddAndEvenNum){
				_oddAll = false;
			}else{
				_oddAll = false;
				_evenAll = false;
			}
		}
		
		if(_oddAll || _evenAll){
			for (var i = 0; i < array.length; i++) {
				if(parseInt(array[i]) % 2 == 0){
					_oddAll = false;
				}else{
					_evenAll = false;
				}
			}
		}
		
		if(_oddAll){
			_ret = "第1-"+maxweeknum+"周（单周）";
		}else if(_evenAll){
			_ret = "第1-"+maxweeknum+"周（双周）";
		}else{
			_ret = getNumSimpleShow(strWeeks, maxweeknum);
		}
	}
	return _ret;
}

// 获取一些周数合适的显示字段
module.exports.getWeeksShowText = function(strWeeks, maxweeknum){
	return getWeeksShowText(strWeeks, maxweeknum);
};

/**
 * 获取节次的显示文本
 * @param _section
 * @returns
 */
function getSectionShowText(_section){
	var _sectionText = "";
	if(!isNullOrEmpty(_section)){
		var _sectionList = _section.split(",");
		if(null != _sectionList && 
				_sectionList.length > 0){
			var _maxSectionNum = parseInt(_sectionList[_sectionList.length - 1]);
			_sectionText = getNumSimpleShow(_section, _maxSectionNum);
		} else {
			_sectionText = _section;
		}
	}
	var _sectionShowText = "";
	if("" != _sectionText){
		_sectionShowText = "第" + _sectionText + "节";
	}
	return _sectionShowText;
}

//获取节次的显示文本
module.exports.getSectionShowText = function(_section){
	return getSectionShowText(_section);
};

/**
 * 获取当前周数显示文本
 * @param _schoolStartTime
 * @param _lessonTime
 * @returns
 */
function getCurWeekNumShowText(_schoolStartTime, _lessonTime){
	var _date = new Date();
	_date.setTime(_schoolStartTime);
	var _nowYear = _date.getFullYear();
	var _nowMonth = _date.getMonth();
	var _nowDay = _date.getDate();
	var _weekDay = _date.getDay();
	if(0 == _weekDay){
		_weekDay = 7;
	}
	_date.setFullYear(_nowYear, _nowMonth, _nowDay - _weekDay);
	_date.setHours(0, 0, 0, 0);
	var _dayLengthTime = 24 * 3600 * 1000;
	var _weekStartTime = _date.getTime() + _dayLengthTime;
	
	var _curWeekNum = Math.floor((parseInt(_lessonTime) - _weekStartTime) / 
		(7 * _dayLengthTime)) + 1;
	var _curWeekNumText = "（第" + _curWeekNum + "周）";
	return _curWeekNumText;
}

//获取当前周数显示文本
module.exports.getCurWeekNumShowText = function(_schoolStartTime, _lessonTime){
	return getCurWeekNumShowText(_schoolStartTime, _lessonTime);
};

/**
 * 比较函数（用于课程卡片排序，节次先后顺序）
 * @param _lesson1
 * @param _lesson2
 * @returns
 */
function compareLesson(_lesson1, _lesson2) {
    var _section1 = _lesson1.section;
    if(null == _section1){
    	_section1 = [0, 0];
	}
    var _section2 = _lesson2.section;
    if(null == _section2){
    	_section2 = [0, 0];
	}
    if (_section1[0] < _section2[0]) {
        return -1;
    } else if (_section1[0] > _section2[0]) {
        return 1;
    } else {
    	if (_section1[1] < _section2[1]) {
            return -1;
        } else if (_section1[1] > _section2[1]) {
            return 1;
        } else {
            return 0;
        }
    }
}

//比较函数（用于课程卡片排序，节次先后顺序）
module.exports.compareLesson = function(_lesson1, _lesson2){
	return compareLesson(_lesson1, _lesson2);
};